@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,11 +7,14 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: add-dialog.c 8671 2009-06-11 16:17:48Z charles $
+ * $Id: add-dialog.c 11460 2010-12-05 18:23:12Z charles $
  */
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+
+#include <string.h>
+
 #include "add-dialog.h"
 #include "conf.h"
 #include "file-list.h"
@@ -27,15 +30,15 @@
 static GSList*
 get_recent_destinations( void )
 {
-    int      i;
+    int i;
     GSList * list = NULL;
 
-    for( i = 0; i < N_RECENT; ++i )
+    for( i=0; i<N_RECENT; ++i )
     {
-        char         key[64];
+        char key[64];
         const char * val;
-        g_snprintf( key, sizeof( key ), "recent-download-dir-%d", i + 1 );
-        if( ( val = pref_string_get( key ) ) )
+        g_snprintf( key, sizeof( key ), "recent-download-dir-%d", i+1 );
+        if(( val = pref_string_get( key )))
             list = g_slist_append( list, (void*)val );
     }
     return list;
@@ -44,15 +47,15 @@ get_recent_destinations( void )
 static void
 save_recent_destination( TrCore * core, const char * dir  )
 {
-    int      i;
-    GSList * list = get_recent_destinations( );
+    int i;
     GSList * l;
+    GSList * list = get_recent_destinations( );
 
-    if( !dir )
+    if( dir == NULL )
         return;
 
     /* if it was already in the list, remove it */
-    if( ( l = g_slist_find_custom( list, dir, (GCompareFunc)strcmp ) ) )
+    if(( l = g_slist_find_custom( list, dir, (GCompareFunc)strcmp )))
         list = g_slist_delete_link( list, l );
 
     /* add it to the front of the list */
@@ -60,12 +63,11 @@ save_recent_destination( TrCore * core, const char * dir  )
 
     /* make local copies of the strings that aren't
      * invalidated by pref_string_set() */
-    for( l = list; l; l = l->next )
+    for( l=list; l; l=l->next )
         l->data = g_strdup( l->data );
 
     /* save the first N_RECENT directories */
-    for( l = list, i = 0; l && ( i < N_RECENT ); ++i, l = l->next )
-    {
+    for( l=list, i=0; l && ( i<N_RECENT ); ++i, l=l->next ) {
         char key[64];
         g_snprintf( key, sizeof( key ), "recent-download-dir-%d", i + 1 );
         pref_string_set( key, l->data );
@@ -84,9 +86,10 @@ save_recent_destination( TrCore * core, const char * dir  )
 struct AddData
 {
     TrCore *     core;
-    GtkWidget *  list;
+    GtkWidget *  file_list;
     GtkWidget *  run_check;
     GtkWidget *  trash_check;
+    GtkWidget *  priority_combo;
     char *       filename;
     char *       downloadDir;
     TrTorrent *  gtor;
@@ -98,7 +101,7 @@ removeOldTorrent( struct AddData * data )
 {
     if( data->gtor )
     {
-        file_list_clear( data->list );
+        file_list_clear( data->file_list );
         tr_torrent_set_remove_flag( data->gtor, TRUE );
         g_object_unref( G_OBJECT( data->gtor ) );
         data->gtor = NULL;
@@ -120,14 +123,18 @@ addResponseCB( GtkDialog * dialog,
         }
         else
         {
+            tr_torrent * tor = tr_torrent_handle( data->gtor );
+
+            tr_torrentSetPriority( tor, gtr_priority_combo_get_value( data->priority_combo ) );
+
             if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data->run_check ) ) )
-                tr_torrentStart( tr_torrent_handle( data->gtor ) );
+                tr_torrentStart( tor );
 
             tr_core_add_torrent( data->core, data->gtor, FALSE );
 
             if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( data->trash_check ) ) )
+                gtr_file_trash_or_remove( data->filename );
 
-                tr_file_trash_or_remove( data->filename );
             save_recent_destination( data->core, data->downloadDir );
         }
     }
@@ -142,12 +149,21 @@ addResponseCB( GtkDialog * dialog,
 static void
 updateTorrent( struct AddData * o )
 {
+    const gboolean isLocalFile = tr_ctorGetSourceFile( o->ctor ) != NULL;
+    gtk_widget_set_sensitive( o->trash_check, isLocalFile );
+
     if( !o->gtor )
-        file_list_clear( o->list );
-    else {
+    {
+        file_list_clear( o->file_list );
+        gtk_widget_set_sensitive( o->file_list, FALSE );
+    }
+    else
+    {
         tr_torrent * tor = tr_torrent_handle( o->gtor );
         tr_torrentSetDownloadDir( tor, o->downloadDir );
-        file_list_set_torrent( o->list, tr_torrentId( tor ) );
+        gtk_widget_set_sensitive( o->file_list, tr_torrentHasMetadata( tor ) );
+        file_list_set_torrent( o->file_list, tr_torrentId( tor ) );
+        tr_torrentVerify( tor );
     }
 }
 
@@ -159,22 +175,19 @@ updateTorrent( struct AddData * o )
  * metadata when that happens.
  */
 static void
-sourceChanged( GtkFileChooserButton * b,
-               gpointer               gdata )
+sourceChanged( GtkFileChooserButton * b, gpointer gdata )
 {
     struct AddData * data = gdata;
-    char *           filename = gtk_file_chooser_get_filename(
-         GTK_FILE_CHOOSER( b ) );
+    char * filename = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( b ) );
 
     /* maybe instantiate a torrent */
-    if( data->filename || !data->gtor )
+    if( filename || !data->gtor )
     {
-        int          err = 0;
-        int          new_file = 0;
+        int err = 0;
+        int new_file = 0;
         tr_torrent * torrent;
 
-        if( filename
-          && ( !data->filename || strcmp( filename, data->filename ) ) )
+        if( filename && ( !data->filename || strcmp( filename, data->filename ) ) )
         {
             g_free( data->filename );
             data->filename = g_strdup( filename );
@@ -203,30 +216,16 @@ sourceChanged( GtkFileChooserButton * b,
 }
 
 static void
-verifyRequested( GtkButton * button UNUSED,
-                 gpointer           gdata )
+downloadDirChanged( GtkFileChooserButton * b, gpointer gdata )
 {
     struct AddData * data = gdata;
-
-    if( data->gtor )
-        tr_torrentVerify( tr_torrent_handle( data->gtor ) );
-}
-
-static void
-downloadDirChanged( GtkFileChooserButton * b,
-                    gpointer               gdata )
-{
-    char *           fname = gtk_file_chooser_get_filename(
-         GTK_FILE_CHOOSER( b ) );
-    struct AddData * data = gdata;
+    char * fname = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( b ) );
 
     if( fname && ( !data->downloadDir || strcmp( fname, data->downloadDir ) ) )
     {
         g_free( data->downloadDir );
         data->downloadDir = g_strdup( fname );
-
         updateTorrent( data );
-        verifyRequested( NULL, data );
     }
 
     g_free( fname );
@@ -253,9 +252,7 @@ addTorrentFilters( GtkFileChooser * chooser )
 ****/
 
 GtkWidget*
-addSingleTorrentDialog( GtkWindow * parent,
-                        TrCore *    core,
-                        tr_ctor *   ctor )
+addSingleTorrentDialog( GtkWindow * parent, TrCore * core, tr_ctor * ctor )
 {
     int              row;
     int              col;
@@ -264,6 +261,7 @@ addSingleTorrentDialog( GtkWindow * parent,
     GtkWidget *      d;
     GtkWidget *      t;
     GtkWidget *      l;
+    GtkWidget *      grab;
     GtkWidget *      source_chooser;
     struct AddData * data;
     uint8_t          flag;
@@ -271,13 +269,12 @@ addSingleTorrentDialog( GtkWindow * parent,
     GSList *         walk;
 
     /* make the dialog */
-    d = gtk_dialog_new_with_buttons( _(
-                                         "Torrent Options" ), parent,
+    d = gtk_dialog_new_with_buttons( _( "Torrent Options" ), parent,
                                      GTK_DIALOG_DESTROY_WITH_PARENT |
                                      GTK_DIALOG_NO_SEPARATOR,
-                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                     GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
                                      NULL );
+    gtk_dialog_add_button( GTK_DIALOG( d ), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL );
+    grab = gtk_dialog_add_button( GTK_DIALOG( d ), GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT );
     gtk_dialog_set_default_response( GTK_DIALOG( d ),
                                      GTK_RESPONSE_ACCEPT );
     gtk_dialog_set_alternative_button_order( GTK_DIALOG( d ),
@@ -294,11 +291,14 @@ addSingleTorrentDialog( GtkWindow * parent,
     data->ctor = ctor;
     data->filename = g_strdup( tr_ctorGetSourceFile( ctor ) );
     data->downloadDir = g_strdup( str );
-    data->list = file_list_new( core, 0 );
-    data->trash_check =
-        gtk_check_button_new_with_mnemonic( _( "_Move source file to Trash" ) );
-    data->run_check =
-        gtk_check_button_new_with_mnemonic( _( "_Start when added" ) );
+    data->file_list = file_list_new( core, 0 );
+    str = _( "Mo_ve .torrent file to the trash" );
+    data->trash_check = gtk_check_button_new_with_mnemonic( str );
+    str = _( "_Start when added" );
+    data->run_check = gtk_check_button_new_with_mnemonic( str );
+
+    data->priority_combo = gtr_priority_combo_new( );
+    gtr_priority_combo_set_value( data->priority_combo, TR_PRI_NORMAL );
 
     g_signal_connect( G_OBJECT( d ), "response",
                       G_CALLBACK( addResponseCB ), data );
@@ -311,15 +311,12 @@ addSingleTorrentDialog( GtkWindow * parent,
     row = col = 0;
     l = gtk_label_new_with_mnemonic( _( "_Torrent file:" ) );
     gtk_misc_set_alignment( GTK_MISC( l ), 0.0f, 0.5f );
-    gtk_table_attach( GTK_TABLE(
-                          t ), l, col, col + 1, row, row + 1, GTK_FILL, 0,
-                      0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), l, col, col + 1, row, row + 1, GTK_FILL, 0, 0, 0 );
     ++col;
     w = gtk_file_chooser_button_new( _( "Select Source File" ),
                                      GTK_FILE_CHOOSER_ACTION_OPEN );
     source_chooser = w;
-    gtk_table_attach( GTK_TABLE(
-                          t ), w, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), w, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
     gtk_label_set_mnemonic_widget( GTK_LABEL( l ), w );
     addTorrentFilters( GTK_FILE_CHOOSER( w ) );
     g_signal_connect( w, "selection-changed",
@@ -329,41 +326,36 @@ addSingleTorrentDialog( GtkWindow * parent,
     col = 0;
     l = gtk_label_new_with_mnemonic( _( "_Destination folder:" ) );
     gtk_misc_set_alignment( GTK_MISC( l ), 0.0f, 0.5f );
-    gtk_table_attach( GTK_TABLE(
-                          t ), l, col, col + 1, row, row + 1, GTK_FILL, 0,
-                      0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), l, col, col + 1, row, row + 1, GTK_FILL, 0, 0, 0 );
     ++col;
-    w = gtk_file_chooser_button_new( _(
-                                         "Select Destination Folder" ),
+    w = gtk_file_chooser_button_new( _( "Select Destination Folder" ),
                                      GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER );
-    if( !gtk_file_chooser_set_filename( GTK_FILE_CHOOSER( w ),
-                                        data->downloadDir ) )
+    if( !gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( w ),
+                                              data->downloadDir ) )
         g_warning( "couldn't select '%s'", data->downloadDir );
     list = get_recent_destinations( );
     for( walk = list; walk; walk = walk->next )
-        gtk_file_chooser_add_shortcut_folder( GTK_FILE_CHOOSER(
-                                                  w ), walk->data, NULL );
+        gtk_file_chooser_add_shortcut_folder( GTK_FILE_CHOOSER( w ), walk->data, NULL );
     g_slist_free( list );
-    gtk_table_attach( GTK_TABLE(
-                          t ), w, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), w, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
     gtk_label_set_mnemonic_widget( GTK_LABEL( l ), w );
     g_signal_connect( w, "selection-changed",
                       G_CALLBACK( downloadDirChanged ), data );
 
     ++row;
     col = 0;
-    w = data->list;
+    w = data->file_list;
     gtk_widget_set_size_request ( w, 466u, 300u );
-    gtk_table_attach_defaults( GTK_TABLE(
-                                   t ), w, col, col + 2, row, row + 1 );
+    gtk_table_attach_defaults( GTK_TABLE( t ), w, col, col + 2, row, row + 1 );
 
     ++row;
     col = 0;
-    w = gtk_button_new_with_mnemonic( _( "_Verify Local Data" ) );
-    gtk_table_attach( GTK_TABLE(
-                          t ), w, col, col + 1, row, row + 1, GTK_FILL, 0,
-                      0, 0 );
-    g_signal_connect( w, "clicked", G_CALLBACK( verifyRequested ), data );
+    w = gtk_label_new_with_mnemonic( _( "Torrent _priority:" ) );
+    gtk_misc_set_alignment( GTK_MISC( w ), 0.0f, 0.5f );
+    gtk_table_attach( GTK_TABLE( t ), w, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
+    ++col;
+    gtk_table_attach( GTK_TABLE( t ), data->priority_combo, col, col + 1, row, row + 1, ~0, 0, 0, 0 );
+    gtk_label_set_mnemonic_widget( GTK_LABEL( w ), data->priority_combo );
 
     ++row;
     col = 0;
@@ -371,9 +363,7 @@ addSingleTorrentDialog( GtkWindow * parent,
     if( tr_ctorGetPaused( ctor, TR_FORCE, &flag ) )
         g_assert_not_reached( );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), !flag );
-    gtk_table_attach( GTK_TABLE(
-                          t ), w, col, col + 2, row, row + 1, GTK_FILL, 0,
-                      0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), w, col, col + 2, row, row + 1, GTK_FILL, 0, 0, 0 );
 
     ++row;
     col = 0;
@@ -381,9 +371,11 @@ addSingleTorrentDialog( GtkWindow * parent,
     if( tr_ctorGetDeleteSource( ctor, &flag ) )
         g_assert_not_reached( );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( w ), flag );
-    gtk_table_attach( GTK_TABLE(
-                          t ), w, col, col + 2, row, row + 1, GTK_FILL, 0,
-                      0, 0 );
+    gtk_table_attach( GTK_TABLE( t ), w, col, col + 2, row, row + 1, GTK_FILL, 0, 0, 0 );
+
+    ++row;
+    col = 0;
+    w = data->priority_combo;
 
     /* trigger sourceChanged, either directly or indirectly,
      * so that it creates the tor/gtor objects */
@@ -394,6 +386,8 @@ addSingleTorrentDialog( GtkWindow * parent,
         sourceChanged( GTK_FILE_CHOOSER_BUTTON( w ), data );
 
     gtk_box_pack_start( GTK_BOX( GTK_DIALOG( d )->vbox ), t, TRUE, TRUE, 0 );
+
+    gtk_widget_grab_focus( grab );
     gtk_widget_show_all( d );
     return d;
 }
@@ -457,7 +451,7 @@ addDialog( GtkWindow * parent,
     if( ( folder = pref_string_get( PREF_KEY_OPEN_DIALOG_FOLDER ) ) )
         gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( w ), folder );
 
-    c = gtk_check_button_new_with_mnemonic( _( "Display _options dialog" ) );
+    c = gtk_check_button_new_with_mnemonic( _( "Show _options dialog" ) );
     gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( c ),
                                  pref_flag_get( PREF_KEY_OPTIONS_PROMPT ) );
     gtk_file_chooser_set_extra_widget( GTK_FILE_CHOOSER( w ), c );
@@ -467,3 +461,96 @@ addDialog( GtkWindow * parent,
     return w;
 }
 
+/***
+****
+***/
+
+static void
+onAddURLResponse( GtkDialog * dialog, int response, gpointer user_data )
+{
+    gboolean destroy = TRUE;
+
+    if( response == GTK_RESPONSE_ACCEPT )
+    {
+        GtkWidget * e = GTK_WIDGET( g_object_get_data( G_OBJECT( dialog ), "url-entry" ) );
+        char * url = g_strdup( gtk_entry_get_text( GTK_ENTRY( e ) ) );
+        g_strstrip( url );
+
+        if( url && *url )
+        {
+            TrCore * core = user_data;
+
+            if( gtr_is_supported_url( url ) || gtr_is_magnet_link( url )
+                                            || gtr_is_hex_hashcode( url ) )
+            {
+                tr_core_add_from_url( core, url );
+            }
+            else
+            {
+                gtr_unrecognized_url_dialog( GTK_WIDGET( dialog ), url );
+                destroy = FALSE;
+            }
+        }
+
+        g_free( url );
+    }
+
+    if( destroy )
+        gtk_widget_destroy( GTK_WIDGET( dialog ) );
+}
+
+static void
+paste_clipboard_url_into_entry( GtkWidget * e )
+{
+  size_t i;
+
+  char * text[] = {
+    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_PRIMARY ) ),
+    gtk_clipboard_wait_for_text( gtk_clipboard_get( GDK_SELECTION_CLIPBOARD ) )
+  };
+
+  for( i=0; i<G_N_ELEMENTS(text); ++i ) {
+      char * s = text[i];
+      if( s && ( gtr_is_supported_url( s ) || gtr_is_magnet_link( s ) ) ) {
+          gtk_entry_set_text( GTK_ENTRY( e ), s );
+          break;
+      }
+  }
+
+  for( i=0; i<G_N_ELEMENTS(text); ++i )
+    g_free( text[i] );
+}
+
+GtkWidget*
+addURLDialog( GtkWindow * parent, TrCore * core )
+{
+    int row;
+    GtkWidget * e;
+    GtkWidget * t;
+    GtkWidget * w;
+
+    w = gtk_dialog_new_with_buttons( _( "Add URL" ), parent,
+                                     GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
+                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                     GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
+                                     NULL );
+    gtk_dialog_set_alternative_button_order( GTK_DIALOG( w ),
+                                             GTK_RESPONSE_ACCEPT,
+                                             GTK_RESPONSE_CANCEL,
+                                             -1 );
+    g_signal_connect( w, "response", G_CALLBACK( onAddURLResponse ), core );
+
+    row = 0;
+    t = hig_workarea_create( );
+    hig_workarea_add_section_title( t, &row, _( "Add torrent from URL" ) );
+    e = gtk_entry_new( );
+    gtk_widget_set_size_request( e, 400, -1 );
+    paste_clipboard_url_into_entry( e );
+    g_object_set_data( G_OBJECT( w ), "url-entry", e );
+    hig_workarea_add_row( t, &row, _( "_URL" ), e, NULL );
+
+    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( w )->vbox ), t, TRUE, TRUE, 0 );
+    gtk_widget_show_all( t );
+    gtk_widget_show( w );
+    return w;
+}
