@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: conf.c 8929 2009-08-14 12:53:08Z charles $
+ * $Id: conf.c 10391 2010-03-17 19:23:03Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -22,7 +22,6 @@
  * DEALINGS IN THE SOFTWARE.
  *****************************************************************************/
 
-#include <assert.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -36,7 +35,6 @@
 
 #include <libtransmission/transmission.h>
 #include <libtransmission/bencode.h>
-#include <libtransmission/platform.h>
 
 #include "conf.h"
 #include "tr-prefs.h"
@@ -109,7 +107,7 @@ lockfile( const char             * filename,
 static char*
 getLockFilename( void )
 {
-    assert( gl_confdir != NULL );
+    g_assert( gl_confdir != NULL );
     return g_build_filename( gl_confdir, "lock", NULL );
 }
 
@@ -165,6 +163,7 @@ tr_prefs_init_defaults( tr_benc * d )
     tr_bencDictAddBool( d, PREF_KEY_DIR_WATCH_ENABLED, FALSE );
 #endif
 
+    tr_bencDictAddBool( d, PREF_KEY_USER_HAS_GIVEN_INFORMED_CONSENT, FALSE );
     tr_bencDictAddBool( d, PREF_KEY_INHIBIT_HIBERNATION, FALSE );
     tr_bencDictAddBool( d, PREF_KEY_BLOCKLIST_UPDATES_ENABLED, TRUE );
 
@@ -174,7 +173,11 @@ tr_prefs_init_defaults( tr_benc * d )
     tr_bencDictAddBool( d, PREF_KEY_FILTERBAR, TRUE );
     tr_bencDictAddBool( d, PREF_KEY_STATUSBAR, TRUE );
     tr_bencDictAddBool( d, PREF_KEY_SHOW_TRAY_ICON, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_PLAY_DOWNLOAD_COMPLETE_SOUND, TRUE );
     tr_bencDictAddBool( d, PREF_KEY_SHOW_DESKTOP_NOTIFICATION, TRUE );
+    tr_bencDictAddBool( d, PREF_KEY_SHOW_MORE_TRACKER_INFO, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_SHOW_MORE_PEER_INFO, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_SHOW_BACKUP_TRACKERS, FALSE );
     tr_bencDictAddStr( d, PREF_KEY_STATUSBAR_STATS, "total-ratio" );
 
     tr_bencDictAddBool( d, PREF_KEY_OPTIONS_PROMPT, TRUE );
@@ -198,16 +201,13 @@ tr_prefs_init_defaults( tr_benc * d )
     tr_bencDictAddStr( d, PREF_KEY_FILTER_MODE, "show-all" );
     tr_bencDictAddStr( d, PREF_KEY_SORT_MODE, "sort-by-name" );
     tr_bencDictAddBool( d, PREF_KEY_SORT_REVERSED, FALSE );
-    tr_bencDictAddBool( d, PREF_KEY_MINIMAL_VIEW, FALSE );
-
-    tr_bencDictAddBool( d, PREF_KEY_START, TRUE );
-    tr_bencDictAddBool( d, PREF_KEY_TRASH_ORIGINAL, FALSE );
+    tr_bencDictAddBool( d, PREF_KEY_COMPACT_VIEW, FALSE );
 }
 
 static char*
 getPrefsFilename( void )
 {
-    assert( gl_confdir != NULL );
+    g_assert( gl_confdir != NULL );
     return g_build_filename( gl_confdir, "settings.json", NULL );
 }
 
@@ -359,7 +359,7 @@ tr_file_set_contents( const char *   filename,
 static char*
 getCompat090PrefsFilename( void )
 {
-    assert( gl_confdir != NULL );
+    g_assert( gl_confdir != NULL );
 
     return g_build_filename( g_get_home_dir( ), ".transmission", "gtk", "prefs.ini", NULL );
 }
