@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: file-list.c 9364 2009-10-22 02:45:14Z charles $
+ * $Id: file-list.c 11461 2010-12-05 18:25:57Z charles $
  */
 
 #include <stddef.h>
@@ -22,6 +22,7 @@
 #include "file-list.h"
 #include "hig.h"
 #include "icons.h"
+#include "tr-prefs.h"
 
 enum
 {
@@ -41,19 +42,16 @@ enum
     FC_HAVE,
     FC_PRIORITY,
     FC_ENABLED,
-    FC_IS_FILE,
     N_FILE_COLS
 };
 
 typedef struct
 {
     TrCore        * core;
-    tr_torrent    * tor;
     GtkWidget     * top;
     GtkWidget     * view;
     GtkTreeModel  * model; /* same object as store, but recast */
     GtkTreeStore  * store; /* same object as model, but recast */
-    tr_file_stat  * refresh_file_stat;
     int             torrentId;
     guint           timeout_tag;
 }
@@ -81,23 +79,34 @@ freeData( gpointer data )
 ****
 ***/
 
+struct RefreshData
+{
+    int sort_column_id;
+    gboolean resort_needed;
+
+    tr_file_stat  * refresh_file_stat;
+    tr_torrent * tor;
+
+    FileData * file_data;
+};
+
 static gboolean
 refreshFilesForeach( GtkTreeModel * model,
                      GtkTreePath  * path UNUSED,
                      GtkTreeIter  * iter,
                      gpointer       gdata )
 {
-    FileData * data = gdata;
-    gboolean is_file;
+    struct RefreshData * refresh_data = gdata;
+    FileData * data = refresh_data->file_data;
     unsigned int index;
     uint64_t size;
     uint64_t old_have;
     int old_prog;
     int old_priority;
     int old_enabled;
+    const gboolean is_file = !gtk_tree_model_iter_has_child( model, iter );
 
-    gtk_tree_model_get( model, iter, FC_IS_FILE, &is_file,
-                                     FC_ENABLED, &old_enabled,
+    gtk_tree_model_get( model, iter, FC_ENABLED, &old_enabled,
                                      FC_PRIORITY, &old_priority,
                                      FC_INDEX, &index,
                                      FC_HAVE, &old_have,
@@ -107,18 +116,38 @@ refreshFilesForeach( GtkTreeModel * model,
 
     if( is_file )
     {
-        tr_torrent * tor = data->tor;
-        const int enabled = tr_torrentGetFileDL( tor, index );
-        const int priority = tr_torrentGetFilePriority( tor, index );
-        const uint64_t have = data->refresh_file_stat[index].bytesCompleted;
-        const int prog = (int)((100.0*have)/size);
+        tr_torrent * tor = refresh_data->tor;
+        const tr_info * inf = tr_torrentInfo( tor );
+        const int enabled = !inf->files[index].dnd;
+        const int priority = inf->files[index].priority;
+        const uint64_t have = refresh_data->refresh_file_stat[index].bytesCompleted;
+        const int prog = size ? (int)((100.0*have)/size) : 1;
 
         if( (priority!=old_priority) || (enabled!=old_enabled) || (have!=old_have) || (prog!=old_prog) )
+        {
+            /* Changing a value in the sort column can trigger a resort
+             * which breaks this foreach() call.  (See #3529)
+             * As a workaround: if that's about to happen, temporarily disable
+             * sorting until we finish walking the tree. */
+            if( !refresh_data->resort_needed )
+            {
+                if(( refresh_data->resort_needed =
+                    (( refresh_data->sort_column_id==FC_PRIORITY ) && ( priority!=old_priority )) ||
+                    (( refresh_data->sort_column_id==FC_ENABLED ) && ( enabled!=old_enabled ))))
+                {
+                    refresh_data->resort_needed = TRUE;
+                    gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( data->model ),
+                                                          GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID,
+                                                          GTK_SORT_ASCENDING );
+                }
+            }
+
             gtk_tree_store_set( data->store, iter, FC_PRIORITY, priority,
                                                    FC_ENABLED, enabled,
                                                    FC_HAVE, have,
                                                    FC_PROG, prog,
                                                    -1 );
+        }
     }
     else
     {
@@ -159,7 +188,7 @@ refreshFilesForeach( GtkTreeModel * model,
         }
         while( gtk_tree_model_iter_next( model, &child ) );
 
-        prog = (int)((100.0*have)/sub_size);
+        prog = sub_size ? (int)((100.0*have)/sub_size) : 1;
 
         if( (size!=sub_size) || (have!=old_have)
                              || (priority!=old_priority)
@@ -216,15 +245,25 @@ refresh( FileData * data )
     }
     else
     {
+        GtkSortType order;
+        int sort_column_id;
         tr_file_index_t fileCount;
-        data->tor = tr_torrentFindFromId( session, data->torrentId );
-        data->refresh_file_stat = tr_torrentFiles( tor, &fileCount );
+        struct RefreshData refresh_data;
+        GtkTreeSortable * sortable = GTK_TREE_SORTABLE( data->model );
+        gtk_tree_sortable_get_sort_column_id( sortable, &sort_column_id, &order );
 
-        gtr_tree_model_foreach_postorder( data->model, refreshFilesForeach, data );
+        refresh_data.sort_column_id = sort_column_id;
+        refresh_data.resort_needed = FALSE;
+        refresh_data.refresh_file_stat = tr_torrentFiles( tor, &fileCount );
+        refresh_data.tor = tr_torrentFindFromId( session, data->torrentId );
+        refresh_data.file_data = data;
 
-        tr_torrentFilesFree( data->refresh_file_stat, fileCount );
-        data->refresh_file_stat = NULL;
-        data->tor = NULL;
+        gtr_tree_model_foreach_postorder( data->model, refreshFilesForeach, &refresh_data );
+
+        if( refresh_data.resort_needed )
+            gtk_tree_sortable_set_sort_column_id( sortable, sort_column_id, order );
+
+        tr_torrentFilesFree( refresh_data.refresh_file_stat, fileCount );
     }
 }
 
@@ -251,18 +290,16 @@ getSelectedFilesForeach( GtkTreeModel * model,
                          GtkTreeIter  * iter,
                          gpointer       gdata )
 {
-    struct ActiveData * data = gdata;
-    unsigned int        i;
-    gboolean            is_file = FALSE;
-    gboolean            is_active = FALSE;
+    const gboolean is_file = !gtk_tree_model_iter_has_child( model, iter );
 
-    /* active == if it's selected, or any ancestor is selected */
-    gtk_tree_model_get( model, iter, FC_IS_FILE, &is_file,
-                                     FC_INDEX, &i,
-                                     -1 );
     if( is_file )
     {
-        is_active = gtk_tree_selection_iter_is_selected( data->sel, iter );
+        struct ActiveData * data = gdata;
+    
+        /* active means: if it's selected or any ancestor is selected */
+
+        gboolean is_active = gtk_tree_selection_iter_is_selected( data->sel, iter );
+
         if( !is_active )
         {
             GtkTreeIter walk = *iter;
@@ -273,10 +310,14 @@ getSelectedFilesForeach( GtkTreeModel * model,
                 walk = parent;
             }
         }
-    }
 
-    if( is_active )
-        g_array_append_val( data->array, i );
+        if( is_active )
+        {
+            unsigned int i;
+            gtk_tree_model_get( model, iter, FC_INDEX, &i, -1 );
+            g_array_append_val( data->array, i );
+        }
+    }
 
     return FALSE; /* keep walking */
 }
@@ -304,16 +345,19 @@ getSubtreeForeach( GtkTreeModel   * model,
                    GtkTreeIter    * iter,
                    gpointer         gdata )
 {
-    struct SubtreeForeachData * data = gdata;
-    unsigned int i;
-    gboolean is_file = FALSE;
+    const gboolean is_file = !gtk_tree_model_iter_has_child( model, iter );
 
-    gtk_tree_model_get( model, iter, FC_IS_FILE, &is_file,
-                                     FC_INDEX, &i,
-                                     -1 );
     if( is_file )
+    {
+        struct SubtreeForeachData * data = gdata;
+
         if( !gtk_tree_path_compare( path, data->path ) || gtk_tree_path_is_descendant( path, data->path ) )
+        {
+            unsigned int i;
+            gtk_tree_model_get( model, iter, FC_INDEX, &i, -1 );
             g_array_append_val( data->array, i );
+        }
+    }
 
     return FALSE; /* keep walking */
 }
@@ -386,8 +430,9 @@ buildTree( GNode * node, gpointer gdata )
 
     const char * mime_type = isLeaf ? get_mime_type_from_filename( child_data->name ) : DIRECTORY_MIME_TYPE;
     GdkPixbuf * icon = get_mime_type_icon( mime_type, GTK_ICON_SIZE_MENU, build->w );
-    const int priority = isLeaf ? tr_torrentGetFilePriority( build->tor, child_data->index ) : 0;
-    const gboolean enabled = isLeaf ? tr_torrentGetFileDL( build->tor, child_data->index ) : TRUE;
+    const tr_info * inf = tr_torrentInfo( build->tor );
+    const int priority = isLeaf ? inf->files[ child_data->index ].priority : 0;
+    const gboolean enabled = isLeaf ? !inf->files[ child_data->index ].dnd : TRUE;
 #if GTK_CHECK_VERSION(2,10,0)
     gtk_tree_store_insert_with_values( build->store, &child_iter, build->iter, INT_MAX,
                                        FC_INDEX, child_data->index,
@@ -396,7 +441,6 @@ buildTree( GNode * node, gpointer gdata )
                                        FC_ICON, icon,
                                        FC_PRIORITY, priority,
                                        FC_ENABLED, enabled,
-                                       FC_IS_FILE, isLeaf,
                                        -1 );
 #else
     gtk_tree_store_append( build->store, &child_iter, build->iter );
@@ -407,7 +451,6 @@ buildTree( GNode * node, gpointer gdata )
                         FC_ICON, icon,
                         FC_PRIORITY, priority,
                         FC_ENABLED, enabled,
-                        FC_IS_FILE, isLeaf,
                         -1 );
 #endif
 
@@ -456,11 +499,7 @@ file_list_set_torrent( GtkWidget * w, int torrentId )
                                  G_TYPE_UINT64,    /* size */
                                  G_TYPE_UINT64,    /* have */
                                  G_TYPE_INT,       /* priority */
-                                 G_TYPE_INT,       /* dl enabled */
-                                 G_TYPE_BOOLEAN,   /* is file */
-                                 G_TYPE_UINT64,    /* sub size */
-                                 G_TYPE_UINT64,    /* sub have */
-                                 G_TYPE_INT );     /* sub state */
+                                 G_TYPE_INT );     /* dl enabled */
 
     data->store = store;
     data->model = GTK_TREE_MODEL( store );
@@ -521,7 +560,7 @@ file_list_set_torrent( GtkWidget * w, int torrentId )
         }
 
         refresh( data );
-        data->timeout_tag = gtr_timeout_add_seconds( 2, refreshModel, data );
+        data->timeout_tag = gtr_timeout_add_seconds( SECONDARY_WINDOW_REFRESH_INTERVAL_SECONDS, refreshModel, data );
     }
 
     gtk_tree_view_set_model( GTK_TREE_VIEW( data->view ), data->model );
