@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,16 +7,15 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: makemeta-ui.h 7658 2009-01-10 23:09:07Z charles $
+ * $Id: makemeta-ui.h 9868 2010-01-04 21:00:47Z charles $
  */
 
 #ifndef MAKE_META_UI__H
 #define MAKE_META_UI__H
 
 #include <gtk/gtk.h>
-#include <libtransmission/transmission.h>
+#include "tr-core.h"
 
-GtkWidget* make_meta_ui( GtkWindow  * parent,
-                         tr_session * session );
+GtkWidget* make_meta_ui( GtkWindow  * parent, TrCore * core );
 
 #endif
