@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -14,8 +14,10 @@
 #define GTR_RELOCATE_H
 
 #include <gtk/gtk.h>
-#include <libtransmission/transmission.h>
+#include "tr-core.h"
 
-GtkWidget * gtr_relocate_dialog_new( GtkWindow * parent, tr_torrent * tor );
+GtkWidget * gtr_relocate_dialog_new( GtkWindow * parent,
+                                     TrCore    * core,
+                                     GSList    * torrentIds );
 
 #endif
