@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,10 +7,9 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent-cell-renderer.c 9108 2009-09-12 07:26:14Z charles $
+ * $Id: torrent-cell-renderer.c 11462 2010-12-05 18:27:01Z charles $
  */
 
-#include "assert.h"
 #include <string.h> /* strcmp() */
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -30,12 +29,12 @@ enum
     P_UPLOAD_SPEED,
     P_DOWNLOAD_SPEED,
     P_BAR_HEIGHT,
-    P_MINIMAL
+    P_COMPACT
 };
 
 #define DEFAULT_BAR_HEIGHT 12
 #define SMALL_SCALE 0.9
-#define MINIMAL_ICON_SIZE GTK_ICON_SIZE_MENU
+#define COMPACT_ICON_SIZE GTK_ICON_SIZE_MENU
 #define FULL_ICON_SIZE GTK_ICON_SIZE_DND
 
 /***
@@ -50,12 +49,12 @@ getProgressString( const tr_torrent * tor,
     const int      isDone = torStat->leftUntilDone == 0;
     const uint64_t haveTotal = torStat->haveUnchecked + torStat->haveValid;
     const int      isSeed = torStat->haveValid >= info->totalSize;
-    char           buf1[32], buf2[32], buf3[32], buf4[32];
+    char           buf1[32], buf2[32], buf3[32], buf4[32], buf5[32];
     char *         str;
     double         seedRatio;
-    gboolean       hasSeedRatio = FALSE;
+    const gboolean hasSeedRatio = tr_torrentGetSeedRatio( tor, &seedRatio );
 
-    if( !isDone )
+    if( !isDone ) /* downloading */
     {
         str = g_strdup_printf(
             /* %1$s is how much we've got,
@@ -66,48 +65,72 @@ getProgressString( const tr_torrent * tor,
             tr_strlsize( buf2, torStat->sizeWhenDone, sizeof( buf2 ) ),
             tr_truncd( torStat->percentDone * 100.0, 2 ) );
     }
-    else if( !isSeed )
+    else if( !isSeed ) /* partial seeds */
     {
-        str = g_strdup_printf(
-            /* %1$s is how much we've got,
-               %2$s is the torrent's total size,
-               %3$.2f%% is a percentage of the two,
-               %4$s is how much we've uploaded,
-               %5$s is our upload-to-download ratio */
-            _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s)" ),
-            tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
-            tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
-            tr_truncd( torStat->percentComplete * 100.0, 2 ),
-            tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
-            tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ) );
+        if( hasSeedRatio )
+        {
+            str = g_strdup_printf(
+                /* %1$s is how much we've got,
+                   %2$s is the torrent's total size,
+                   %3$.2f%% is a percentage of the two,
+                   %4$s is how much we've uploaded,
+                   %5$s is our upload-to-download ratio,
+                   %6$s is the ratio we want to reach before we stop uploading */
+                _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s Goal: %6$s)" ),
+                tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
+                tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
+                tr_truncd( torStat->percentComplete * 100.0, 2 ),
+                tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
+                tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ),
+                tr_strlratio( buf5, seedRatio, sizeof( buf5 ) ) );
+        }
+        else
+        {
+            str = g_strdup_printf(
+                /* %1$s is how much we've got,
+                   %2$s is the torrent's total size,
+                   %3$.2f%% is a percentage of the two,
+                   %4$s is how much we've uploaded,
+                   %5$s is our upload-to-download ratio */
+                _( "%1$s of %2$s (%3$.2f%%), uploaded %4$s (Ratio: %5$s)" ),
+                tr_strlsize( buf1, haveTotal, sizeof( buf1 ) ),
+                tr_strlsize( buf2, info->totalSize, sizeof( buf2 ) ),
+                tr_truncd( torStat->percentComplete * 100.0, 2 ),
+                tr_strlsize( buf3, torStat->uploadedEver, sizeof( buf3 ) ),
+                tr_strlratio( buf4, torStat->ratio, sizeof( buf4 ) ) );
+        }
     }
-    else if(( hasSeedRatio = tr_torrentGetSeedRatio( tor, &seedRatio )))
+    else /* seeding */
     {
-        str = g_strdup_printf(
-            /* %1$s is the torrent's total size,
-               %2$s is how much we've uploaded,
-               %3$s is our upload-to-download ratio,
-               $4$s is the ratio we want to reach before we stop uploading */
-            _( "%1$s, uploaded %2$s (Ratio: %3$s Goal: %4$s)" ),
-            tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
-            tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
-            tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ),
-            tr_strlratio( buf4, seedRatio, sizeof( buf4 ) ) );
-    }
-    else /* seeding w/o a ratio */
-    {
-        str = g_strdup_printf(
-            /* %1$s is the torrent's total size,
-               %2$s is how much we've uploaded,
-               %3$s is our upload-to-download ratio */
-            _( "%1$s, uploaded %2$s (Ratio: %3$s)" ),
-            tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
-            tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
-            tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ) );
+        if( hasSeedRatio )
+        {
+            str = g_strdup_printf(
+                /* %1$s is the torrent's total size,
+                   %2$s is how much we've uploaded,
+                   %3$s is our upload-to-download ratio,
+                   %4$s is the ratio we want to reach before we stop uploading */
+                _( "%1$s, uploaded %2$s (Ratio: %3$s Goal: %4$s)" ),
+                tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
+                tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
+                tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ),
+                tr_strlratio( buf4, seedRatio, sizeof( buf4 ) ) );
+        }
+        else /* seeding w/o a ratio */
+        {
+            str = g_strdup_printf(
+                /* %1$s is the torrent's total size,
+                   %2$s is how much we've uploaded,
+                   %3$s is our upload-to-download ratio */
+                _( "%1$s, uploaded %2$s (Ratio: %3$s)" ),
+                tr_strlsize( buf1, info->totalSize, sizeof( buf1 ) ),
+                tr_strlsize( buf2, torStat->uploadedEver, sizeof( buf2 ) ),
+                tr_strlratio( buf3, torStat->ratio, sizeof( buf3 ) ) );
+        }
     }
 
     /* add time when downloading */
-    if( hasSeedRatio || ( torStat->activity == TR_STATUS_DOWNLOAD ) )
+    if( ( torStat->activity == TR_STATUS_DOWNLOAD )
+        || ( hasSeedRatio && ( torStat->activity == TR_STATUS_SEED ) ) )
     {
         const int eta = torStat->eta;
         GString * gstr = g_string_new( str );
@@ -129,15 +152,17 @@ getProgressString( const tr_torrent * tor,
 }
 
 static char*
-getShortTransferString( const tr_stat * torStat,
-                        double          uploadSpeed,
-                        double          downloadSpeed,
-                        char          * buf,
-                        size_t          buflen )
+getShortTransferString( const tr_torrent  * tor,
+                        const tr_stat     * torStat,
+                        double              uploadSpeed,
+                        double              downloadSpeed,
+                        char              * buf,
+                        size_t              buflen )
 {
-    char      downStr[32], upStr[32];
-    const int haveDown = torStat->peersSendingToUs > 0;
-    const int haveUp = torStat->peersGettingFromUs > 0;
+    char downStr[32], upStr[32];
+    const int haveMeta = tr_torrentHasMetadata( tor );
+    const int haveDown = haveMeta && torStat->peersSendingToUs > 0;
+    const int haveUp = haveMeta && torStat->peersGettingFromUs > 0;
 
     if( haveDown )
         tr_strlspeed( downStr, downloadSpeed, sizeof( downStr ) );
@@ -145,35 +170,42 @@ getShortTransferString( const tr_stat * torStat,
         tr_strlspeed( upStr, uploadSpeed, sizeof( upStr ) );
 
     if( haveDown && haveUp )
-        /* Translators: "speed|" is here for disambiguation.  Please remove it from your translation.
-           %1$s is the download speed
-           %2$s is the upload speed */
-        g_snprintf( buf, buflen, Q_(
-                        "speed|Down: %1$s, Up: %2$s" ), downStr, upStr );
+        /* 1==down speed, 2==down arrow, 3==up speed, 4==down arrow */
+        g_snprintf( buf, buflen, _( "%1$s %2$s, %3$s %4$s" ),
+                    gtr_get_unicode_string( GTR_UNICODE_DOWN ), downStr,
+                    gtr_get_unicode_string( GTR_UNICODE_UP ), upStr );
     else if( haveDown )
-        /* download speed */
-        g_snprintf( buf, buflen, _( "Down: %s" ), downStr );
+        /* bandwidth speed + unicode arrow */
+        g_snprintf( buf, buflen, _( "%1$s %2$s" ),
+                    gtr_get_unicode_string( GTR_UNICODE_DOWN ), downStr );
     else if( haveUp )
-        /* upload speed */
-        g_snprintf( buf, buflen, _( "Up: %s" ), upStr );
-    else
+        /* bandwidth speed + unicode arrow */
+        g_snprintf( buf, buflen, _( "%1$s %2$s" ),
+                    gtr_get_unicode_string( GTR_UNICODE_UP ), upStr );
+    else if( tr_torrentHasMetadata( tor ) )
         /* the torrent isn't uploading or downloading */
         g_strlcpy( buf, _( "Idle" ), buflen );
+    else
+        *buf = '\0';
 
     return buf;
 }
 
 static char*
-getShortStatusString( const tr_stat * torStat,
-                      double          uploadSpeed,
-                      double          downloadSpeed )
+getShortStatusString( const tr_torrent  * tor,
+                      const tr_stat     * torStat,
+                      double              uploadSpeed,
+                      double              downloadSpeed )
 {
     GString * gstr = g_string_new( NULL );
 
     switch( torStat->activity )
     {
         case TR_STATUS_STOPPED:
-            g_string_assign( gstr, _( "Paused" ) );
+            if( torStat->finished )
+                g_string_assign( gstr, _( "Finished" ) );
+            else
+                g_string_assign( gstr, _( "Paused" ) );
             break;
 
         case TR_STATUS_CHECK_WAIT:
@@ -193,10 +225,10 @@ getShortStatusString( const tr_stat * torStat,
             if( torStat->activity != TR_STATUS_DOWNLOAD )
             {
                 tr_strlratio( buf, torStat->ratio, sizeof( buf ) );
-                g_string_append_printf( gstr, _( "Ratio: %s" ), buf );
+                g_string_append_printf( gstr, _( "Ratio %s" ), buf );
                 g_string_append( gstr, ", " );
             }
-            getShortTransferString( torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
+            getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
             g_string_append( gstr, buf );
             break;
         }
@@ -209,9 +241,10 @@ getShortStatusString( const tr_stat * torStat,
 }
 
 static char*
-getStatusString( const tr_stat * torStat,
-                 const double    uploadSpeed,
-                 const double    downloadSpeed )
+getStatusString( const tr_torrent  * tor,
+                 const tr_stat     * torStat,
+                 const double        uploadSpeed,
+                 const double        downloadSpeed )
 {
     const int isActive = torStat->activity != TR_STATUS_STOPPED;
     const int isChecking = torStat->activity == TR_STATUS_CHECK
@@ -227,18 +260,21 @@ getStatusString( const tr_stat * torStat,
         g_string_append_printf( gstr, _( fmt[torStat->error] ), torStat->errorString );
     }
     else switch( torStat->activity )
+    {
+        case TR_STATUS_STOPPED:
+        case TR_STATUS_CHECK_WAIT:
+        case TR_STATUS_CHECK:
         {
-            case TR_STATUS_STOPPED:
-            case TR_STATUS_CHECK_WAIT:
-            case TR_STATUS_CHECK:
-            {
-                char * pch = getShortStatusString( torStat, uploadSpeed, downloadSpeed );
-                g_string_assign( gstr, pch );
-                g_free( pch );
-                break;
-            }
+            char * pch = getShortStatusString( tor, torStat, uploadSpeed, downloadSpeed );
+            g_string_assign( gstr, pch );
+            g_free( pch );
+            break;
+        }
 
-            case TR_STATUS_DOWNLOAD:
+        case TR_STATUS_DOWNLOAD:
+        {
+            if( tr_torrentHasMetadata( tor ) )
+            {
                 g_string_append_printf( gstr,
                     ngettext( "Downloading from %1$'d of %2$'d connected peer",
                               "Downloading from %1$'d of %2$'d connected peers",
@@ -247,23 +283,35 @@ getStatusString( const tr_stat * torStat,
                     torStat->webseedsSendingToUs,
                     torStat->peersConnected +
                     torStat->webseedsSendingToUs );
-                break;
-
-            case TR_STATUS_SEED:
+            }
+            else
+            {
                 g_string_append_printf( gstr,
-                    ngettext( "Seeding to %1$'d of %2$'d connected peer",
-                              "Seeding to %1$'d of %2$'d connected peers",
+                    ngettext( "Downloading metadata from %1$'d peer (%2$d%% done)",
+                              "Downloading metadata from %1$'d peers (%2$d%% done)",
                               torStat->peersConnected ),
-                    torStat->peersGettingFromUs,
-                    torStat->peersConnected );
-                break;
+                    torStat->peersConnected + torStat->webseedsSendingToUs,
+                    (int)(100.0*torStat->metadataPercentComplete) );
+            }
+            break;
         }
 
+        case TR_STATUS_SEED:
+            g_string_append_printf( gstr,
+                ngettext( "Seeding to %1$'d of %2$'d connected peer",
+                          "Seeding to %1$'d of %2$'d connected peers",
+                          torStat->peersConnected ),
+                torStat->peersGettingFromUs,
+                torStat->peersConnected );
+                break;
+    }
+
     if( isActive && !isChecking )
     {
         char buf[256];
-        getShortTransferString( torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
-        g_string_append_printf( gstr, " - %s", buf );
+        getShortTransferString( tor, torStat, uploadSpeed, downloadSpeed, buf, sizeof( buf ) );
+        if( *buf )
+            g_string_append_printf( gstr, " - %s", buf );
     }
 
     return g_string_free( gstr, FALSE );
@@ -293,7 +341,7 @@ struct TorrentCellRendererPrivate
     /* @see upload_speed */
     double download_speed;
 
-    gboolean minimal;
+    gboolean compact;
 };
 
 /***
@@ -306,7 +354,11 @@ get_icon( const tr_torrent * tor, GtkIconSize icon_size, GtkWidget * for_widget
     const char * mime_type;
     const tr_info * info = tr_torrentInfo( tor );
 
-    if( info->fileCount > 1 )
+    if( info->fileCount == 0  )
+        mime_type = UNKNOWN_MIME_TYPE;
+    else if( info->fileCount > 1 )
+        mime_type = DIRECTORY_MIME_TYPE;
+    else if( strchr( info->files[0].name, '/' ) != NULL )
         mime_type = DIRECTORY_MIME_TYPE;
     else
         mime_type = get_mime_type_from_filename( info->files[0].name );
@@ -325,7 +377,7 @@ get_text_renderer( const tr_stat * st, TorrentCellRenderer * r )
 ***/
 
 static void
-get_size_minimal( TorrentCellRenderer * cell,
+get_size_compact( TorrentCellRenderer * cell,
                   GtkWidget           * widget,
                   gint                * width,
                   gint                * height )
@@ -343,9 +395,9 @@ get_size_minimal( TorrentCellRenderer * cell,
     const tr_torrent * tor = p->tor;
     const tr_stat * st = tr_torrentStatCached( (tr_torrent*)tor );
 
-    icon = get_icon( tor, MINIMAL_ICON_SIZE, widget );
+    icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( st, p->upload_speed, p->download_speed );
+    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
 
     /* get the idealized cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
@@ -366,10 +418,11 @@ get_size_minimal( TorrentCellRenderer * cell,
     *** LAYOUT
     **/
 
+#define BAR_WIDTH 50
     if( width != NULL )
-        *width = cell->parent.xpad * 2 + icon_area.width + GUI_PAD + name_area.width + GUI_PAD + stat_area.width;
+        *width = cell->parent.xpad * 2 + icon_area.width + GUI_PAD + name_area.width + GUI_PAD + BAR_WIDTH + GUI_PAD + stat_area.width;
     if( height != NULL )
-        *height = cell->parent.ypad * 2 + name_area.height + p->bar_height;
+        *height = cell->parent.ypad * 2 + MAX( name_area.height, p->bar_height );
 
     /* cleanup */
     g_free( status );
@@ -402,7 +455,7 @@ get_size_full( TorrentCellRenderer * cell,
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( st, p->upload_speed, p->download_speed );
+    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
     progress = getProgressString( tor, inf, st );
 
     /* get the idealized cell dimensions */
@@ -456,8 +509,8 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
         struct TorrentCellRendererPrivate * p = self->priv;
         int w, h;
 
-        if( p->minimal )
-            get_size_minimal( TORRENT_CELL_RENDERER( cell ), widget, &w, &h );
+        if( p->compact )
+            get_size_compact( TORRENT_CELL_RENDERER( cell ), widget, &w, &h );
         else
             get_size_full( TORRENT_CELL_RENDERER( cell ), widget, &w, &h );
 
@@ -467,18 +520,16 @@ torrent_cell_renderer_get_size( GtkCellRenderer  * cell,
         if( height )
             *height = h;
 
-        if( cell_area ) {
-            if( x_offset ) *x_offset = 0;
-            if( y_offset ) {
-                *y_offset = 0.5 * ( cell_area->height - ( h + ( 2 * cell->ypad ) ) );
-                *y_offset = MAX( *y_offset, 0 );
-            }
-        }
+        if( x_offset )
+            *x_offset = cell_area ? cell_area->x : 0;
+          
+        if( y_offset )
+            *x_offset = cell_area ? (int)((cell_area->height - (cell->ypad*2 +h)) / 2.0) : 0;
     }
 }
 
 static void
-render_minimal( TorrentCellRenderer   * cell,
+render_compact( TorrentCellRenderer   * cell,
                 GdkDrawable           * window,
                 GtkWidget             * widget,
                 GdkRectangle          * background_area,
@@ -504,9 +555,9 @@ render_minimal( TorrentCellRenderer   * cell,
     const double percentDone = MAX( 0.0, st->percentDone );
     const gboolean sensitive = active || st->error;
 
-    icon = get_icon( tor, MINIMAL_ICON_SIZE, widget );
+    icon = get_icon( tor, COMPACT_ICON_SIZE, widget );
     name = tr_torrentInfo( tor )->name;
-    status = getShortStatusString( st, p->upload_speed, p->download_speed );
+    status = getShortStatusString( tor, st, p->upload_speed, p->download_speed );
 
     /* get the cell dimensions */
     g_object_set( p->icon_renderer, "pixbuf", icon, NULL );
@@ -522,6 +573,13 @@ render_minimal( TorrentCellRenderer   * cell,
     gtk_cell_renderer_get_size( text_renderer, widget, NULL, NULL, NULL, &w, &h );
     stat_area.width = w;
     stat_area.height = h;
+    prog_area.height = p->bar_height;
+
+    h = 1;
+    h = MAX( h, stat_area.height );
+    h = MAX( h, name_area.height );
+    h = MAX( h, icon_area.height );
+    h = MAX( h, prog_area.height );
 
     /**
     *** LAYOUT
@@ -535,35 +593,36 @@ render_minimal( TorrentCellRenderer   * cell,
 
     /* icon */
     icon_area.x = fill_area.x;
-    icon_area.y = fill_area.y + ( fill_area.height - icon_area.height ) / 2;
+    icon_area.y = fill_area.y;
+    icon_area.height = h;
+
+    /* progressbar */
+    prog_area.x = fill_area.x + fill_area.width - BAR_WIDTH;
+    prog_area.y = fill_area.y;
+    prog_area.width = BAR_WIDTH;
+    prog_area.height = h;
 
     /* short status (right justified) */
-    stat_area.x = fill_area.x + fill_area.width - stat_area.width;
-    stat_area.y = fill_area.y + ( name_area.height - stat_area.height ) / 2;
+    stat_area.x = prog_area.x - GUI_PAD - stat_area.width;
+    stat_area.y = fill_area.y + ( h - stat_area.height ) / 2;
 
     /* name */
     name_area.x = icon_area.x + icon_area.width + GUI_PAD;
-    name_area.y = fill_area.y;
+    name_area.y = fill_area.y + ( h - name_area.height ) / 2;
     name_area.width = stat_area.x - GUI_PAD - name_area.x;
 
-    /* progressbar */
-    prog_area.x = name_area.x;
-    prog_area.y = name_area.y + name_area.height;
-    prog_area.width = name_area.width + GUI_PAD + stat_area.width;
-    prog_area.height = p->bar_height;
-
     /**
     *** RENDER
     **/
 
     g_object_set( p->icon_renderer, "pixbuf", icon, "sensitive", sensitive, NULL );
     gtk_cell_renderer_render( p->icon_renderer, window, widget, &icon_area, &icon_area, &icon_area, flags );
+    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", NULL, "sensitive", sensitive, NULL );
+    gtk_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
     g_object_set( text_renderer, "text", status, "scale", SMALL_SCALE, "sensitive", sensitive, "ellipsize", PANGO_ELLIPSIZE_END, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &stat_area, &stat_area, &stat_area, flags );
     g_object_set( text_renderer, "text", name, "scale", 1.0, NULL );
     gtk_cell_renderer_render( text_renderer, window, widget, &name_area, &name_area, &name_area, flags );
-    g_object_set( p->progress_renderer, "value", (int)(percentDone*100.0), "text", "", "sensitive", sensitive, NULL );
-    gtk_cell_renderer_render( p->progress_renderer, window, widget, &prog_area, &prog_area, &prog_area, flags );
 
     /* cleanup */
     g_free( status );
@@ -602,7 +661,7 @@ render_full( TorrentCellRenderer   * cell,
 
     icon = get_icon( tor, FULL_ICON_SIZE, widget );
     name = inf->name;
-    status = getStatusString( st, p->upload_speed, p->download_speed );
+    status = getStatusString( tor, st, p->upload_speed, p->download_speed );
     progress = getProgressString( tor, inf, st );
 
     /* get the idealized cell dimensions */
@@ -699,8 +758,8 @@ torrent_cell_renderer_render( GtkCellRenderer       * cell,
     if( self && self->priv->tor )
     {
         struct TorrentCellRendererPrivate * p = self->priv;
-        if( p->minimal )
-            render_minimal( self, window, widget, background_area, cell_area, expose_area, flags );
+        if( p->compact )
+            render_compact( self, window, widget, background_area, cell_area, expose_area, flags );
         else
             render_full( self, window, widget, background_area, cell_area, expose_area, flags );
     }
@@ -725,7 +784,7 @@ torrent_cell_renderer_set_property( GObject      * object,
         case P_UPLOAD_SPEED:   p->upload_speed   = g_value_get_double( v ); break;
         case P_DOWNLOAD_SPEED: p->download_speed = g_value_get_double( v ); break;
         case P_BAR_HEIGHT:     p->bar_height     = g_value_get_int( v ); break;
-        case P_MINIMAL:        p->minimal        = g_value_get_boolean( v ); break;
+        case P_COMPACT:        p->compact        = g_value_get_boolean( v ); break;
         default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
     }
 }
@@ -745,7 +804,7 @@ torrent_cell_renderer_get_property( GObject     * object,
         case P_UPLOAD_SPEED:   g_value_set_double( v, p->upload_speed ); break;
         case P_DOWNLOAD_SPEED: g_value_set_double( v, p->download_speed ); break;
         case P_BAR_HEIGHT:     g_value_set_int( v, p->bar_height ); break;
-        case P_MINIMAL:        g_value_set_boolean( v, p->minimal ); break;
+        case P_COMPACT:        g_value_set_boolean( v, p->compact ); break;
         default: G_OBJECT_WARN_INVALID_PROPERTY_ID( object, property_id, pspec ); break;
     }
 }
@@ -810,9 +869,9 @@ torrent_cell_renderer_class_init( TorrentCellRendererClass * klass )
                                                       DEFAULT_BAR_HEIGHT,
                                                       G_PARAM_READWRITE ) );
 
-    g_object_class_install_property( gobject_class, P_MINIMAL,
-                                    g_param_spec_boolean( "minimal", NULL,
-                                                          "Minimal Mode",
+    g_object_class_install_property( gobject_class, P_COMPACT,
+                                    g_param_spec_boolean( "compact", NULL,
+                                                          "Compact Mode",
                                                           FALSE,
                                                           G_PARAM_READWRITE ) );
 }
@@ -839,10 +898,10 @@ torrent_cell_renderer_init( GTypeInstance *  instance,
     p->progress_renderer = gtk_cell_renderer_progress_new(  );
     p->icon_renderer = gtk_cell_renderer_pixbuf_new(  );
     g_object_set( p->text_renderer_err, "foreground", "red", NULL );
-    tr_object_ref_sink( p->text_renderer );
-    tr_object_ref_sink( p->text_renderer_err );
-    tr_object_ref_sink( p->progress_renderer );
-    tr_object_ref_sink( p->icon_renderer );
+    gtr_object_ref_sink( p->text_renderer );
+    gtr_object_ref_sink( p->text_renderer_err );
+    gtr_object_ref_sink( p->progress_renderer );
+    gtr_object_ref_sink( p->icon_renderer );
 
     p->bar_height = DEFAULT_BAR_HEIGHT;
 }
