@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tr-core.c 9070 2009-09-08 18:59:48Z charles $
+ * $Id: tr-core.c 11464 2010-12-05 18:30:28Z charles $
  *
  * Copyright (c) 2007-2008 Transmission authors and contributors
  *
@@ -38,6 +38,7 @@
 #include <libtransmission/rpcimpl.h>
 #include <libtransmission/json.h>
 #include <libtransmission/utils.h> /* tr_free */
+#include <libtransmission/web.h>
 
 #include "conf.h"
 #include "notify.h"
@@ -292,7 +293,7 @@ compareByName( GtkTreeModel *             model,
 
     gtk_tree_model_get( model, a, MC_NAME_COLLATED, &ca, -1 );
     gtk_tree_model_get( model, b, MC_NAME_COLLATED, &cb, -1 );
-    ret = strcmp( ca, cb );
+    ret = gtr_strcmp0( ca, cb );
     g_free( cb );
     g_free( ca );
     return ret;
@@ -392,12 +393,25 @@ compareByTracker( GtkTreeModel * model,
                   GtkTreeIter  * b,
                   gpointer       user_data UNUSED )
 {
-    const tr_torrent *ta, *tb;
+    const tr_torrent * ta;
+    const tr_torrent * tb;
+    const tr_info * aInf;
+    const tr_info * bInf;
+    const char * aTracker;
+    const char * bTracker;
 
     gtk_tree_model_get( model, a, MC_TORRENT_RAW, &ta, -1 );
     gtk_tree_model_get( model, b, MC_TORRENT_RAW, &tb, -1 );
-    return strcmp( tr_torrentInfo( ta )->trackers[0].announce,
-                   tr_torrentInfo( tb )->trackers[0].announce );
+
+    aInf = tr_torrentInfo( ta );
+    bInf = tr_torrentInfo( tb );
+    aTracker = aInf->trackerCount > 0 ? aInf->trackers[0].announce : NULL;
+    bTracker = bInf->trackerCount > 0 ? bInf->trackers[0].announce : NULL;
+
+    if( !aTracker && !bTracker ) return 0;
+    if( !aTracker ) return -1;
+    if( !bTracker ) return 1;
+    return strcmp( aTracker, bTracker );
 }
 
 static void
@@ -441,11 +455,11 @@ static void
 tr_core_apply_defaults( tr_ctor * ctor )
 {
     if( tr_ctorGetPaused( ctor, TR_FORCE, NULL ) )
-        tr_ctorSetPaused( ctor, TR_FORCE, !pref_flag_get( PREF_KEY_START ) );
+        tr_ctorSetPaused( ctor, TR_FORCE, !pref_flag_get( TR_PREFS_KEY_START ) );
 
     if( tr_ctorGetDeleteSource( ctor, NULL ) )
         tr_ctorSetDeleteSource( ctor,
-                               pref_flag_get( PREF_KEY_TRASH_ORIGINAL ) );
+                               pref_flag_get( TR_PREFS_KEY_TRASH_ORIGINAL ) );
 
     if( tr_ctorGetPeerLimit( ctor, TR_FORCE, NULL ) )
         tr_ctorSetPeerLimit( ctor, TR_FORCE,
@@ -458,35 +472,118 @@ tr_core_apply_defaults( tr_ctor * ctor )
     }
 }
 
-static int
-tr_strcmp( const void * a,
-           const void * b )
+static char *
+torrentTrackerString( tr_torrent * tor )
 {
-    if( a && b ) return strcmp( a, b );
-    if( a ) return 1;
-    if( b ) return -1;
-    return 0;
+    int i;
+    GString * str = g_string_new( NULL );
+    const tr_info * inf = tr_torrentInfo( tor );
+
+    for( i = 0; i < inf->trackerCount; ++i )
+    {
+        const tr_tracker_info * t = &inf->trackers[i];
+        g_string_append( str, t->announce );
+    }
+
+    return g_string_free( str, FALSE );
+}
+
+static gboolean
+isTorrentActive( tr_torrent * tor )
+{
+    const tr_stat * st = tr_torrentStat( tor );
+    return ( st->peersSendingToUs > 0 )
+        || ( st->peersGettingFromUs > 0 )
+        || ( st->activity == TR_STATUS_CHECK );
 }
 
 #ifdef HAVE_GIO
-static gboolean
-watchFolderIdle( gpointer gcore )
+
+struct watchdir_file
 {
-    TrCore * core = TR_CORE( gcore );
+    char * filename;
+    time_t mtime;
+};
 
-    core->priv->adding_from_watch_dir = TRUE;
-    tr_core_add_list_defaults( core, core->priv->monitor_files, TRUE );
-    core->priv->adding_from_watch_dir = FALSE;
-
-    /* cleanup */
-    core->priv->monitor_files = NULL;
-    core->priv->monitor_idle_tag = 0;
-    return FALSE;
+static int
+compare_watchdir_file_to_filename( const void * a, const void * filename )
+{
+    return strcmp( ((const struct watchdir_file*)a)->filename, filename );
 }
 
 static void
-maybeAddTorrent( TrCore *     core,
-                 const char * filename )
+watchdir_file_update_mtime( struct watchdir_file * file )
+{
+    GFile * gfile = g_file_new_for_path( file->filename );
+    GFileInfo * info = g_file_query_info( gfile, G_FILE_ATTRIBUTE_TIME_MODIFIED, 0, NULL, NULL );
+
+    file->mtime = g_file_info_get_attribute_uint64( info, G_FILE_ATTRIBUTE_TIME_MODIFIED );
+
+    g_object_unref( G_OBJECT( info ) );
+    g_object_unref( G_OBJECT( gfile ) );
+}
+
+static struct watchdir_file*
+watchdir_file_new( const char * filename )
+{
+    struct watchdir_file * f;
+
+    f = g_new( struct watchdir_file, 1 );
+    f->filename = g_strdup( filename );
+    watchdir_file_update_mtime( f );
+
+    return f;
+}
+
+static void
+watchdir_file_free( struct watchdir_file * f )
+{
+    g_free( f->filename );
+    g_free( f );
+}
+
+static gboolean
+watchFolderIdle( gpointer gcore )
+{
+    GSList * l;
+    GSList * addme = NULL;
+    GSList * monitor_files = NULL;
+    TrCore * core = TR_CORE( gcore );
+    const time_t now = time( NULL );
+    struct TrCorePrivate * p = core->priv;
+
+    /* of the monitor_files, make a list of those that haven't
+     * changed lately, since they should be ready to add */
+    for( l=p->monitor_files; l!=NULL; l=l->next ) {
+        struct watchdir_file * f = l->data;
+        watchdir_file_update_mtime( f );
+        if( f->mtime + 2 >= now )
+            monitor_files = g_slist_prepend( monitor_files, f );
+        else {
+            addme = g_slist_prepend( addme, g_strdup( f->filename ) );
+            watchdir_file_free( f );
+        }
+    }
+
+    /* add the torrents from that list */
+    core->priv->adding_from_watch_dir = TRUE;
+    tr_core_add_list_defaults( core, addme, TRUE );
+    core->priv->adding_from_watch_dir = FALSE;
+
+    /* update the monitor_files list */
+    g_slist_free( p->monitor_files );
+    p->monitor_files = monitor_files;
+
+    /* if monitor_files is nonempty, keep checking every second */
+    if( core->priv->monitor_files )
+        return TRUE;
+    core->priv->monitor_idle_tag = 0;
+    return FALSE;
+
+}
+
+static void
+maybeAddTorrent( TrCore * core, const char * filename )
 {
     const gboolean isTorrent = g_str_has_suffix( filename, ".torrent" );
 
@@ -494,10 +591,9 @@ maybeAddTorrent( TrCore *     core,
     {
         struct TrCorePrivate * p = core->priv;
 
-        if( !g_slist_find_custom( p->monitor_files, filename,
-                                  (GCompareFunc)strcmp ) )
-            p->monitor_files =
-                g_slist_append( p->monitor_files, g_strdup( filename ) );
+        if( !g_slist_find_custom( p->monitor_files, filename, (GCompareFunc)compare_watchdir_file_to_filename ) )
+            p->monitor_files = g_slist_append( p->monitor_files, watchdir_file_new( filename ) );
+
         if( !p->monitor_idle_tag )
             p->monitor_idle_tag = gtr_timeout_add_seconds( 1, watchFolderIdle, core );
     }
@@ -525,18 +621,21 @@ scanWatchDir( TrCore * core )
 
     if( isEnabled )
     {
-        const char * basename;
         const char * dirname = pref_string_get( PREF_KEY_DIR_WATCH );
         GDir * dir = g_dir_open( dirname, 0, NULL );
 
-        while(( basename = g_dir_read_name( dir )))
+        if( dir != NULL )
         {
-            char * filename = g_build_filename( dirname, basename, NULL );
-            maybeAddTorrent( core, filename );
-            g_free( filename );
-        }
+            const char * basename;
+            while(( basename = g_dir_read_name( dir )))
+            {
+                char * filename = g_build_filename( dirname, basename, NULL );
+                maybeAddTorrent( core, filename );
+                g_free( filename );
+            }
 
-        g_dir_close( dir );
+            g_dir_close( dir );
+        }
     }
 }
 
@@ -548,7 +647,7 @@ updateWatchDir( TrCore * core )
         PREF_KEY_DIR_WATCH_ENABLED );
     struct TrCorePrivate * p = TR_CORE( core )->priv;
 
-    if( p->monitor && ( !isEnabled || tr_strcmp( filename, p->monitor_path ) ) )
+    if( p->monitor && ( !isEnabled || gtr_strcmp0( filename, p->monitor_path ) ) )
     {
         g_signal_handler_disconnect( p->monitor, p->monitor_tag );
         g_free( p->monitor_path );
@@ -625,7 +724,11 @@ tr_core_init( GTypeInstance *  instance,
                       G_TYPE_POINTER,   /* tr_torrent* */
                       G_TYPE_DOUBLE,    /* tr_stat.pieceUploadSpeed */
                       G_TYPE_DOUBLE,    /* tr_stat.pieceDownloadSpeed */
-                      G_TYPE_INT };     /* tr_stat.status */
+                      G_TYPE_BOOLEAN,   /* filter.c:ACTIVITY_FILTER_ACTIVE */
+                      G_TYPE_INT,       /* tr_stat.activity */
+                      G_TYPE_UCHAR,     /* tr_stat.finished */
+                      G_TYPE_CHAR,      /* tr_priority_t */
+                      G_TYPE_STRING };  /* concatenated trackers string */
 
     p = self->priv = G_TYPE_INSTANCE_GET_PRIVATE( self,
                                                   TR_CORE_TYPE,
@@ -726,9 +829,9 @@ tr_core_session( TrCore * core )
 static char*
 doCollate( const char * in )
 {
-    const char * end = in + strlen( in );
-    char *       casefold;
-    char *       ret;
+    char * ret;
+    char * casefold;
+    const char * end = in ? in + strlen( in ) : NULL;
 
     while( in < end )
     {
@@ -759,6 +862,7 @@ tr_core_add_torrent( TrCore     * self,
     const tr_stat * st = tr_torrent_stat( gtor );
     tr_torrent *    tor = tr_torrent_handle( gtor );
     char *          collated = doCollate( inf->name );
+    char *          trackers = torrentTrackerString( tor );
     GtkListStore *  store = GTK_LIST_STORE( tr_core_model( self ) );
     GtkTreeIter     unused;
 
@@ -769,7 +873,11 @@ tr_core_add_torrent( TrCore     * self,
                                        MC_TORRENT_RAW,   tor,
                                        MC_SPEED_UP,      st->pieceUploadSpeed,
                                        MC_SPEED_DOWN,    st->pieceDownloadSpeed,
+                                       MC_ACTIVE,        isTorrentActive( tor ),
                                        MC_ACTIVITY,      st->activity,
+                                       MC_FINISHED,      st->finished,
+                                       MC_PRIORITY,      tr_torrentGetPriority( tor ),
+                                       MC_TRACKERS,      trackers,
                                        -1 );
 
     if( doNotify )
@@ -778,6 +886,7 @@ tr_core_add_torrent( TrCore     * self,
     /* cleanup */
     g_object_unref( G_OBJECT( gtor ) );
     g_free( collated );
+    g_free( trackers );
 }
 
 int
@@ -861,44 +970,158 @@ add_ctor( TrCore * core, tr_ctor * ctor, gboolean doPrompt, gboolean doNotify )
     return err;
 }
 
+void
+tr_core_add_ctor( TrCore * core, tr_ctor * ctor )
+{
+    const gboolean doPrompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
+    const gboolean doNotify = FALSE;
+    tr_core_apply_defaults( ctor );
+    add_ctor( core, ctor, doPrompt, doNotify );
+}
+
 /* invoked remotely via dbus. */
 gboolean
 tr_core_add_metainfo( TrCore      * core,
-                      const char  * base64_metainfo,
-                      gboolean    * setme_success,
+                      const char  * payload,
+                      const char  * filename,
+                      gboolean    * setme_handled,
                       GError     ** gerr UNUSED )
 {
     tr_session * session = tr_core_session( core );
 
     if( !session )
     {
-        *setme_success = FALSE;
+        *setme_handled = FALSE;
+    }
+    else if( gtr_is_supported_url( payload ) || gtr_is_magnet_link( payload ) )
+    {
+        tr_core_add_from_url( core, payload );
+        *setme_handled = TRUE;
     }
     else
     {
-        int err;
-        int file_length;
         tr_ctor * ctor;
-        char * file_contents;
-        gboolean do_prompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
+        gboolean has_metainfo = FALSE;
+        const gboolean do_prompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT );
 
+        /* create the constructor */
         ctor = tr_ctorNew( session );
         tr_core_apply_defaults( ctor );
 
-        file_contents = tr_base64_decode( base64_metainfo, -1, &file_length );
-        err = tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
+        if( !has_metainfo && g_file_test( filename, G_FILE_TEST_IS_REGULAR ) )
+        {
+            /* set the metainfo from a local file */
+            has_metainfo = !tr_ctorSetMetainfoFromFile( ctor, filename );
+        }
 
-        if( !err )
-            err = add_ctor( core, ctor, do_prompt, TRUE );
+        if( !has_metainfo )
+        {
+            /* base64-encoded metainfo */
+            int file_length;
+            char * file_contents = tr_base64_decode( payload, -1, &file_length );
+            has_metainfo = !tr_ctorSetMetainfo( ctor, (const uint8_t*)file_contents, file_length );
+            tr_free( file_contents );
+        }
 
-        tr_free( file_contents );
-        tr_core_torrents_added( core );
-        *setme_success = TRUE;
+        if( has_metainfo )
+        {
+            add_ctor( core, ctor, do_prompt, TRUE );
+            tr_core_torrents_added( core );
+        }
+
+        *setme_handled = has_metainfo;
     }
 
     return TRUE;
 }
 
+/***
+****
+***/
+
+struct url_dialog_data
+{
+    TrCore * core;
+    tr_ctor * ctor;
+    char * url;
+    long response_code;
+};
+
+static gboolean
+onURLDoneIdle( gpointer vdata )
+{
+    struct url_dialog_data * data = vdata;
+    const gboolean doPrompt = pref_flag_get( PREF_KEY_OPTIONS_PROMPT ); 
+    const gboolean doNotify = FALSE; 
+    const int err = add_ctor( data->core, data->ctor, doPrompt, doNotify ); 
+
+    if( err == TR_PARSE_ERR ) 
+        tr_core_errsig( data->core, TR_PARSE_ERR, data->url ); 
+    
+     tr_core_torrents_added( data->core ); 
+
+    /* cleanup */
+    g_free( data->url );
+    g_free( data );
+    return FALSE;
+}
+
+static void
+onURLDone( tr_session       * session,
+           long               response_code,
+           const void       * response,
+           size_t             response_byte_count,
+           void             * vdata )
+{
+    struct url_dialog_data * data = vdata;
+
+    data->response_code = response_code; 
+    data->ctor = tr_ctorNew( session ); 
+    tr_core_apply_defaults( data->ctor ); 
+    tr_ctorSetMetainfo( data->ctor, response, response_byte_count ); 
+
+    gtr_idle_add( onURLDoneIdle, data ); 
+}
+
+void
+tr_core_add_from_url( TrCore * core, const char * url )
+{
+    tr_session * session = tr_core_session( core );
+    const gboolean is_magnet_link = gtr_is_magnet_link( url );
+
+    if( is_magnet_link || gtr_is_hex_hashcode( url ) )
+    {
+        int err;
+        char * tmp = NULL;
+        tr_ctor * ctor = tr_ctorNew( session );
+
+        if( gtr_is_hex_hashcode( url ) )
+            url = tmp = g_strdup_printf( "magnet:?xt=urn:btih:%s", url );
+
+        err = tr_ctorSetMetainfoFromMagnetLink( ctor, url );
+
+        if( !err )
+            tr_core_add_ctor( core, ctor );
+        else {
+            gtr_unrecognized_url_dialog( NULL, url );
+            tr_ctorFree( ctor );
+        }
+
+        g_free( tmp );
+    }
+    else
+    {
+        struct url_dialog_data * data = g_new( struct url_dialog_data, 1 );
+        data->core = core;
+        data->url = g_strdup( url );
+        tr_webRun( session, url, NULL, onURLDone, data );
+    }
+}
+
+/***
+****
+***/
+
 static void
 add_filename( TrCore      * core,
               const char  * filename,
@@ -907,18 +1130,31 @@ add_filename( TrCore      * core,
               gboolean      doNotify )
 {
     tr_session * session = tr_core_session( core );
-    if( filename && session )
+
+    if( session == NULL )
+        return;
+
+    if( gtr_is_supported_url( filename ) || gtr_is_magnet_link( filename ) )
+    {
+        tr_core_add_from_url( core, filename );
+    }
+    else if( g_file_test( filename, G_FILE_TEST_EXISTS ) )
     {
         int err;
+
         tr_ctor * ctor = tr_ctorNew( session );
+        tr_ctorSetMetainfoFromFile( ctor, filename );
         tr_core_apply_defaults( ctor );
         tr_ctorSetPaused( ctor, TR_FORCE, !doStart );
-        tr_ctorSetMetainfoFromFile( ctor, filename );
 
         err = add_ctor( core, ctor, doPrompt, doNotify );
         if( err == TR_PARSE_ERR )
             tr_core_errsig( core, TR_PARSE_ERR, filename );
     }
+    else if( gtr_is_hex_hashcode( filename ) )
+    {
+        tr_core_add_from_url( core, filename );
+    }
 }
 
 gboolean
@@ -926,7 +1162,10 @@ tr_core_present_window( TrCore      * core UNUSED,
                         gboolean *         success,
                         GError     ** err  UNUSED )
 {
-    action_activate( "present-main-window" );
+    /* Setting the toggle-main-window GtkCheckMenuItem to
+       make sure its state is correctly set */
+    action_toggle( "toggle-main-window", TRUE);
+    
     *success = TRUE;
     return TRUE;
 }
@@ -938,7 +1177,7 @@ tr_core_add_list( TrCore       * core,
                   pref_flag_t    prompt,
                   gboolean       doNotify )
 {
-    const gboolean doStart = pref_flag_eval( start, PREF_KEY_START );
+    const gboolean doStart = pref_flag_eval( start, TR_PREFS_KEY_START );
     const gboolean doPrompt = pref_flag_eval( prompt, PREF_KEY_OPTIONS_PROMPT );
     GSList * l;
 
@@ -980,48 +1219,40 @@ findTorrentInModel( TrCore *      core,
 }
 
 void
-tr_core_torrent_destroyed( TrCore * core,
-                           int      id )
+tr_core_remove_torrent( TrCore * core, TrTorrent * gtor, gboolean deleteFiles )
+{
+    const tr_torrent * tor = tr_torrent_handle( gtor );
+
+    if( tor != NULL )
+        tr_core_remove_torrent_from_id( core, tr_torrentId( tor ), deleteFiles );
+}
+
+void
+tr_core_remove_torrent_from_id( TrCore * core, int id, gboolean deleteFiles )
 {
     GtkTreeIter iter;
 
     if( findTorrentInModel( core, id, &iter ) )
     {
-        TrTorrent * gtor;
+        TrTorrent * gtor = NULL;
+        tr_torrent * tor = NULL;
         GtkTreeModel * model = tr_core_model( core );
-        gtk_tree_model_get( model, &iter, MC_TORRENT, &gtor, -1 );
-        tr_torrent_clear( gtor );
+
+        gtk_tree_model_get( model, &iter, MC_TORRENT, &gtor,
+                                          MC_TORRENT_RAW, &tor,
+                                          -1 );
+
+        /* remove from the gui */
         gtk_list_store_remove( GTK_LIST_STORE( model ), &iter );
-        g_object_unref( G_OBJECT( gtor ) );
-    }
-}
 
-void
-tr_core_remove_torrent( TrCore *    core,
-                        TrTorrent * gtor,
-                        int         deleteFiles )
-{
-    const tr_torrent * tor = tr_torrent_handle( gtor );
+        /* maybe delete the downloaded files */
+        if( deleteFiles )
+            tr_torrentDeleteLocalData( tor, gtr_file_trash_or_remove );
 
-    if( tor )
-    {
-        int         id = tr_torrentId( tor );
-        GtkTreeIter iter;
-        if( findTorrentInModel( core, id, &iter ) )
-        {
-            GtkTreeModel * model = tr_core_model( core );
-
-            /* remove from the gui */
-            gtk_list_store_remove( GTK_LIST_STORE( model ), &iter );
-
-            /* maybe delete the downloaded files */
-            if( deleteFiles )
-                tr_torrent_delete_files( gtor );
-
-            /* remove the torrent */
-            tr_torrent_set_remove_flag( gtor, TRUE );
-            g_object_unref( G_OBJECT( gtor ) );
-        }
+        /* remove the torrent */
+        tr_torrent_set_remove_flag( gtor, TRUE );
+        gtr_warn_if_fail( G_OBJECT( gtor )->ref_count == 1 );
+        g_object_unref( G_OBJECT( gtor ) ); /* remove the last refcount */
     }
 }
 
@@ -1036,33 +1267,55 @@ update_foreach( GtkTreeModel * model,
                 gpointer       data UNUSED )
 {
     int oldActivity, newActivity;
+    tr_bool oldFinished, newFinished;
+    tr_priority_t oldPriority, newPriority;
+    char * oldTrackers, * newTrackers;
     double oldUpSpeed, newUpSpeed;
     double oldDownSpeed, newDownSpeed;
+    gboolean oldActive, newActive;
     const tr_stat * st;
     TrTorrent * gtor;
+    tr_torrent * tor;
 
     /* get the old states */
     gtk_tree_model_get( model, iter,
                         MC_TORRENT, &gtor,
+                        MC_ACTIVE, &oldActive,
                         MC_ACTIVITY, &oldActivity,
+                        MC_FINISHED, &oldFinished,
+                        MC_PRIORITY, &oldPriority,
+                        MC_TRACKERS, &oldTrackers,
                         MC_SPEED_UP, &oldUpSpeed,
                         MC_SPEED_DOWN, &oldDownSpeed,
                         -1 );
 
     /* get the new states */
-    st = tr_torrentStat( tr_torrent_handle( gtor ) );
+    tor = tr_torrent_handle( gtor );
+    st = tr_torrentStat( tor );
+    newActive = isTorrentActive( tor );
     newActivity = st->activity;
+    newFinished = st->finished;
+    newPriority = tr_torrentGetPriority( tor );
+    newTrackers = torrentTrackerString( tor );
     newUpSpeed = st->pieceUploadSpeed;
     newDownSpeed = st->pieceDownloadSpeed;
 
     /* updating the model triggers off resort/refresh,
        so don't do it unless something's actually changed... */
-    if( ( newActivity != oldActivity ) ||
-        ( (int)(newUpSpeed*10.0) != (int)(oldUpSpeed*10.0) ) ||
-        ( (int)(newDownSpeed*10.0) != (int)(oldDownSpeed*10.0) ) )
+    if( ( newActive != oldActive )
+        || ( newActivity  != oldActivity )
+        || ( newFinished != oldFinished )
+        || ( newPriority != oldPriority )
+        || ( gtr_strcmp0( oldTrackers, newTrackers ) )
+        || ( (int)(newUpSpeed*10.0) != (int)(oldUpSpeed*10.0) )
+        || ( (int)(newDownSpeed*10.0) != (int)(oldDownSpeed*10.0) ) )
     {
         gtk_list_store_set( GTK_LIST_STORE( model ), iter,
+                            MC_ACTIVE, newActive,
                             MC_ACTIVITY, newActivity,
+                            MC_FINISHED, newFinished,
+                            MC_PRIORITY, newPriority,
+                            MC_TRACKERS, newTrackers,
                             MC_SPEED_UP, newUpSpeed,
                             MC_SPEED_DOWN, newDownSpeed,
                             -1 );
@@ -1070,6 +1323,8 @@ update_foreach( GtkTreeModel * model,
 
     /* cleanup */
     g_object_unref( gtor );
+    g_free( newTrackers );
+    g_free( oldTrackers );
     return FALSE;
 }
 
@@ -1112,8 +1367,11 @@ tr_core_quit( TrCore * core )
 static DBusGProxy*
 get_hibernation_inhibit_proxy( void )
 {
-    GError *          error = NULL;
     DBusGConnection * conn;
+    GError * error = NULL;
+    const char * name = "org.gnome.SessionManager";
+    const char * path = "/org/gnome/SessionManager";
+    const char * interface = "org.gnome.SessionManager";
 
     conn = dbus_g_bus_get( DBUS_BUS_SESSION, &error );
     if( error )
@@ -1123,11 +1381,7 @@ get_hibernation_inhibit_proxy( void )
         return NULL;
     }
 
-    return dbus_g_proxy_new_for_name (
-               conn,
-               "org.freedesktop.PowerManagement",
-               "/org/freedesktop/PowerManagement/Inhibit",
-               "org.freedesktop.PowerManagement.Inhibit" );
+    return dbus_g_proxy_new_for_name ( conn, name, path, interface );
 }
 
 static gboolean
@@ -1138,15 +1392,21 @@ gtr_inhibit_hibernation( guint * cookie )
 
     if( proxy )
     {
-        GError *     error = NULL;
+        GError * error = NULL;
+        const int toplevel_xid = 0;
         const char * application = _( "Transmission Bittorrent Client" );
         const char * reason = _( "BitTorrent Activity" );
+        const int flags = 4; /* Inhibit suspending the session or computer */
+
         success = dbus_g_proxy_call( proxy, "Inhibit", &error,
                                      G_TYPE_STRING, application,
+                                     G_TYPE_UINT, toplevel_xid,
                                      G_TYPE_STRING, reason,
-                                     G_TYPE_INVALID,
+                                     G_TYPE_UINT, flags,
+                                     G_TYPE_INVALID, /* sentinel - end of input args */
                                      G_TYPE_UINT, cookie,
-                                     G_TYPE_INVALID );
+                                     G_TYPE_INVALID /* senitnel - end of output args */ );
+
         if( success )
             tr_inf( "%s", _( "Disallowing desktop hibernation" ) );
         else
@@ -1170,7 +1430,7 @@ gtr_uninhibit_hibernation( guint inhibit_cookie )
     if( proxy )
     {
         GError * error = NULL;
-        gboolean success = dbus_g_proxy_call( proxy, "UnInhibit", &error,
+        gboolean success = dbus_g_proxy_call( proxy, "Uninhibit", &error,
                                               G_TYPE_UINT, inhibit_cookie,
                                               G_TYPE_INVALID,
                                               G_TYPE_INVALID );
@@ -1220,17 +1480,12 @@ tr_core_set_hibernation_allowed( TrCore * core,
 static void
 maybeInhibitHibernation( TrCore * core )
 {
-    gboolean inhibit = pref_flag_get( PREF_KEY_INHIBIT_HIBERNATION );
-
-    /* always allow hibernation when all the torrents are paused */
-    if( inhibit ) {
-        tr_session * session = tr_core_session( core );
-
-        if( tr_sessionGetActiveTorrentCount( session ) == 0 )
-            inhibit = FALSE;
-    }
-
-    tr_core_set_hibernation_allowed( core, !inhibit );
+    /* hibernation is allowed if EITHER
+     * (a) the "inhibit" pref is turned off OR
+     * (b) there aren't any active torrents */
+    const gboolean hibernation_allowed = !pref_flag_get( PREF_KEY_INHIBIT_HIBERNATION )
+                                      || !tr_core_get_active_torrent_count( core );
+    tr_core_set_hibernation_allowed( core, hibernation_allowed );
 }
 
 /**
@@ -1252,7 +1507,7 @@ tr_core_set_pref( TrCore *     self,
 {
     const char * oldval = pref_string_get( key );
 
-    if( tr_strcmp( oldval, newval ) )
+    if( gtr_strcmp0( oldval, newval ) )
     {
         pref_string_set( key, newval );
         commitPrefsChange( self, key );
@@ -1464,3 +1719,55 @@ tr_core_exec( TrCore * core, const tr_benc * top )
     tr_core_exec_json( core, json );
     tr_free( json );
 }
+
+/***
+****
+***/
+
+void
+tr_core_torrent_changed( TrCore * core, int id )
+{
+    GtkTreeIter iter;
+    GtkTreeModel * model = tr_core_model( core );
+
+    if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+    {
+        tr_torrent * tor;
+        gtk_tree_model_get( model, &iter, MC_TORRENT_RAW, &tor, -1 );
+        if( tr_torrentId( tor ) == id )
+        {
+            GtkTreePath * path = gtk_tree_model_get_path( model, &iter );
+            gtk_tree_model_row_changed( model, path, &iter );
+            gtk_tree_path_free( path );
+            break;
+        }
+    }
+    while( gtk_tree_model_iter_next( model, &iter ) );
+}
+
+int
+tr_core_get_torrent_count( TrCore * core )
+{
+    return gtk_tree_model_iter_n_children( tr_core_model( core ), NULL );
+}
+
+int
+tr_core_get_active_torrent_count( TrCore * core )
+{
+    GtkTreeIter iter;
+    GtkTreeModel * model = tr_core_model( core );
+    int activeCount = 0;
+
+    if( gtk_tree_model_get_iter_first( model, &iter ) ) do
+    {
+        int activity;
+        gtk_tree_model_get( model, &iter, MC_ACTIVITY, &activity, -1 );
+
+        if( activity != TR_STATUS_STOPPED )
+            ++activeCount;
+    }
+    while( gtk_tree_model_iter_next( model, &iter ) );
+
+    return activeCount;
+}
+
