@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,21 +7,23 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: bencode.c 9051 2009-09-07 06:19:26Z charles $
+ * $Id: bencode.c 11482 2010-12-05 19:01:18Z charles $
  */
 
 #include <assert.h>
-#include <ctype.h> /* isdigit, isprint, isspace */
+#include <ctype.h> /* isdigit() */
 #include <errno.h>
-#include <math.h> /* fabs */
+#include <math.h> /* fabs() */
 #include <stdio.h>
-#include <stdlib.h>
+#include <stdlib.h> /* realpath() */
 #include <string.h>
 
+#include <sys/types.h> /* stat() */
+#include <sys/stat.h> /* stat() */
 #include <locale.h>
-#include <unistd.h> /* close() */
+#include <unistd.h> /* stat(), close() */
 
-#include <event.h> /* evbuffer */
+#include <event.h> /* struct evbuffer */
 
 #include "ConvertUTF.h"
 
@@ -29,6 +31,7 @@
 #include "bencode.h"
 #include "json.h"
 #include "list.h"
+#include "platform.h" /* TR_PATH_MAX */
 #include "ptrarray.h"
 #include "utils.h" /* tr_new(), tr_free() */
 
@@ -83,7 +86,6 @@ tr_bencParseInt( const uint8_t *  buf,
                  const uint8_t ** setme_end,
                  int64_t *        setme_val )
 {
-    int          err = 0;
     char *       endptr;
     const void * begin;
     const void * end;
@@ -102,16 +104,13 @@ tr_bencParseInt( const uint8_t *  buf,
     errno = 0;
     val = evutil_strtoll( begin, &endptr, 10 );
     if( errno || ( endptr != end ) ) /* incomplete parse */
-        err = EILSEQ;
-    else if( val && *(const char*)begin == '0' ) /* no leading zeroes! */
-        err = EILSEQ;
-    else
-    {
-        *setme_end = (const uint8_t*)end + 1;
-        *setme_val = val;
-    }
+        return EILSEQ;
+    if( val && *(const char*)begin == '0' ) /* no leading zeroes! */
+        return EILSEQ;
 
-    return err;
+    *setme_end = (const uint8_t*)end + 1;
+    *setme_val = val;
+    return 0;
 }
 
 /**
@@ -357,23 +356,16 @@ tr_bencLoad( const void * buf_in,
 ****
 ***/
 
-/* returns true if the given string length would fit in our string buffer */
-static TR_INLINE int
-stringFitsInBuffer( const tr_benc * val, int len )
-{
-    return len < (int)sizeof( val->val.s.str.buf );
-}
-
 /* returns true if the benc's string was malloced.
  * this occurs when the string is too long for our string buffer */
-static TR_INLINE int
+static inline int
 stringIsAlloced( const tr_benc * val )
 {
-    return !stringFitsInBuffer( val, val->val.s.len );
+    return val->val.s.len >= sizeof( val->val.s.str.buf );
 }
 
 /* returns a const pointer to the benc's string */
-static TR_INLINE const char*
+static inline const char*
 getStr( const tr_benc* val )
 {
     return stringIsAlloced(val) ? val->val.s.str.ptr : val->val.s.str.buf;
@@ -579,29 +571,34 @@ tr_bencDictFindRaw( tr_benc         * dict,
 void
 tr_bencInitRaw( tr_benc * val, const void * src, size_t byteCount )
 {
+    char * setme;
     tr_bencInit( val, TR_TYPE_STR );
 
-    if( stringFitsInBuffer( val, val->val.s.len = byteCount ))
-        memcpy( val->val.s.str.buf, src, byteCount );
+    /* There's no way in benc notation to distinguish between
+     * zero-terminated strings and raw byte arrays.
+     * Because of this, tr_bencMergeDicts() and tr_bencListCopy()
+     * don't know whether or not a TR_TYPE_STR node needs a '\0'.
+     * Append one, een to the raw arrays, just to be safe. */
+
+    if( byteCount < sizeof( val->val.s.str.buf ) )
+        setme = val->val.s.str.buf;
     else
-        val->val.s.str.ptr = tr_memdup( src, byteCount );
+        setme = val->val.s.str.ptr = tr_new( char, byteCount + 1 );
+
+    memcpy( setme, src, byteCount );
+    setme[byteCount] = '\0';
+    val->val.s.len = byteCount;
 }
 
 void
 tr_bencInitStr( tr_benc * val, const void * str, int len )
 {
-    tr_bencInit( val, TR_TYPE_STR );
-
     if( str == NULL )
         len = 0;
     else if( len < 0 )
         len = strlen( str );
 
-    if( stringFitsInBuffer( val, val->val.s.len = len )) {
-        memcpy( val->val.s.str.buf, str, len );
-        val->val.s.str.buf[len] = '\0';
-    } else
-        val->val.s.str.ptr = tr_strndup( str, len );
+    tr_bencInitRaw( val, str, len );
 }
 
 void
@@ -673,8 +670,7 @@ tr_bencListAdd( tr_benc * list )
 }
 
 tr_benc *
-tr_bencListAddInt( tr_benc * list,
-                   int64_t   val )
+tr_bencListAddInt( tr_benc * list, int64_t val )
 {
     tr_benc * node = tr_bencListAdd( list );
 
@@ -683,15 +679,37 @@ tr_bencListAddInt( tr_benc * list,
 }
 
 tr_benc *
-tr_bencListAddStr( tr_benc *    list,
-                   const char * val )
+tr_bencListAddReal( tr_benc * list, double val )
 {
     tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitReal( node, val );
+    return node;
+}
 
+tr_benc *
+tr_bencListAddBool( tr_benc * list, tr_bool val )
+{
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitBool( node, val );
+    return node;
+}
+
+tr_benc *
+tr_bencListAddStr( tr_benc * list, const char * val )
+{
+    tr_benc * node = tr_bencListAdd( list );
     tr_bencInitStr( node, val, -1 );
     return node;
 }
 
+tr_benc *
+tr_bencListAddRaw( tr_benc * list, const uint8_t * val, size_t len )
+{
+    tr_benc * node = tr_bencListAdd( list );
+    tr_bencInitRaw( node, val, len );
+    return node;
+}
+
 tr_benc*
 tr_bencListAddList( tr_benc * list,
                     size_t    reserveCount )
@@ -804,6 +822,35 @@ tr_bencDictAddStr( tr_benc * dict, const char * key, const char * val )
     return child;
 }
 
+tr_benc*
+tr_bencDictAddRaw( tr_benc *    dict,
+                   const char * key,
+                   const void * src,
+                   size_t       len )
+{
+    tr_benc * child;
+
+    /* see if it already exists, and if so, try to reuse it */
+    if(( child = tr_bencDictFind( dict, key ))) {
+        if( tr_bencIsString( child ) ) {
+            if( stringIsAlloced( child ) )
+                tr_free( child->val.s.str.ptr );
+        } else {
+            tr_bencDictRemove( dict, key );
+            child = NULL;
+        }
+    }
+
+    /* if it doesn't exist, create it */
+    if( child == NULL )
+        child = tr_bencDictAdd( dict, key );
+
+    /* set it */
+    tr_bencInitRaw( child, src, len );
+
+    return child;
+}
+
 tr_benc*
 tr_bencDictAddList( tr_benc *    dict,
                     const char * key,
@@ -826,18 +873,6 @@ tr_bencDictAddDict( tr_benc *    dict,
     return child;
 }
 
-tr_benc*
-tr_bencDictAddRaw( tr_benc *    dict,
-                   const char * key,
-                   const void * src,
-                   size_t       len )
-{
-    tr_benc * child = tr_bencDictAdd( dict, key );
-
-    tr_bencInitRaw( child, src, len );
-    return child;
-}
-
 int
 tr_bencDictRemove( tr_benc *    dict,
                    const char * key )
@@ -888,18 +923,16 @@ struct SaveNode
     int *            children;
 };
 
-static struct SaveNode*
-nodeNewDict( const tr_benc * val )
+static void
+nodeInitDict( struct SaveNode * node, const tr_benc * val )
 {
     int               i, j;
     int               nKeys;
-    struct SaveNode * node;
     struct KeyIndex * indices;
 
     assert( tr_bencIsDict( val ) );
 
     nKeys = val->val.l.count / 2;
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
     node->children = tr_new0( int, nKeys * 2 );
 
@@ -920,53 +953,40 @@ nodeNewDict( const tr_benc * val )
 
     assert( node->childCount == nKeys * 2 );
     tr_free( indices );
-    return node;
 }
 
-static struct SaveNode*
-nodeNewList( const tr_benc * val )
+static void
+nodeInitList( struct SaveNode * node, const tr_benc * val )
 {
     int               i, n;
-    struct SaveNode * node;
 
     assert( tr_bencIsList( val ) );
 
     n = val->val.l.count;
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
     node->childCount = n;
     node->children = tr_new0( int, n );
     for( i = 0; i < n; ++i ) /* a list's children don't need to be reordered */
         node->children[i] = i;
-
-    return node;
 }
 
-static struct SaveNode*
-nodeNewLeaf( const tr_benc * val )
+static void
+nodeInitLeaf( struct SaveNode * node, const tr_benc * val )
 {
-    struct SaveNode * node;
-
     assert( !isContainer( val ) );
 
-    node = tr_new0( struct SaveNode, 1 );
     node->val = val;
-    return node;
 }
 
-static struct SaveNode*
-nodeNew( const tr_benc * val )
+static void
+nodeInit( struct SaveNode * node, const tr_benc * val )
 {
-    struct SaveNode * node;
+    static const struct SaveNode INIT_NODE = { NULL, 0, 0, 0, NULL };
+    *node = INIT_NODE;
 
-    if( tr_bencIsList( val ) )
-        node = nodeNewList( val );
-    else if( tr_bencIsDict( val ) )
-        node = nodeNewDict( val );
-    else
-        node = nodeNewLeaf( val );
-
-    return node;
+         if( tr_bencIsList( val ) ) nodeInitList( node, val );
+    else if( tr_bencIsDict( val ) ) nodeInitDict( node, val );
+    else                            nodeInitLeaf( node, val );
 }
 
 typedef void ( *BencWalkFunc )( const tr_benc * val, void * user_data );
@@ -992,13 +1012,15 @@ bencWalk( const tr_benc          * top,
           const struct WalkFuncs * walkFuncs,
           void                   * user_data )
 {
-    tr_ptrArray stack = TR_PTR_ARRAY_INIT;
+    int stackSize = 0;
+    int stackAlloc = 64;
+    struct SaveNode * stack = tr_new( struct SaveNode, stackAlloc );
 
-    tr_ptrArrayAppend( &stack, nodeNew( top ) );
+    nodeInit( &stack[stackSize++], top );
 
-    while( !tr_ptrArrayEmpty( &stack ) )
+    while( stackSize > 0 )
     {
-        struct SaveNode * node = tr_ptrArrayBack( &stack );
+        struct SaveNode * node = &stack[stackSize-1];
         const tr_benc *   val;
 
         if( !node->valIsVisited )
@@ -1015,9 +1037,8 @@ bencWalk( const tr_benc          * top,
         {
             if( isContainer( node->val ) )
                 walkFuncs->containerEndFunc( node->val, user_data );
-            tr_ptrArrayPop( &stack );
+            --stackSize;
             tr_free( node->children );
-            tr_free( node );
             continue;
         }
 
@@ -1040,17 +1061,27 @@ bencWalk( const tr_benc          * top,
                     break;
 
                 case TR_TYPE_LIST:
-                    if( val != node->val )
-                        tr_ptrArrayAppend( &stack, nodeNew( val ) );
-                    else
+                    if( val == node->val )
                         walkFuncs->listBeginFunc( val, user_data );
+                    else {
+                        if( stackAlloc == stackSize ) {
+                            stackAlloc *= 2;
+                            stack = tr_renew( struct SaveNode, stack, stackAlloc );
+                        }
+                        nodeInit( &stack[stackSize++], val );
+                    }
                     break;
 
                 case TR_TYPE_DICT:
-                    if( val != node->val )
-                        tr_ptrArrayAppend( &stack, nodeNew( val ) );
-                    else
+                    if( val == node->val )
                         walkFuncs->dictBeginFunc( val, user_data );
+                    else {
+                        if( stackAlloc == stackSize ) {
+                            stackAlloc *= 2;
+                            stack = tr_renew( struct SaveNode, stack, stackAlloc );
+                        }
+                        nodeInit( &stack[stackSize++], val );
+                    }
                     break;
 
                 default:
@@ -1060,7 +1091,7 @@ bencWalk( const tr_benc          * top,
             }
     }
 
-    tr_ptrArrayDestruct( &stack, NULL );
+    tr_free( stack );
 }
 
 /****
@@ -1138,48 +1169,42 @@ static const struct WalkFuncs saveFuncs = { saveIntFunc,
 ***/
 
 static void
-freeDummyFunc( const tr_benc * val UNUSED,
-               void * buf          UNUSED  )
+freeDummyFunc( const tr_benc * val UNUSED, void * buf UNUSED  )
 {}
 
 static void
-freeStringFunc( const tr_benc * val,
-                void *          freeme )
+freeStringFunc( const tr_benc * val, void * unused UNUSED )
 {
     if( stringIsAlloced( val ) )
-        tr_ptrArrayAppend( freeme, val->val.s.str.ptr );
+        tr_free( val->val.s.str.ptr );
 }
 
 static void
-freeContainerBeginFunc( const tr_benc * val,
-                        void *          freeme )
+freeContainerEndFunc( const tr_benc * val, void * unused UNUSED )
 {
-    tr_ptrArrayAppend( freeme, val->val.l.vals );
+    tr_free( val->val.l.vals );
 }
 
 static const struct WalkFuncs freeWalkFuncs = { freeDummyFunc,
                                                 freeDummyFunc,
                                                 freeDummyFunc,
                                                 freeStringFunc,
-                                                freeContainerBeginFunc,
-                                                freeContainerBeginFunc,
-                                                freeDummyFunc };
+                                                freeDummyFunc,
+                                                freeDummyFunc,
+                                                freeContainerEndFunc };
 
 void
 tr_bencFree( tr_benc * val )
 {
     if( isSomething( val ) )
-    {
-        tr_ptrArray a = TR_PTR_ARRAY_INIT;
-        bencWalk( val, &freeWalkFuncs, &a );
-        tr_ptrArrayDestruct( &a, tr_free );
-    }
+        bencWalk( val, &freeWalkFuncs, NULL );
 }
 
 /***
 ****
 ***/
 
+/** @brief Implementation helper class for tr_bencToBuffer(TR_FMT_JSON) */
 struct ParentState
 {
     int    bencType;
@@ -1187,6 +1212,7 @@ struct ParentState
     int    childCount;
 };
 
+/** @brief Implementation helper class for tr_bencToBuffer(TR_FMT_JSON) */
 struct jsonWalk
 {
     tr_bool doIndent;
@@ -1322,7 +1348,6 @@ jsonStringFunc( const tr_benc * val, void * vdata )
     {
         switch( *it )
         {
-            case '/': evbuffer_add( data->out, "\\/", 2 ); break;
             case '\b': evbuffer_add( data->out, "\\b", 2 ); break;
             case '\f': evbuffer_add( data->out, "\\f", 2 ); break;
             case '\n': evbuffer_add( data->out, "\\n", 2 ); break;
@@ -1339,7 +1364,7 @@ jsonStringFunc( const tr_benc * val, void * vdata )
                     UTF32        buf = 0;
                     UTF32 *      u32 = &buf;
                     ConversionResult result = ConvertUTF8toUTF32( &tmp, end, &u32, &buf + 1, 0 );
-                    if((( result==conversionOK ) || (result==targetExhausted)) && (tmp!=it)) { 
+                    if((( result==conversionOK ) || (result==targetExhausted)) && (tmp!=it)) {
                         evbuffer_add_printf( data->out, "\\u%04x", (unsigned int)buf );
                         it = tmp - 1;
                     }
@@ -1404,6 +1429,51 @@ static const struct WalkFuncs jsonWalkFuncs = { jsonIntFunc,
 ****
 ***/
 
+static void
+tr_bencListCopy( tr_benc * target, const tr_benc * src )
+{
+    int i = 0;
+    const tr_benc * val;
+
+    while(( val = tr_bencListChild( (tr_benc*)src, i++ )))
+    {
+       if( tr_bencIsBool( val ) )
+       {
+           tr_bool boolVal = 0;
+           tr_bencGetBool( val, &boolVal );
+           tr_bencListAddBool( target, boolVal );
+       }
+       else if( tr_bencIsReal( val ) )
+       {
+           double realVal = 0;
+           tr_bencGetReal( val, &realVal );
+           tr_bencListAddReal( target, realVal );
+       }
+       else if( tr_bencIsInt( val ) )
+       {
+           int64_t intVal = 0;
+           tr_bencGetInt( val, &intVal );
+           tr_bencListAddInt( target, intVal );
+       }
+       else if( tr_bencIsString( val ) )
+       {
+           tr_bencListAddRaw( target, (const uint8_t*)getStr( val ), val->val.s.len );
+       }
+       else if( tr_bencIsDict( val ) )
+       {
+           tr_bencMergeDicts( tr_bencListAddDict( target, 0 ), val );
+       }
+       else if ( tr_bencIsList( val ) )
+       {
+           tr_bencListCopy( tr_bencListAddList( target, 0 ), val );
+       }
+       else
+       {
+           tr_err( "tr_bencListCopy skipping item" );
+       }
+   }
+}
+
 static size_t
 tr_bencDictSize( const tr_benc * dict )
 {
@@ -1470,14 +1540,19 @@ tr_bencMergeDicts( tr_benc * target, const tr_benc * source )
             }
             else if( tr_bencIsString( val ) )
             {
-                const char * strVal = NULL;
-                tr_bencGetStr( val, &strVal );
-                tr_bencDictAddStr( target, key, strVal );
+                tr_bencDictAddRaw( target, key, getStr( val ), val->val.s.len );
             }
             else if( tr_bencIsDict( val ) && tr_bencDictFindDict( target, key, &t ) )
             {
                 tr_bencMergeDicts( t, val );
             }
+            else if( tr_bencIsList( val ) )
+            {
+                if( tr_bencDictFind( target, key ) == NULL )
+                {
+                    tr_bencListCopy( tr_bencDictAddList( target, key, tr_bencListSize( val ) ), val );
+                }
+            }
             else
             {
                 tr_dbg( "tr_bencMergeDicts skipping \"%s\"", key );
@@ -1494,6 +1569,7 @@ void
 tr_bencToBuf( const tr_benc * top, tr_fmt_mode mode, struct evbuffer * buf )
 {
     evbuffer_drain( buf, EVBUFFER_LENGTH( buf ) );
+    evbuffer_expand( buf, 4096 ); /* alloc a little memory to start off with */
 
     switch( mode )
     {
@@ -1531,31 +1607,72 @@ tr_bencToStr( const tr_benc * top, tr_fmt_mode mode, int * len )
 int
 tr_bencToFile( const tr_benc * top, tr_fmt_mode mode, const char * filename )
 {
+    char * tmp;
+    int fd;
     int err = 0;
-    FILE * fp = fopen( filename, "wb+" );
+    char buf[TR_PATH_MAX];
 
-    if( fp == NULL )
-    {
-        err = errno;
-        tr_err( _( "Couldn't open \"%1$s\": %2$s" ),
-                filename, tr_strerror( errno ) );
-    }
-    else
+    /* follow symlinks to find the "real" file, to make sure the temporary
+     * we build with mkstemp() is created on the right partition */
+    if( realpath( filename, buf ) != NULL )
+        filename = buf;
+
+    /* if the file already exists, try to move it out of the way & keep it as a backup */
+    tmp = tr_strdup_printf( "%s.tmp.XXXXXX", filename );
+    fd = mkstemp( tmp );
+    if( fd >= 0 )
     {
         int len;
         char * str = tr_bencToStr( top, mode, &len );
+        tr_dbg( "Writing %d bytes to temporary file \"%s\"", (int)len, tmp );
 
-        if( fwrite( str, 1, len, fp ) == (size_t)len )
-            tr_dbg( "tr_bencToFile saved \"%s\"", filename );
-        else {
+        if( write( fd, str, len ) == (ssize_t)len )
+        {
+            struct stat sb;
+            const tr_bool already_exists = !stat( filename, &sb ) && S_ISREG( sb.st_mode );
+
+            fsync( fd );
+            close( fd );
+
+            if( !already_exists || !unlink( filename ) )
+            {
+                tr_dbg( "Renaming \"%s\" as \"%s\"", tmp, filename );
+
+                if( !rename( tmp, filename ) )
+                {
+                    tr_inf( _( "Saved \"%s\"" ), filename );
+                }
+                else
+                {
+                    err = errno;
+                    tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
+                    unlink( tmp );
+                }
+            }
+            else
+            {
+                err = errno;
+                tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( err ) );
+                unlink( tmp );
+            }
+        }
+        else
+        {
             err = errno;
-            tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), filename, tr_strerror( errno ) );
+            tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
+            close( fd );
+            unlink( tmp );
         }
 
         tr_free( str );
-        fclose( fp );
+    }
+    else
+    {
+        err = errno;
+        tr_err( _( "Couldn't save temporary file \"%1$s\": %2$s" ), tmp, tr_strerror( err ) );
     }
 
+    tr_free( tmp );
     return err;
 }
 
