@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2008-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2008-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,11 +7,11 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: blocklist.c 7658 2009-01-10 23:09:07Z charles $
+ * $Id: blocklist.c 10754 2010-06-15 00:38:10Z charles $
  */
 
 #include <stdio.h>
-#include <stdlib.h> /* free */
+#include <stdlib.h> /* free() */
 #include <string.h>
 
 #ifdef WIN32
@@ -27,8 +27,6 @@
 #include <unistd.h>
 #include <assert.h>
 
-#include "ggets.h"
-
 #include "transmission.h"
 #include "platform.h"
 #include "blocklist.h"
@@ -212,7 +210,8 @@ _tr_blocklistHasAddress( tr_blocklist     * b,
         return 0;
 
     blocklistEnsureLoaded( b );
-    if( !b->rules )
+
+    if( !b->rules || !b->ruleCount )
         return 0;
 
     needle = ntohl( addr->addr.addr4.s_addr );
@@ -226,14 +225,88 @@ _tr_blocklistHasAddress( tr_blocklist     * b,
     return range != NULL;
 }
 
+/*
+ * level1 format: "comment:x.x.x.x-y.y.y.y"
+ */
+static tr_bool
+parseLine1( const char * line, struct tr_ip_range * range )
+{
+    char * walk;
+    int b[4];
+    int e[4];
+    char str[64];
+    tr_address addr;
+
+    walk = strrchr( line, ':' );
+    if( !walk )
+        return FALSE;
+    ++walk; /* walk past the colon */
+
+    if( sscanf( walk, "%d.%d.%d.%d-%d.%d.%d.%d",
+                &b[0], &b[1], &b[2], &b[3],
+                &e[0], &e[1], &e[2], &e[3] ) != 8 )
+        return FALSE;
+
+    tr_snprintf( str, sizeof( str ), "%d.%d.%d.%d", b[0], b[1], b[2], b[3] );
+    if( tr_pton( str, &addr ) == NULL )
+        return FALSE;
+    range->begin = ntohl( addr.addr.addr4.s_addr );
+
+    tr_snprintf( str, sizeof( str ), "%d.%d.%d.%d", e[0], e[1], e[2], e[3] );
+    if( tr_pton( str, &addr ) == NULL )
+        return FALSE;
+    range->end = ntohl( addr.addr.addr4.s_addr );
+
+    return TRUE;
+}
+
+/*
+ * "000.000.000.000 - 000.255.255.255 , 000 , invalid ip"
+ */
+static tr_bool
+parseLine2( const char * line, struct tr_ip_range * range )
+{
+    int unk;
+    int a[4];
+    int b[4];
+    char str[32];
+    tr_address addr;
+
+    if( sscanf( line, "%3d.%3d.%3d.%3d - %3d.%3d.%3d.%3d , %3d , ",
+                &a[0], &a[1], &a[2], &a[3],
+                &b[0], &b[1], &b[2], &b[3],
+                &unk ) != 9 )
+        return FALSE;
+
+    tr_snprintf( str, sizeof(str), "%d.%d.%d.%d", a[0], a[1], a[2], a[3] );
+    if( tr_pton( str, &addr ) == NULL )
+        return FALSE;
+    range->begin = ntohl( addr.addr.addr4.s_addr );
+
+    tr_snprintf( str, sizeof(str), "%d.%d.%d.%d", b[0], b[1], b[2], b[3] );
+    if( tr_pton( str, &addr ) == NULL )
+        return FALSE;
+    range->end = ntohl( addr.addr.addr4.s_addr );
+
+    return TRUE;
+}
+
+static int
+parseLine( const char * line, struct tr_ip_range * range )
+{
+    return parseLine1( line, range )
+        || parseLine2( line, range );
+}
+
 int
 _tr_blocklistSetContent( tr_blocklist * b,
                          const char *   filename )
 {
-    FILE *       in;
-    FILE *       out;
-    char *       line;
-    int          lineCount = 0;
+    FILE * in;
+    FILE * out;
+    int inCount = 0;
+    int outCount = 0;
+    char line[2048];
     const char * err_fmt = _( "Couldn't read \"%1$s\": %2$s" );
 
     if( !filename )
@@ -259,57 +332,45 @@ _tr_blocklistSetContent( tr_blocklist * b,
         return 0;
     }
 
-    while( !fggets( &line, in ) )
+    while( fgets( line, sizeof( line ), in ) != NULL )
     {
-        char * rangeBegin;
-        char * rangeEnd;
-        char * crpos;
-        tr_address  addr;
+        char * walk;
         struct tr_ip_range range;
 
-        rangeBegin = strrchr( line, ':' );
-        if( !rangeBegin ){ free( line ); continue; }
-        ++rangeBegin;
+        ++inCount;
 
-        rangeEnd = strchr( rangeBegin, '-' );
-        if( !rangeEnd ){ free( line ); continue; }
-        *rangeEnd++ = '\0';
-        if(( crpos = strchr( rangeEnd, '\r' )))
-            *crpos = '\0';
+        /* zap the linefeed */
+        if(( walk = strchr( line, '\r' ))) *walk = '\0'; 
+        if(( walk = strchr( line, '\n' ))) *walk = '\0'; 
 
-        if( !tr_pton( rangeBegin, &addr ) )
-            tr_err( "blocklist skipped invalid address [%s]\n", rangeBegin );
-        range.begin = ntohl( addr.addr.addr4.s_addr );
-
-        if( !tr_pton( rangeEnd, &addr ) )
-            tr_err( "blocklist skipped invalid address [%s]\n", rangeEnd );
-        range.end = ntohl( addr.addr.addr4.s_addr );
-
-        free( line );
+        if( !parseLine( line, &range ) )
+        {
+            /* don't try to display the actual lines - it causes issues */
+            tr_err( _( "blocklist skipped invalid address at line %d" ), inCount );
+            continue;
+        }
 
         if( fwrite( &range, sizeof( struct tr_ip_range ), 1, out ) != 1 )
         {
-            tr_err( _(
-                       "Couldn't save file \"%1$s\": %2$s" ), b->filename,
+            tr_err( _( "Couldn't save file \"%1$s\": %2$s" ), b->filename,
                    tr_strerror( errno ) );
             break;
         }
 
-        ++lineCount;
+        ++outCount;
     }
 
     {
         char * base = tr_basename( b->filename );
-        tr_inf( _( "Blocklist \"%1$s\" updated with %2$'d entries" ), base, lineCount );
+        tr_inf( _( "Blocklist \"%1$s\" updated with %2$'d entries" ), base, outCount );
         tr_free( base );
     }
 
-
     fclose( out );
     fclose( in );
 
     blocklistLoad( b );
 
-    return lineCount;
+    return outCount;
 }
 
