@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: handshake.c 9366 2009-10-22 02:50:59Z charles $
+ * $Id: handshake.c 10506 2010-04-21 04:44:35Z charles $
  */
 
 #include <assert.h>
@@ -64,7 +64,7 @@ enum
     CRYPTO_PROVIDE_CRYPTO          = 2,
 
     /* how long to wait before giving up on a handshake */
-    HANDSHAKE_TIMEOUT_SEC          = 60
+    HANDSHAKE_TIMEOUT_SEC          = 30
 };
 
 
@@ -101,6 +101,7 @@ enum
 
 struct tr_handshake
 {
+    tr_bool               haveReadAnythingFromPeer;
     tr_bool               havePeerID;
     tr_bool               haveSentBitTorrentHandshake;
     tr_peerIo *           io;
@@ -225,10 +226,10 @@ buildHandshakeMessage( tr_handshake * handshake, uint8_t * buf )
     HANDSHAKE_SET_LTEP( walk );
     HANDSHAKE_SET_FASTEXT( walk );
 
-    /* Note that this doesn't depend on whether the torrent is private.  We
-       don't accept DHT peers for a private torrent, but we participate in
-       the DHT regardless. */
-    if(tr_dhtEnabled(handshake->session))
+    /* Note that this doesn't depend on whether the torrent is private.
+     * We don't accept DHT peers for a private torrent,
+     * but we participate in the DHT regardless. */
+    if( tr_dhtEnabled( handshake->session ) )
         HANDSHAKE_SET_DHT( walk );
 
     walk += HANDSHAKE_FLAGS_LEN;
@@ -312,10 +313,7 @@ parseHandshake( tr_handshake *    handshake,
 
     tr_peerIoEnableFEXT( handshake->io, HANDSHAKE_HAS_FASTEXT( reserved ) );
 
-    /* This is independent of whether or not DHT is actually used.
-     * it's okay to set this flag even for private torrents */
-    if( tor && tr_sessionAllowsDHT( tor->session ) )
-        tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
+    tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
 
     return HANDSHAKE_OK;
 }
@@ -375,7 +373,7 @@ static uint32_t
 getCryptoSelect( const tr_handshake * handshake,
                  uint32_t             crypto_provide )
 {
-    uint32_t choices[4];
+    uint32_t choices[2];
     int      i, nChoices = 0;
 
     switch( handshake->encryptionMode )
@@ -434,6 +432,8 @@ readYb( tr_handshake *    handshake,
         return READ_NOW;
     }
 
+    handshake->haveReadAnythingFromPeer = TRUE;
+
     /* compute the secret */
     evbuffer_remove( inbuf, yb, KEY_LEN );
     secret = tr_cryptoComputeSecret( handshake->crypto, yb );
@@ -616,6 +616,8 @@ readHandshake( tr_handshake *    handshake,
     if( EVBUFFER_LENGTH( inbuf ) < INCOMING_HANDSHAKE_LEN )
         return READ_LATER;
 
+    handshake->haveReadAnythingFromPeer = TRUE;
+
     pstrlen = EVBUFFER_DATA( inbuf )[0]; /* peek, don't read.  We may be
                                           handing inbuf to AWAITING_YA */
 
@@ -675,10 +677,7 @@ readHandshake( tr_handshake *    handshake,
 
     tr_peerIoEnableFEXT( handshake->io, HANDSHAKE_HAS_FASTEXT( reserved ) );
 
-    /* This is independent of whether or not DHT is actually used.
-     * it's okay to set this flag even for private torrents */
-    if( tr_sessionAllowsDHT( handshake->session ) )
-        tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
+    tr_peerIoEnableDHT( handshake->io, HANDSHAKE_HAS_DHT( reserved ) );
 
     /* torrent hash */
     tr_peerIoReadBytes( handshake->io, inbuf, hash, sizeof( hash ) );
@@ -868,7 +867,7 @@ readCryptoProvide( tr_handshake *    handshake,
         const tr_bool clientIsSeed = tr_torrentIsSeed( tor );
         const tr_bool peerIsSeed = tr_peerMgrPeerIsSeed( tor, tr_peerIoGetAddress( handshake->io, NULL ) );
         dbgmsg( handshake, "got INCOMING connection's encrypted handshake for torrent [%s]",
-                tor->info.name );
+                tr_torrentName( tor ) );
         tr_peerIoSetTorrentHash( handshake->io, tor->info.hash );
 
         if( clientIsSeed && peerIsSeed )
@@ -1032,6 +1031,8 @@ canRead( struct tr_peerIo * io, void * arg, size_t * piece )
     ReadState         ret;
     tr_bool           readyForMore = TRUE;
 
+    assert( tr_isPeerIo( io ) );
+
     /* no piece data in handshake */
     *piece = 0;
 
@@ -1104,6 +1105,7 @@ fireDoneFunc( tr_handshake * handshake,
                             : NULL;
     const int success = ( *handshake->doneCB )( handshake,
                                                 handshake->io,
+                                                handshake->haveReadAnythingFromPeer,
                                                 isConnected,
                                                 peer_id,
                                                 handshake->doneUserData );
@@ -1141,15 +1143,16 @@ tr_handshakeDone( tr_handshake * handshake,
 void
 tr_handshakeAbort( tr_handshake * handshake )
 {
-    tr_handshakeDone( handshake, FALSE );
+    if( handshake != NULL )
+        tr_handshakeDone( handshake, FALSE );
 }
 
 static void
 gotError( tr_peerIo  * io UNUSED,
           short        what,
-          void       * arg )
+          void       * vhandshake )
 {
-    tr_handshake * handshake = (tr_handshake *) arg;
+    tr_handshake * handshake = vhandshake;
 
     /* if the error happened while we were sending a public key, we might
      * have encountered a peer that doesn't do encryption... reconnect and
