@@ -1,5 +1,6 @@
 /******************************************************************************
- * $Id: net.c 8889 2009-08-10 20:04:08Z charles $
+ *
+ * $Id: net.c 10753 2010-06-14 23:14:58Z charles $
  *
  * Copyright (c) 2005-2008 Transmission authors and contributors
  *
@@ -34,13 +35,16 @@
  #include <winsock2.h> /* inet_addr */
  #include <WS2tcpip.h>
 #else
+ #include <sys/socket.h>
+ #include <netinet/in.h>
+ #include <netinet/tcp.h>
  #include <arpa/inet.h> /* inet_addr */
  #include <netdb.h>
  #include <fcntl.h>
 #endif
 #include <unistd.h>
 
-#include <stdarg.h> /* 1.4.x versions of evutil.h need this */
+#include <stdarg.h> /* some 1.4.x versions of evutil.h need this */
 #include <evutil.h>
 
 #include "transmission.h"
@@ -185,9 +189,6 @@ tr_compareAddresses( const tr_address * a, const tr_address * b)
 {
     static const int sizes[2] = { sizeof(struct in_addr), sizeof(struct in6_addr) };
 
-    assert( tr_isAddress( a ) );
-    assert( tr_isAddress( b ) );
-
     /* IPv6 addresses are always "greater than" IPv4 */
     if( a->type != b->type )
         return a->type == TR_AF_INET ? 1 : -1;
@@ -209,6 +210,18 @@ tr_netSetTOS( int s, int tos )
 #endif
 }
 
+int
+tr_netSetCongestionControl( int s, const char *algorithm )
+{
+#ifdef TCP_CONGESTION
+    return setsockopt( s, IPPROTO_TCP, TCP_CONGESTION,
+                       algorithm, strlen(algorithm) + 1 );
+#else
+    errno = ENOSYS;
+    return -1;
+#endif
+}
+
 static socklen_t
 setup_sockaddr( const tr_address        * addr,
                 tr_port                   port,
@@ -239,56 +252,11 @@ setup_sockaddr( const tr_address        * addr,
     }
 }
 
-static tr_bool
-isMulticastAddress( const tr_address * addr )
-{
-    if( addr->type == TR_AF_INET && IN_MULTICAST( htonl( addr->addr.addr4.s_addr ) ) )
-        return TRUE;
-
-    if( addr->type == TR_AF_INET6 && ( addr->addr.addr6.s6_addr[0] == 0xff ) )
-        return TRUE;
-
-    return FALSE;
-}
-
-static TR_INLINE tr_bool
-isIPv4MappedOrCompatAddress( const tr_address * addr )
-{
-    if( addr->type == TR_AF_INET6 )
-    {
-        if( IN6_IS_ADDR_V4MAPPED( &addr->addr.addr6 ) ||
-            IN6_IS_ADDR_V4COMPAT( &addr->addr.addr6 ) )
-            return TRUE;
-    }
-    return FALSE;
-}
-
-static TR_INLINE tr_bool
-isIPv6LinkLocalAddress( const tr_address * addr )
-{
-    if( addr->type == TR_AF_INET6 &&
-        IN6_IS_ADDR_LINKLOCAL( &addr->addr.addr6 ) )
-        return TRUE;
-    return FALSE;
-}
-
-tr_bool
-tr_isValidPeerAddress( const tr_address * addr, tr_port port )
-{
-    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ) ||
-        isIPv4MappedOrCompatAddress( addr ) )
-        return FALSE;
-
-    if( port == 0 )
-        return FALSE;
-
-    return TRUE;
-}
-
 int
-tr_netOpenTCP( tr_session        * session,
-               const tr_address  * addr,
-               tr_port             port )
+tr_netOpenPeerSocket( tr_session        * session,
+                      const tr_address  * addr,
+                      tr_port             port,
+                      tr_bool             clientIsSeed )
 {
     static const int domains[NUM_TR_AF_INET_TYPES] = { AF_INET, AF_INET6 };
     int                     s;
@@ -300,15 +268,22 @@ tr_netOpenTCP( tr_session        * session,
 
     assert( tr_isAddress( addr ) );
 
-    if( isMulticastAddress( addr ) || isIPv6LinkLocalAddress( addr ) )
+    if( !tr_isValidPeerAddress( addr, port ) )
         return -EINVAL;
 
-    s = tr_fdSocketCreate( domains[addr->type], SOCK_STREAM );
+    s = tr_fdSocketCreate( session, domains[addr->type], SOCK_STREAM );
     if( s < 0 )
         return -1;
 
+    /* seeds don't need much of a read buffer... */
+    if( clientIsSeed ) {
+        int n = 8192;
+        if( setsockopt( s, SOL_SOCKET, SO_RCVBUF, &n, sizeof(n) ) )
+            tr_inf( "Unable to set SO_RCVBUF on socket %d: %s", s, tr_strerror( sockerrno ) );
+    }
+
     if( evutil_make_socket_nonblocking( s ) < 0 ) {
-        tr_netClose( s );
+        tr_netClose( session, s );
         return -1;
     }
 
@@ -339,7 +314,7 @@ tr_netOpenTCP( tr_session        * session,
             tr_err( _( "Couldn't connect socket %d to %s, port %d (errno %d - %s)" ),
                     s, tr_ntop_non_ts( addr ), (int)port, tmperrno,
                     tr_strerror( tmperrno ) );
-        tr_netClose( s );
+        tr_netClose( session, s );
         s = -tmperrno;
     }
 
@@ -368,7 +343,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
 
     if( evutil_make_socket_nonblocking( fd ) < 0 ) {
         *errOut = sockerrno;
-        EVUTIL_CLOSESOCKET( fd );
+        tr_netCloseSocket( fd );
         return -1;
     }
 
@@ -389,9 +364,23 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
     if( bind( fd, (struct sockaddr *) &sock, addrlen ) ) {
         const int err = sockerrno;
         if( !suppressMsgs )
-            tr_err( _( "Couldn't bind port %d on %s: %s" ),
-                    port, tr_ntop_non_ts( addr ), tr_strerror( err ) );
-        EVUTIL_CLOSESOCKET( fd );
+        {
+            const char * fmt;
+            const char * hint;
+
+            if( err == EADDRINUSE )
+                hint = _( "Is another copy of Transmission already running?" );
+            else
+                hint = NULL;
+
+            if( hint == NULL )
+                fmt = _( "Couldn't bind port %d on %s: %s" );
+            else
+                fmt = _( "Couldn't bind port %d on %s: %s (%s)" );
+
+            tr_err( fmt, port, tr_ntop_non_ts( addr ), tr_strerror( err ), hint );
+        }
+        tr_netCloseSocket( fd );
         *errOut = err;
         return -1;
     }
@@ -401,7 +390,7 @@ tr_netBindTCPImpl( const tr_address * addr, tr_port port, tr_bool suppressMsgs,
 
     if( listen( fd, 128 ) == -1 ) {
         *errOut = sockerrno;
-        EVUTIL_CLOSESOCKET( fd );
+        tr_netCloseSocket( fd );
         return -1;
     }
 
@@ -428,7 +417,7 @@ tr_net_hasIPv6( tr_port port )
         if( fd >= 0 || err != EAFNOSUPPORT ) /* we support ipv6 */
             result = TRUE;
         if( fd >= 0 )
-            EVUTIL_CLOSESOCKET( fd );
+            tr_netCloseSocket( fd );
         alreadyDone = TRUE;
     }
 
@@ -436,15 +425,15 @@ tr_net_hasIPv6( tr_port port )
 }
 
 int
-tr_netAccept( tr_session  * session UNUSED,
+tr_netAccept( tr_session  * session,
               int           b,
               tr_address  * addr,
               tr_port     * port )
 {
-    int fd = tr_fdSocketAccept( b, addr, port );
+    int fd = tr_fdSocketAccept( session, b, addr, port );
 
     if( fd>=0 && evutil_make_socket_nonblocking(fd)<0 ) {
-        tr_netClose( fd );
+        tr_netClose( session, fd );
         fd = -1;
     }
 
@@ -452,7 +441,227 @@ tr_netAccept( tr_session  * session UNUSED,
 }
 
 void
-tr_netClose( int s )
+tr_netCloseSocket( int fd )
 {
-    tr_fdSocketClose( s );
+    EVUTIL_CLOSESOCKET( fd );
+}
+
+void
+tr_netClose( tr_session * session, int s )
+{
+    tr_fdSocketClose( session, s );
+}
+
+/*
+   get_source_address() and global_unicast_address() were written by
+   Juliusz Chroboczek, and are covered under the same license as dht.c.
+   Please feel free to copy them into your software if it can help
+   unbreaking the double-stack Internet. */
+
+/* Get the source address used for a given destination address.  Since
+   there is no official interface to get this information, we create
+   a connected UDP socket (connected UDP... hmm...) and check its source
+   address. */
+static int
+get_source_address( const struct sockaddr  * dst,
+                    socklen_t                dst_len,
+                    struct sockaddr        * src,
+                    socklen_t              * src_len )
+{
+    int s, rc, save;
+
+    s = socket(dst->sa_family, SOCK_DGRAM, 0);
+    if(s < 0)
+        goto fail;
+
+    /* Since it's a UDP socket, this doesn't actually send any packets. */
+    rc = connect(s, dst, dst_len);
+    if(rc < 0)
+        goto fail;
+
+    rc = getsockname(s, src, src_len);
+    if(rc < 0)
+        goto fail;
+
+    EVUTIL_CLOSESOCKET( s );
+
+    return rc;
+
+ fail:
+    save = errno;
+    EVUTIL_CLOSESOCKET( s );
+    errno = save;
+    return -1;
+}
+
+/* We all hate NATs. */
+static int
+global_unicast_address(struct sockaddr *sa)
+{
+    if(sa->sa_family == AF_INET) {
+        const unsigned char *a =
+            (unsigned char*)&((struct sockaddr_in*)sa)->sin_addr;
+        if(a[0] == 0 || a[0] == 127 || a[0] >= 224 ||
+           a[0] == 10 || (a[0] == 172 && a[1] >= 16 && a[1] <= 31) ||
+           (a[0] == 192 && a[1] == 168))
+            return 0;
+        return 1;
+    } else if(sa->sa_family == AF_INET6) {
+        const unsigned char *a =
+            (unsigned char*)&((struct sockaddr_in6*)sa)->sin6_addr;
+        /* 2000::/3 */
+        return (a[0] & 0xE0) == 0x20;
+    } else {
+        errno = EAFNOSUPPORT;
+        return -1;
+    }
+}
+
+static int
+tr_globalAddress( int af, void *addr, int *addr_len )
+{
+    struct sockaddr_storage ss;
+    socklen_t sslen = sizeof(ss);
+    struct sockaddr_in sin;
+    struct sockaddr_in6 sin6;
+    struct sockaddr *sa;
+    socklen_t salen;
+    int rc;
+
+    switch(af) {
+    case AF_INET:
+        memset(&sin, 0, sizeof(sin));
+        sin.sin_family = AF_INET;
+        inet_pton(AF_INET, "91.121.74.28", &sin.sin_addr);
+        sin.sin_port = htons(6969);
+        sa = (struct sockaddr*)&sin;
+        salen = sizeof(sin);
+        break;
+    case AF_INET6:
+        memset(&sin6, 0, sizeof(sin6));
+        sin6.sin6_family = AF_INET6;
+        /* In order for address selection to work right, this should be
+           a native IPv6 address, not Teredo or 6to4. */
+        inet_pton(AF_INET6, "2001:1890:1112:1::20", &sin6.sin6_addr);
+        sin6.sin6_port = htons(6969);
+        sa = (struct sockaddr*)&sin6;
+        salen = sizeof(sin6);
+        break;
+    default:
+        return -1;
+    }
+
+    rc = get_source_address( sa, salen, (struct sockaddr*)&ss, &sslen );
+
+    if( rc < 0 )
+        return -1;
+
+    if( !global_unicast_address( (struct sockaddr*)&ss) )
+        return -1;
+
+    switch(af) {
+    case AF_INET:
+        if(*addr_len < 4)
+            return -1;
+        memcpy(addr, &((struct sockaddr_in*)&ss)->sin_addr, 4);
+        *addr_len = 4;
+        return 1;
+    case AF_INET6:
+        if(*addr_len < 16)
+            return -1;
+        memcpy(addr, &((struct sockaddr_in6*)&ss)->sin6_addr, 16);
+        *addr_len = 16;
+        return 1;
+    default:
+        return -1;
+    }
+}
+
+/* Return our global IPv6 address, with caching. */
+
+const unsigned char *
+tr_globalIPv6( void )
+{
+    static unsigned char ipv6[16];
+    static time_t last_time = 0;
+    static int have_ipv6 = 0;
+    const time_t now = tr_time( );
+
+    /* Re-check every half hour */
+    if( last_time < now - 1800 )
+    {
+        int addrlen = 16;
+        const int rc = tr_globalAddress( AF_INET6, ipv6, &addrlen );
+        have_ipv6 = ( rc >= 0 ) && ( addrlen == 16 );
+        last_time = now;
+    }
+
+    return have_ipv6 ? ipv6 : NULL;
+}
+
+/***
+****
+****
+***/
+
+static tr_bool
+isIPv4MappedAddress( const tr_address * addr )
+{
+    return ( addr->type == TR_AF_INET6 ) && IN6_IS_ADDR_V4MAPPED( &addr->addr.addr6 );
+}
+
+static tr_bool
+isIPv6LinkLocalAddress( const tr_address * addr )
+{
+    return ( ( addr->type == TR_AF_INET6 )
+                  && IN6_IS_ADDR_LINKLOCAL( &addr->addr.addr6 ) );
+}
+
+/* isMartianAddr was written by Juliusz Chroboczek,
+   and is covered under the same license as third-party/dht/dht.c. */
+static tr_bool
+isMartianAddr( const struct tr_address * a )
+{
+    static const unsigned char zeroes[16] =
+        { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+
+    assert( tr_isAddress( a ) );
+
+    switch( a->type )
+    {
+        case TR_AF_INET: {
+            const unsigned char * address = (const unsigned char*)&a->addr.addr4;
+            return (address[0] == 0) ||
+                   (address[0] == 127) ||
+                   ((address[0] & 0xE0) == 0xE0);
+            break;
+        }
+
+        case TR_AF_INET6: {
+            const unsigned char * address = (const unsigned char*)&a->addr.addr6;
+            return (address[0] == 0xFF) ||
+                   (memcmp(address, zeroes, 15) == 0 &&
+                    (address[15] == 0 || address[15] == 1)) ||
+                   /* Addresses outside of 2000::/3 are currently reserved,
+                      but might be allocated at some future time.  Since
+                      there are a lot of buggy peers pushing around such
+                      addresses over PEX, we reject them until the end of
+                      the 13th Baktun. */
+                   (tr_time() < 1356130800 && (address[0] & 0xE0) != 0x20);
+            break;
+        }
+
+        default:
+            return TRUE;
+    }
+}
+
+tr_bool
+tr_isValidPeerAddress( const tr_address * addr, tr_port port )
+{
+    return ( port != 0 )
+        && ( tr_isAddress( addr ) )
+        && ( !isIPv6LinkLocalAddress( addr ) )
+        && ( !isIPv4MappedAddress( addr ) )
+        && ( !isMartianAddr( addr ) );
 }
