@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-io.c 8889 2009-08-10 20:04:08Z charles $
+ * $Id: peer-io.c 10694 2010-05-26 15:23:21Z charles $
  */
 
 #include <assert.h>
@@ -31,9 +31,9 @@
 #include "crypto.h"
 #include "list.h"
 #include "net.h"
+#include "peer-common.h" /* MAX_BLOCK_SIZE */
 #include "peer-io.h"
-#include "platform.h" /* MAX_STACK_ARRAY_SIZE */
-#include "trevent.h"
+#include "trevent.h" /* tr_runInEventThread() */
 #include "utils.h"
 
 #define MAGIC_NUMBER 206745
@@ -57,7 +57,7 @@ guessPacketOverhead( size_t d )
      *  (1500-72)/(38+1500) = 92.8479 %  IPv6, TCP timestamps
      *  (1500-72)/(42+1500) = 92.6070 %  802.1q, IPv6, ICP timestamps
      */
-    static const double assumed_payload_data_rate = 94.0;
+    const double assumed_payload_data_rate = 94.0;
 
     return (size_t)( d * ( 100.0 / assumed_payload_data_rate ) - d );
 }
@@ -76,7 +76,6 @@ struct tr_datatype
 {
     tr_bool  isPieceData;
     size_t   length;
-    struct __tr_list head;
 };
 
 /***
@@ -88,7 +87,8 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 {
      while( bytes_transferred && tr_isPeerIo( io ) )
      {
-        struct tr_datatype * next = __tr_list_entry( io->outbuf_datatypes.next, struct tr_datatype, head );
+        struct tr_datatype * next = io->outbuf_datatypes->data;
+
         const size_t payload = MIN( next->length, bytes_transferred );
         const size_t overhead = guessPacketOverhead( payload );
 
@@ -105,7 +105,7 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
             bytes_transferred -= payload;
             next->length -= payload;
             if( !next->length ) {
-                __tr_list_remove( io->outbuf_datatypes.next );
+                tr_list_pop_front( &io->outbuf_datatypes );
                 tr_free( next );
             }
         }
@@ -115,18 +115,22 @@ didWriteWrapper( tr_peerIo * io, size_t bytes_transferred )
 static void
 canReadWrapper( tr_peerIo * io )
 {
-    tr_bool done = 0;
     tr_bool err = 0;
-    tr_session * session = io->session;
+    tr_bool done = 0;
+    tr_session * session;
 
     dbgmsg( io, "canRead" );
 
+    assert( tr_isPeerIo( io ) );
+    assert( tr_isSession( io->session ) );
     tr_peerIoRef( io );
 
+    session = io->session;
+
     /* try to consume the input buffer */
     if( io->canRead )
     {
-        tr_globalLock( session );
+        tr_sessionLock( session );
 
         while( !done && !err )
         {
@@ -160,9 +164,11 @@ canReadWrapper( tr_peerIo * io )
                     err = 1;
                     break;
             }
+
+            assert( tr_isPeerIo( io ) );
         }
 
-        tr_globalUnlock( session );
+        tr_sessionUnlock( session );
     }
 
     /* keep the iobuf's excess capacity from growing too large */
@@ -171,6 +177,7 @@ canReadWrapper( tr_peerIo * io )
         io->inbuf = evbuffer_new( );
     }
 
+    assert( tr_isPeerIo( io ) );
     tr_peerIoUnref( io );
 }
 
@@ -350,35 +357,54 @@ event_write_cb( int fd, short event UNUSED, void * vio )
 ***
 **/
 
+static void
+maybeSetCongestionAlgorithm( int socket, const char * algorithm )
+{
+    if( algorithm && *algorithm )
+    {
+        const int rc = tr_netSetCongestionControl( socket, algorithm );
+
+        if( rc < 0 )
+            tr_ninf( "Net", "Can't set congestion control algorithm '%s': %s",
+                     algorithm, tr_strerror( errno ));
+    }
+}
+
 static tr_peerIo*
 tr_peerIoNew( tr_session       * session,
               tr_bandwidth     * parent,
               const tr_address * addr,
               tr_port            port,
               const uint8_t    * torrentHash,
-              int                isIncoming,
+              tr_bool            isIncoming,
+              tr_bool            isSeed,
               int                socket )
 {
     tr_peerIo * io;
 
     assert( session != NULL );
     assert( session->events != NULL );
+    assert( tr_isBool( isIncoming ) );
+    assert( tr_isBool( isSeed ) );
     assert( tr_amInEventThread( session ) );
 
-    if( socket >= 0 )
+    if( socket >= 0 ) {
         tr_netSetTOS( socket, session->peerSocketTOS );
-
+        maybeSetCongestionAlgorithm( socket, session->peer_congestion_algorithm );
+    }
+    
     io = tr_new0( tr_peerIo, 1 );
     io->magicNumber = MAGIC_NUMBER;
     io->refCount = 1;
     io->crypto = tr_cryptoNew( torrentHash, isIncoming );
     io->session = session;
     io->addr = *addr;
+    io->isSeed = isSeed;
     io->port = port;
     io->socket = socket;
     io->isIncoming = isIncoming != 0;
     io->hasFinishedConnecting = FALSE;
-    io->timeCreated = time( NULL );
+    io->timeCreated = tr_time( );
     io->inbuf = evbuffer_new( );
     io->outbuf = evbuffer_new( );
     tr_bandwidthConstruct( &io->bandwidth, session, parent );
@@ -388,8 +414,6 @@ tr_peerIoNew( tr_session       * session,
     event_set( &io->event_read, io->socket, EV_READ, event_read_cb, io );
     event_set( &io->event_write, io->socket, EV_WRITE, event_write_cb, io );
 
-    __tr_list_init( &io->outbuf_datatypes );
-
     return io;
 }
 
@@ -398,13 +422,13 @@ tr_peerIoNewIncoming( tr_session        * session,
                       tr_bandwidth      * parent,
                       const tr_address  * addr,
                       tr_port             port,
-                      int                 socket )
+                      int                 fd )
 {
     assert( session );
     assert( tr_isAddress( addr ) );
-    assert( socket >= 0 );
+    assert( fd >= 0 );
 
-    return tr_peerIoNew( session, parent, addr, port, NULL, 1, socket );
+    return tr_peerIoNew( session, parent, addr, port, NULL, TRUE, FALSE, fd );
 }
 
 tr_peerIo*
@@ -412,29 +436,99 @@ tr_peerIoNewOutgoing( tr_session        * session,
                       tr_bandwidth      * parent,
                       const tr_address  * addr,
                       tr_port             port,
-                      const uint8_t     * torrentHash )
+                      const uint8_t     * torrentHash,
+                      tr_bool             isSeed )
 {
-    int socket;
+    int fd;
 
     assert( session );
     assert( tr_isAddress( addr ) );
     assert( torrentHash );
 
-    socket = tr_netOpenTCP( session, addr, port );
-    dbgmsg( NULL, "tr_netOpenTCP returned fd %d", socket );
+    fd = tr_netOpenPeerSocket( session, addr, port, isSeed );
+    dbgmsg( NULL, "tr_netOpenPeerSocket returned fd %d", fd );
 
-    return socket < 0
-           ? NULL
-           : tr_peerIoNew( session, parent, addr, port, torrentHash, 0, socket );
+    return fd < 0 ? NULL
+                  : tr_peerIoNew( session, parent, addr, port, torrentHash, FALSE, isSeed, fd );
+}
+
+/***
+****
+***/
+
+static void
+event_enable( tr_peerIo * io, short event )
+{
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session != NULL );
+    assert( io->session->events != NULL );
+    assert( event_initialized( &io->event_read ) );
+    assert( event_initialized( &io->event_write ) );
+
+    if( io->socket < 0 )
+        return;
+
+    if( ( event & EV_READ ) && ! ( io->pendingEvents & EV_READ ) )
+    {
+        dbgmsg( io, "enabling libevent ready-to-read polling" );
+        event_add( &io->event_read, NULL );
+        io->pendingEvents |= EV_READ;
+    }
+
+    if( ( event & EV_WRITE ) && ! ( io->pendingEvents & EV_WRITE ) )
+    {
+        dbgmsg( io, "enabling libevent ready-to-write polling" );
+        event_add( &io->event_write, NULL );
+        io->pendingEvents |= EV_WRITE;
+    }
 }
 
 static void
-trDatatypeFree( void * data )
+event_disable( struct tr_peerIo * io, short event )
 {
-    struct tr_datatype * dt = __tr_list_entry( data, struct tr_datatype, head );
-    tr_free(dt);
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session != NULL );
+    assert( io->session->events != NULL );
+    assert( event_initialized( &io->event_read ) );
+    assert( event_initialized( &io->event_write ) );
+
+    if( ( event & EV_READ ) && ( io->pendingEvents & EV_READ ) )
+    {
+        dbgmsg( io, "disabling libevent ready-to-read polling" );
+        event_del( &io->event_read );
+        io->pendingEvents &= ~EV_READ;
+    }
+
+    if( ( event & EV_WRITE ) && ( io->pendingEvents & EV_WRITE ) )
+    {
+        dbgmsg( io, "disabling libevent ready-to-write polling" );
+        event_del( &io->event_write );
+        io->pendingEvents &= ~EV_WRITE;
+    }
 }
 
+void
+tr_peerIoSetEnabled( tr_peerIo    * io,
+                     tr_direction   dir,
+                     tr_bool        isEnabled )
+{
+    const short event = dir == TR_UP ? EV_WRITE : EV_READ;
+
+    assert( tr_isPeerIo( io ) );
+    assert( tr_isDirection( dir ) );
+    assert( tr_amInEventThread( io->session ) );
+    assert( io->session->events != NULL );
+
+    if( isEnabled )
+        event_enable( io, event );
+    else
+        event_disable( io, event );
+}
+
+/***
+****
+***/
+
 static void
 io_dtor( void * vio )
 {
@@ -445,14 +539,13 @@ io_dtor( void * vio )
     assert( io->session->events != NULL );
 
     dbgmsg( io, "in tr_peerIo destructor" );
-    event_del( &io->event_read );
-    event_del( &io->event_write );
+    event_disable( io, EV_READ | EV_WRITE );
     tr_bandwidthDestruct( &io->bandwidth );
     evbuffer_free( io->outbuf );
     evbuffer_free( io->inbuf );
-    tr_netClose( io->socket );
+    tr_netClose( io->session, io->socket );
     tr_cryptoFree( io->crypto );
-    __tr_list_destroy( &io->outbuf_datatypes, trDatatypeFree );
+    tr_list_free( &io->outbuf_datatypes, tr_free );
 
     memset( io, ~0, sizeof( tr_peerIo ) );
     tr_free( io );
@@ -541,15 +634,29 @@ tr_peerIoClear( tr_peerIo * io )
 int
 tr_peerIoReconnect( tr_peerIo * io )
 {
+    int pendingEvents;
+    tr_session * session;
+
+    assert( tr_isPeerIo( io ) );
     assert( !tr_peerIoIsIncoming( io ) );
 
+    session = tr_peerIoGetSession( io );
+
+    pendingEvents = io->pendingEvents;
+    event_disable( io, EV_READ | EV_WRITE );
+
     if( io->socket >= 0 )
-        tr_netClose( io->socket );
+        tr_netClose( session, io->socket );
+
+    io->socket = tr_netOpenPeerSocket( session, &io->addr, io->port, io->isSeed );
+    event_set( &io->event_read, io->socket, EV_READ, event_read_cb, io );
+    event_set( &io->event_write, io->socket, EV_WRITE, event_write_cb, io );
 
-    io->socket = tr_netOpenTCP( io->session, &io->addr, io->port );
     if( io->socket >= 0 )
     {
-        tr_netSetTOS( io->socket, io->session->peerSocketTOS );
+        event_enable( io, pendingEvents );
+        tr_netSetTOS( io->socket, session->peerSocketTOS );
+        maybeSetCongestionAlgorithm( io->socket, session->peer_congestion_algorithm );
         return 0;
     }
 
@@ -607,42 +714,6 @@ tr_peerIoSetPeersId( tr_peerIo *     io,
 ***
 **/
 
-void
-tr_peerIoEnableFEXT( tr_peerIo * io,
-                     tr_bool     flag )
-{
-    assert( tr_isPeerIo( io ) );
-    assert( tr_isBool( flag ) );
-
-    dbgmsg( io, "setting FEXT support flag to %d", (flag!=0) );
-    io->fastExtensionSupported = flag;
-}
-
-void
-tr_peerIoEnableLTEP( tr_peerIo  * io,
-                     tr_bool      flag )
-{
-    assert( tr_isPeerIo( io ) );
-    assert( tr_isBool( flag ) );
-
-    dbgmsg( io, "setting LTEP support flag to %d", (flag!=0) );
-    io->extendedProtocolSupported = flag;
-}
-
-void
-tr_peerIoEnableDHT( tr_peerIo * io, tr_bool flag )
-{
-    assert( tr_isPeerIo( io ) );
-    assert( tr_isBool( flag ) );
-
-    dbgmsg( io, "setting DHT support flag to %d", (flag!=0) );
-    io->dhtSupported = flag;
-}
-
-/**
-***
-**/
-
 static size_t
 getDesiredOutputBufferSize( const tr_peerIo * io, uint64_t now )
 {
@@ -650,11 +721,10 @@ getDesiredOutputBufferSize( const tr_peerIo * io, uint64_t now )
      * being large enough to hold the next 20 seconds' worth of input,
      * or a few blocks, whichever is bigger.
      * It's okay to tweak this as needed */
-    const double maxBlockSize = 16 * 1024; /* 16 KiB is from BT spec */
     const double currentSpeed = tr_bandwidthGetPieceSpeed( &io->bandwidth, now, TR_UP );
     const double period = 15; /* arbitrary */
     const double numBlocks = 3.5; /* the 3 is arbitrary; the .5 is to leave room for messages */
-    return MAX( maxBlockSize*numBlocks, currentSpeed*1024*period );
+    return MAX( MAX_BLOCK_SIZE*numBlocks, currentSpeed*1024*period );
 }
 
 size_t
@@ -704,26 +774,26 @@ tr_peerIoWrite( tr_peerIo   * io,
     datatype = tr_new( struct tr_datatype, 1 );
     datatype->isPieceData = isPieceData != 0;
     datatype->length = byteCount;
-
-    __tr_list_init( &datatype->head );
-    __tr_list_append( &io->outbuf_datatypes, &datatype->head );
+    tr_list_append( &io->outbuf_datatypes, datatype );
 
     switch( io->encryptionMode )
     {
         case PEER_ENCRYPTION_RC4:
         {
             /* FIXME(libevent2): use evbuffer_reserve_space() and evbuffer_commit_space() instead of tmp */
-            uint8_t tmp[MAX_STACK_ARRAY_SIZE];
+            void * tmp = tr_sessionGetBuffer( io->session );
+            const size_t tmplen = SESSION_BUFFER_SIZE;
             const uint8_t * walk = bytes;
             evbuffer_expand( io->outbuf, byteCount );
             while( byteCount > 0 )
             {
-                const size_t thisPass = MIN( byteCount, sizeof( tmp ) );
+                const size_t thisPass = MIN( byteCount, tmplen );
                 tr_cryptoEncrypt( io->crypto, thisPass, walk, tmp );
                 evbuffer_add( io->outbuf, tmp, thisPass );
                 walk += thisPass;
                 byteCount -= thisPass;
             }
+            tr_sessionReleaseBuffer( io->session );
             break;
         }
 
@@ -784,14 +854,17 @@ tr_peerIoDrain( tr_peerIo       * io,
                 struct evbuffer * inbuf,
                 size_t            byteCount )
 {
-    uint8_t tmp[MAX_STACK_ARRAY_SIZE];
+    void * buf = tr_sessionGetBuffer( io->session );
+    const size_t buflen = SESSION_BUFFER_SIZE;
 
     while( byteCount > 0 )
     {
-        const size_t thisPass = MIN( byteCount, sizeof( tmp ) );
-        tr_peerIoReadBytes( io, inbuf, tmp, thisPass );
+        const size_t thisPass = MIN( byteCount, buflen );
+        tr_peerIoReadBytes( io, inbuf, buf, thisPass );
         byteCount -= thisPass;
     }
+
+    tr_sessionReleaseBuffer( io->session );
 }
 
 /***
@@ -847,7 +920,9 @@ tr_peerIoTryWrite( tr_peerIo * io, size_t howmuch )
         {
             const short what = EVBUFFER_WRITE | EVBUFFER_ERROR;
             dbgmsg( io, "tr_peerIoTryWrite got an error. res is %d, what is %hd, errno is %d (%s)", n, what, e, strerror( e ) );
-            io->gotError( io, what, io->userData );
+
+            if( io->gotError != NULL )
+                io->gotError( io, what, io->userData );
         }
     }
 
@@ -878,89 +953,19 @@ int
 tr_peerIoFlushOutgoingProtocolMsgs( tr_peerIo * io )
 {
     size_t byteCount = 0;
-    struct __tr_list * walk;
-    struct __tr_list * fencepost = &io->outbuf_datatypes;
+    tr_list * it;
 
     /* count up how many bytes are used by non-piece-data messages
        at the front of our outbound queue */
-    for( walk=fencepost->next; walk!=fencepost; walk=walk->next ) {
-        struct tr_datatype * d = __tr_list_entry( walk, struct tr_datatype, head );
+    for( it=io->outbuf_datatypes; it!=NULL; it=it->next )
+    {
+        struct tr_datatype * d = it->data;
+
         if( d->isPieceData )
             break;
+
         byteCount += d->length;
     }
 
     return tr_peerIoFlush( io, TR_UP, byteCount );
 }
-
-
-/***
-****
-****/
-
-static void
-event_enable( tr_peerIo * io, short event )
-{
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session != NULL );
-    assert( io->session->events != NULL );
-    assert( event_initialized( &io->event_read ) );
-    assert( event_initialized( &io->event_write ) );
-
-    if( ( event & EV_READ ) && ! ( io->pendingEvents & EV_READ ) )
-    {
-        dbgmsg( io, "enabling libevent ready-to-read polling" );
-        event_add( &io->event_read, NULL );
-        io->pendingEvents |= EV_READ;
-    }
-
-    if( ( event & EV_WRITE ) && ! ( io->pendingEvents & EV_WRITE ) )
-    {
-        dbgmsg( io, "enabling libevent ready-to-write polling" );
-        event_add( &io->event_write, NULL );
-        io->pendingEvents |= EV_WRITE;
-    }
-}
-
-static void
-event_disable( struct tr_peerIo * io, short event )
-{
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session != NULL );
-    assert( io->session->events != NULL );
-    assert( event_initialized( &io->event_read ) );
-    assert( event_initialized( &io->event_write ) );
-
-    if( ( event & EV_READ ) && ( io->pendingEvents & EV_READ ) )
-    {
-        dbgmsg( io, "disabling libevent ready-to-read polling" );
-        event_del( &io->event_read );
-        io->pendingEvents &= ~EV_READ;
-    }
-
-    if( ( event & EV_WRITE ) && ( io->pendingEvents & EV_WRITE ) )
-    {
-        dbgmsg( io, "disabling libevent ready-to-write polling" );
-        event_del( &io->event_write );
-        io->pendingEvents &= ~EV_WRITE;
-    }
-}
-
-
-void
-tr_peerIoSetEnabled( tr_peerIo    * io,
-                     tr_direction   dir,
-                     tr_bool        isEnabled )
-{
-    const short event = dir == TR_UP ? EV_WRITE : EV_READ;
-
-    assert( tr_isPeerIo( io ) );
-    assert( tr_isDirection( dir ) );
-    assert( tr_amInEventThread( io->session ) );
-    assert( io->session->events != NULL );
-
-    if( isEnabled )
-        event_enable( io, event );
-    else
-        event_disable( io, event );
-}
