@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2007-2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2007-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: peer-mgr.h 9086 2009-09-10 02:21:03Z charles $
+ * $Id: peer-mgr.h 11471 2010-12-05 18:43:42Z charles $
  */
 
 #ifndef __TRANSMISSION__
@@ -24,9 +24,11 @@
 #endif
 
 #include "bitfield.h"
+#include "bitset.h"
+#include "history.h"
 #include "net.h"
+#include "peer-common.h" /* struct peer_request */
 #include "publish.h" /* tr_publisher_tag */
-#include "utils.h"
 
 /**
  * @addtogroup peers Peers
@@ -49,7 +51,7 @@ enum
 typedef struct tr_pex
 {
     tr_address addr;
-    tr_port    port;
+    tr_port    port; /* this field is in network byte order */
     uint8_t    flags;
 }
 tr_pex;
@@ -87,14 +89,19 @@ typedef struct tr_peer
     uint8_t                  strikes;
 
     uint8_t                  encryption_preference;
-    tr_port                  port;
     tr_port                  dht_port;
-    tr_address               addr;
+
+    /* how many requests the peer has made that we haven't responded to yet */
+    int                      pendingReqsToClient;
+
+    /* how many requests we've made and are currently awaiting a response for */
+    int                      pendingReqsToPeer;
+
     struct tr_peerIo       * io;
     struct peer_atom       * atom;
 
     struct tr_bitfield     * blame;
-    struct tr_bitfield     * have;
+    struct tr_bitset         have;
 
     /** how complete the peer's copy of the torrent is. [0.0...1.0] */
     float                    progress;
@@ -104,11 +111,18 @@ typedef struct tr_peer
 
     time_t                   chokeChangedAt;
 
+    tr_recentHistory       * blocksSentToClient;
+    tr_recentHistory       * blocksSentToPeer;
+
+    tr_recentHistory       * cancelsSentToClient;
+    tr_recentHistory       * cancelsSentToPeer;
+
     struct tr_peermsgs     * msgs;
     tr_publisher_tag         msgsTag;
 }
 tr_peer;
 
+const tr_address * tr_peerAddress( const tr_peer * );
 
 int tr_pexCompare( const void * a, const void * b );
 
@@ -119,6 +133,18 @@ void tr_peerMgrFree( tr_peerMgr * manager );
 tr_bool tr_peerMgrPeerIsSeed( const tr_torrent * tor,
                               const tr_address * addr );
 
+void tr_peerMgrGetNextRequests( tr_torrent          * torrent,
+                                tr_peer             * peer,
+                                int                   numwant,
+                                tr_block_index_t    * setme,
+                                int                 * numgot );
+
+tr_bool tr_peerMgrDidPeerRequest( const tr_torrent  * torrent,
+                                  const tr_peer     * peer,
+                                  tr_block_index_t    block );
+
+void tr_peerMgrRebuildRequests( tr_torrent * torrent );
+
 void tr_peerMgrAddIncoming( tr_peerMgr  * manager,
                             tr_address  * addr,
                             tr_port       port,
@@ -140,18 +166,31 @@ tr_pex * tr_peerMgrArrayToPex( const void * array,
                                size_t       arrayLen,
                                size_t      * setme_pex_count );
 
+/**
+ * @param seedProbability [0..100] for likelihood that the peer is a seed; -1 for unknown
+ */
 void tr_peerMgrAddPex( tr_torrent     * tor,
                        uint8_t          from,
-                       const tr_pex   * pex );
+                       const tr_pex   * pex,
+                       int8_t           seedProbability );
+
+void tr_peerMgrMarkAllAsSeeds( tr_torrent * tor );
 
 void tr_peerMgrSetBlame( tr_torrent        * tor,
                          tr_piece_index_t    pieceIndex,
                          int                 success );
 
+enum
+{
+    TR_PEERS_CONNECTED,
+    TR_PEERS_ALL
+};
+
 int  tr_peerMgrGetPeers( tr_torrent      * tor,
                          tr_pex         ** setme_pex,
-                         uint8_t           af,
-                         int               maxPeerCount );
+                         uint8_t           address_type,
+                         uint8_t           peer_list_mode,
+                         int               max_peer_count );
 
 void tr_peerMgrStartTorrent( tr_torrent * tor );
 
@@ -168,6 +207,8 @@ void tr_peerMgrTorrentAvailability( const tr_torrent * tor,
 
 struct tr_bitfield* tr_peerMgrGetAvailable( const tr_torrent * tor );
 
+void tr_peerMgrOnBlocklistChanged( tr_peerMgr * manager );
+
 void tr_peerMgrTorrentStats( tr_torrent * tor,
                              int * setmePeersKnown,
                              int * setmePeersConnected,
@@ -189,6 +230,8 @@ double tr_peerGetPieceSpeed( const tr_peer    * peer,
                              uint64_t           now,
                              tr_direction       direction );
 
+void tr_peerMgrClearInterest( tr_torrent * tor );
+
 /* @} */
 
 #endif
