@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: CreatorWindowController.h 7659 2009-01-10 23:37:37Z livings124 $
+ * $Id: CreatorWindowController.h 10514 2010-04-23 02:13:54Z livings124 $
  *
- * Copyright (c) 2007-2009 Transmission authors and contributors
+ * Copyright (c) 2007-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,7 +41,6 @@
     tr_metainfo_builder * fInfo;
     NSString * fPath, * fLocation;
     NSMutableArray * fTrackers;
-    BOOL fOpenTorrent;
     
     NSTimer * fTimer;
     BOOL fStarted;
@@ -54,8 +53,6 @@
 
 - (id) initWithHandle: (tr_session *) handle path: (NSString *) path;
 
-- (void) toggleOpenCheck: (id) sender;
-
 - (void) setLocation: (id) sender;
 - (void) create: (id) sender;
 - (void) cancelCreateWindow: (id) sender;
@@ -63,4 +60,7 @@
 
 - (void) addRemoveTracker: (id) sender;
 
+- (void) copy: (id) sender;
+- (void) paste: (id) sender;
+
 @end
