@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.h 8461 2009-05-21 04:35:40Z livings124 $
+ * $Id: PrefsController.h 10666 2010-05-19 23:08:14Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -84,6 +84,7 @@
 
 - (void) setPEX: (id) sender;
 - (void) setDHT: (id) sender;
+- (void) setLPD: (id) sender;
 
 - (void) setEncryptionMode: (id) sender;
 
@@ -92,9 +93,14 @@
 - (void) setBlocklistAutoUpdate: (id) sender;
 - (void) updateBlocklistFields;
 
+- (void) setAutoStartDownloads: (id) sender;
+
 - (void) setBadge: (id) sender;
+
 - (void) resetWarnings: (id) sender;
 
+- (void) setDefaultForMagnets: (id) sender;
+
 - (void) setQueue: (id) sender;
 - (void) setQueueNumber: (id) sender;
 
@@ -104,6 +110,9 @@
 - (void) setDownloadLocation: (id) sender;
 - (void) folderSheetShow: (id) sender;
 - (void) incompleteFolderSheetShow: (id) sender;
+- (void) setUseIncompleteFolder: (id) sender;
+
+- (void) setRenamePartialFiles: (id) sender;
 
 - (void) applyRatioSetting: (id) sender;
 - (void) updateRatioStopField;
