@@ -1,7 +1,7 @@
 /******************************************************************************
- * $Id: PrefsController.m 9004 2009-08-29 00:44:06Z livings124 $
+ * $Id: PrefsController.m 11105 2010-08-03 23:57:11Z livings124 $
  *
- * Copyright (c) 2005-2009 Transmission authors and contributors
+ * Copyright (c) 2005-2010 Transmission authors and contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -27,8 +27,11 @@
 #import "BlocklistScheduler.h"
 #import "PortChecker.h"
 #import "BonjourController.h"
+#import "NSApplicationAdditions.h"
 #import "NSStringAdditions.h"
 #import "UKKQueue.h"
+
+#import "transmission.h"
 #import "utils.h"
 
 #import <Sparkle/Sparkle.h>
@@ -206,7 +209,7 @@ tr_session * fHandle;
     
     //set proxy type
     [fProxyAddressField setStringValue: [fDefaults stringForKey: @"ProxyAddress"]];
-    int proxyType;
+    NSInteger proxyType;
     switch (tr_sessionGetProxyType(fHandle))
     {
         case TR_PROXY_SOCKS4:
@@ -217,6 +220,9 @@ tr_session * fHandle;
             break;
         case TR_PROXY_HTTP:
             proxyType = PROXY_HTTP;
+            break;
+        default:
+            NSAssert(NO, @"Unknown proxy type received");
     }
     [fProxyTypePopUp selectItemAtIndex: proxyType];
     
@@ -308,9 +314,15 @@ tr_session * fHandle;
     return [item autorelease];
 }
 
+- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
+{
+    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_GROUPS, TOOLBAR_BANDWIDTH,
+                                        TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
+}
+
 - (NSArray *) toolbarSelectableItemIdentifiers: (NSToolbar *) toolbar
 {
-    return [self toolbarDefaultItemIdentifiers: toolbar];
+    return [self toolbarAllowedItemIdentifiers: toolbar];
 }
 
 - (NSArray *) toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar
@@ -318,12 +330,6 @@ tr_session * fHandle;
     return [self toolbarAllowedItemIdentifiers: toolbar];
 }
 
-- (NSArray *) toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar
-{
-    return [NSArray arrayWithObjects: TOOLBAR_GENERAL, TOOLBAR_TRANSFERS, TOOLBAR_GROUPS, TOOLBAR_BANDWIDTH,
-                                        TOOLBAR_PEERS, TOOLBAR_NETWORK, TOOLBAR_REMOTE, nil];
-}
-
 //for a beta release, always use the beta appcast
 #if defined(TR_BETA_RELEASE)
 #define SPARKLE_TAG YES
@@ -337,7 +343,7 @@ tr_session * fHandle;
 
 - (void) setPort: (id) sender
 {
-    const int port = [sender intValue];
+    const tr_port port = [sender intValue];
     [fDefaults setInteger: port forKey: @"BindPort"];
     tr_sessionSetPeerPort(fHandle, port);
     
@@ -348,9 +354,11 @@ tr_session * fHandle;
 - (void) randomPort: (id) sender
 {
     const tr_port port = tr_sessionSetPeerPortRandom(fHandle);
-    
+    [fDefaults setInteger: port forKey: @"BindPort"];
     [fPortField setIntValue: port];
-    [self setPort: fPortField];
+    
+    fPeerPort = -1;
+    [self updatePortStatus];
 }
 
 - (void) setRandomPortOnStart: (id) sender
@@ -418,10 +426,13 @@ tr_session * fHandle;
 {
     NSMutableArray * sounds = [NSMutableArray array];
     
-    NSArray * directories = [NSArray arrayWithObjects: @"/System/Library/Sounds", @"/Library/Sounds", @"Library/Sounds", nil];
+    NSArray * directories = NSSearchPathForDirectoriesInDomains(NSAllLibrariesDirectory,
+                                NSUserDomainMask | NSLocalDomainMask | NSSystemDomainMask, YES);
     
     for (NSString * directory in directories)
     {
+        directory = [directory stringByAppendingPathComponent: @"Sounds"];
+        
         BOOL isDirectory;
         if ([[NSFileManager defaultManager] fileExistsAtPath: directory isDirectory: &isDirectory] && isDirectory)
         {
@@ -470,6 +481,11 @@ tr_session * fHandle;
     tr_sessionSetDHTEnabled(fHandle, [fDefaults boolForKey: @"DHTGlobal"]);
 }
 
+- (void) setLPD: (id) sender
+{
+    tr_sessionSetLPDEnabled(fHandle, [fDefaults boolForKey: @"LocalPeerDiscoveryGlobal"]);
+}
+
 - (void) setEncryptionMode: (id) sender
 {
     const tr_encryption_mode mode = [fDefaults boolForKey: @"EncryptionPrefer"] ? 
@@ -524,12 +540,18 @@ tr_session * fHandle;
         NSDate * updatedDate = [fDefaults objectForKey: @"BlocklistLastUpdate"];
         if (updatedDate)
         {
-            NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
-            [dateFormatter setDateStyle: NSDateFormatterFullStyle];
-            [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
-            
-            updatedDateString = [dateFormatter stringFromDate: updatedDate];
-            [dateFormatter release];
+            if ([NSApp isOnSnowLeopardOrBetter])
+                updatedDateString = [NSDateFormatter localizedStringFromDate: updatedDate dateStyle: NSDateFormatterFullStyle
+                                        timeStyle: NSDateFormatterShortStyle];
+            else
+            {
+                NSDateFormatter * dateFormatter = [[NSDateFormatter alloc] init];
+                [dateFormatter setDateStyle: NSDateFormatterFullStyle];
+                [dateFormatter setTimeStyle: NSDateFormatterShortStyle];
+                
+                updatedDateString = [dateFormatter stringFromDate: updatedDate];
+                [dateFormatter release];
+            }
         }
         else
             updatedDateString = NSLocalizedString(@"N/A", "Prefs -> blocklist -> message");
@@ -541,6 +563,11 @@ tr_session * fHandle;
         NSLocalizedString(@"Last updated", "Prefs -> blocklist -> message"), updatedDateString]];
 }
 
+- (void) setAutoStartDownloads: (id) sender
+{
+    tr_sessionSetPaused(fHandle, ![fDefaults boolForKey: @"AutoStartDownload"]);
+}
+
 - (void) applySpeedSettings: (id) sender
 {
     tr_sessionLimitSpeed(fHandle, TR_UP, [fDefaults boolForKey: @"CheckUpload"]);
@@ -668,9 +695,19 @@ tr_session * fHandle;
     [fDefaults removeObjectForKey: @"WarningFolderDataSameName"];
     [fDefaults removeObjectForKey: @"WarningResetStats"];
     [fDefaults removeObjectForKey: @"WarningCreatorBlankAddress"];
-    [fDefaults removeObjectForKey: @"WarningRemoveBuiltInTracker"];
+    [fDefaults removeObjectForKey: @"WarningCreatorPrivateBlankAddress"];
+    [fDefaults removeObjectForKey: @"WarningRemoveTrackers"];
     [fDefaults removeObjectForKey: @"WarningInvalidOpen"];
     [fDefaults removeObjectForKey: @"WarningDonate"];
+    //[fDefaults removeObjectForKey: @"WarningLegal"];
+}
+
+- (void) setDefaultForMagnets: (id) sender
+{
+    NSString * bundleID = [[NSBundle mainBundle] bundleIdentifier];
+    const OSStatus result = LSSetDefaultHandlerForURLScheme((CFStringRef)@"magnet", (CFStringRef)bundleID);
+    if (result != noErr)
+        NSLog(@"Failed setting default magnet link handler");
 }
 
 - (void) setQueue: (id) sender
@@ -730,6 +767,16 @@ tr_session * fHandle;
         @selector(incompleteFolderSheetClosed:returnCode:contextInfo:) contextInfo: nil];
 }
 
+- (void) setUseIncompleteFolder: (id) sender
+{
+    tr_sessionSetIncompleteDirEnabled(fHandle, [fDefaults boolForKey: @"UseIncompleteDownloadFolder"]);
+}
+
+- (void) setRenamePartialFiles: (id) sender
+{
+    tr_sessionSetIncompleteFileNamingEnabled(fHandle, [fDefaults boolForKey: @"RenamePartialFiles"]);
+}
+
 - (void) setAutoImport: (id) sender
 {
     NSString * path;
@@ -799,6 +846,9 @@ tr_session * fHandle;
             break;
         case PROXY_SOCKS5:
             type = @"SOCKS5";
+            break;
+        default:
+            NSAssert1(NO, @"Unknown index %d received for proxy type", [sender indexOfSelectedItem]);
     }
     
     [fDefaults setObject: type forKey: @"ProxyType"];
@@ -1053,19 +1103,19 @@ tr_session * fHandle;
 
 - (void) helpForPeers: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"PeersPrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"peers"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
 - (void) helpForNetwork: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"NetworkPrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"network"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
 - (void) helpForRemote: (id) sender
 {
-    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"RemotePrefs"
+    [[NSHelpManager sharedHelpManager] openHelpAnchor: @"remote"
         inBook: [[NSBundle mainBundle] objectForInfoDictionaryKey: @"CFBundleHelpBookName"]];
 }
 
@@ -1080,6 +1130,15 @@ tr_session * fHandle;
     NSString * downloadLocation = [[NSString stringWithUTF8String: tr_sessionGetDownloadDir(fHandle)] stringByStandardizingPath];
     [fDefaults setObject: downloadLocation forKey: @"DownloadFolder"];
     
+    NSString * incompleteLocation = [[NSString stringWithUTF8String: tr_sessionGetIncompleteDir(fHandle)] stringByStandardizingPath];
+    [fDefaults setObject: incompleteLocation forKey: @"IncompleteDownloadFolder"];
+    
+    const BOOL useIncomplete = tr_sessionIsIncompleteDirEnabled(fHandle);
+    [fDefaults setBool: useIncomplete forKey: @"UseIncompleteDownloadFolder"];
+    
+    const BOOL usePartialFileRanaming = tr_sessionIsIncompleteFileNamingEnabled(fHandle);
+    [fDefaults setBool: usePartialFileRanaming forKey: @"RenamePartialFiles"];
+    
     //peers
     const uint16_t peersTotal = tr_sessionGetPeerLimit(fHandle);
     [fDefaults setInteger: peersTotal forKey: @"PeersTotal"];
@@ -1095,6 +1154,14 @@ tr_session * fHandle;
     const BOOL dht = tr_sessionIsDHTEnabled(fHandle);
     [fDefaults setBool: dht forKey: @"DHTGlobal"];
     
+    //dht
+    const BOOL lpd = tr_sessionIsLPDEnabled(fHandle);
+    [fDefaults setBool: lpd forKey: @"LocalPeerDiscoveryGlobal"];
+    
+    //auto start
+    const BOOL autoStart = !tr_sessionGetPaused(fHandle);
+    [fDefaults setBool: autoStart forKey: @"AutoStartDownload"];
+    
     //port
     const tr_port port = tr_sessionGetPeerPort(fHandle);
     [fDefaults setInteger: port forKey: @"BindPort"];
@@ -1173,6 +1240,8 @@ tr_session * fHandle;
         
         //dht handled by bindings
         
+        //lpd handled by bindings
+        
         [fPortField setIntValue: port];
         //port forwarding (nat) handled by bindings
         //random port handled by bindings
@@ -1288,7 +1357,12 @@ tr_session * fHandle;
 - (void) incompleteFolderSheetClosed: (NSOpenPanel *) openPanel returnCode: (int) code contextInfo: (void *) info
 {
     if (code == NSOKButton)
-        [fDefaults setObject: [[openPanel filenames] objectAtIndex: 0] forKey: @"IncompleteDownloadFolder"];
+    {
+        NSString * folder = [[openPanel filenames] objectAtIndex: 0];
+        [fDefaults setObject: folder forKey: @"IncompleteDownloadFolder"];
+        
+        tr_sessionSetIncompleteDir(fHandle, [folder UTF8String]);
+    }
     [fIncompleteFolderPopUp selectItemAtIndex: 0];
 }
 
