@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,16 +7,16 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: file-tree.cc 8570 2009-05-31 19:33:48Z charles $
+ * $Id: file-tree.cc 11476 2010-12-05 18:53:04Z charles $
  */
 
 #include <cassert>
 #include <iostream>
 
 #include <QApplication>
+#include <QHeaderView>
 #include <QPainter>
 #include <QResizeEvent>
-#include <QHeaderView>
 #include <QStringList>
 
 #include <libtransmission/transmission.h> // priorities
@@ -43,9 +43,13 @@ FileTreeItem :: ~FileTreeItem( )
 {
     assert( myChildren.isEmpty( ) );
 
-    if( myParent )
-        if( !myParent->myChildren.removeOne( this ) )
+    if( myParent ) {
+        const int pos = myParent->myChildren.indexOf( this );
+        if( pos >= 0 )
+            myParent->myChildren.removeAt( pos );
+        else
             assert( 0 && "failed to remove" );
+    }
 }
 
 void
@@ -82,7 +86,7 @@ FileTreeItem :: data( int column ) const
     QVariant value;
 
     switch( column ) {
-        case COL_NAME: value.setValue( name() ); break;
+        case COL_NAME: value.setValue( fileSizeName( ) ); break;
         case COL_PROGRESS: value.setValue( progress( ) ); break;
         case COL_WANTED: value.setValue( isSubtreeWanted( ) ); break;
         case COL_PRIORITY: value.setValue( priorityString( ) ); break;
@@ -112,8 +116,18 @@ FileTreeItem :: progress( ) const
     return d;
 }
 
+QString
+FileTreeItem :: fileSizeName( ) const
+{
+    uint64_t have(0), total(0);
+    QString str;
+    getSubtreeSize( have, total );
+    str = QString( name() + " (%1)" ).arg( Utils::sizeToString( total ) );
+    return str;
+}
+
 bool
-FileTreeItem :: update( int index, bool wanted, int priority, uint64_t totalSize, uint64_t haveSize )
+FileTreeItem :: update( int index, bool wanted, int priority, uint64_t totalSize, uint64_t haveSize, bool torrentChanged )
 {
     bool changed = false;
 
@@ -122,12 +136,12 @@ FileTreeItem :: update( int index, bool wanted, int priority, uint64_t totalSize
         myIndex = index;
         changed = true;
     }
-    if( myIsWanted != wanted )
+    if( torrentChanged && myIsWanted != wanted )
     {
         myIsWanted = wanted;
         changed = true;
     }
-    if( myPriority != priority )
+    if( torrentChanged && myPriority != priority )
     {
         myPriority = priority;
         changed = true;
@@ -248,7 +262,7 @@ FileTreeModel :: FileTreeModel( QObject *parent ):
 {
     rootItem = new FileTreeItem( -1 );
 }
-     
+
 FileTreeModel :: ~FileTreeModel( )
 {
     clear( );
@@ -295,7 +309,7 @@ FileTreeModel :: headerData( int column, Qt::Orientation orientation, int role )
             default: break;
         }
     }
-                
+
     return data;
 }
 
@@ -399,7 +413,8 @@ FileTreeModel :: addFile( int                   index,
                           int                   priority,
                           uint64_t              size,
                           uint64_t              have,
-                          QList<QModelIndex>  & rowsAdded )
+                          QList<QModelIndex>  & rowsAdded,
+                          bool                  torrentChanged )
 {
     FileTreeItem * i( rootItem );
 
@@ -419,7 +434,7 @@ FileTreeModel :: addFile( int                   index,
     }
 
     if( i != rootItem )
-        if( i->update( index, wanted, priority, size, have ) )
+        if( i->update( index, wanted, priority, size, have, torrentChanged ) )
             dataChanged( indexOf( i, 0 ), indexOf( i, NUM_COLUMNS-1 ) );
 }
 
@@ -521,7 +536,10 @@ FileTreeDelegate :: paint( QPainter                    * painter,
         if( index.model()->hasChildren( index ) )
             icon = style->standardIcon( QStyle::StandardPixmap( QStyle::SP_DirOpenIcon ) );
         else
-            icon = Utils :: guessMimeIcon( index.model()->data(index).toString( ) );
+        {
+            QString name = index.model()->data(index).toString();
+            icon = Utils :: guessMimeIcon( name.left( name.lastIndexOf( " (" ) ) );
+        }
         icon.paint( painter, iconArea, Qt::AlignCenter, QIcon::Normal, QIcon::On );
 
         // draw the name
@@ -585,7 +603,7 @@ FileTreeView :: FileTreeView( QWidget * parent ):
     setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
     installEventFilter( this );
 
-    for( int i=0; i<=NUM_COLUMNS; ++i )
+    for( int i=0; i<NUM_COLUMNS; ++i )
         header()->setResizeMode( i, QHeaderView::Fixed );
 
     connect( this,     SIGNAL(clicked(const QModelIndex&)),
@@ -598,7 +616,7 @@ FileTreeView :: FileTreeView( QWidget * parent ):
              this,     SIGNAL(wantedChanged(const QSet<int>&, bool)));
 }
 
-bool 
+bool
 FileTreeView :: eventFilter( QObject * o, QEvent * event )
 {
     if( o != this )
@@ -632,10 +650,16 @@ FileTreeView :: eventFilter( QObject * o, QEvent * event )
 
 void
 FileTreeView :: update( const FileList& files )
+{
+    update( files, true );
+}
+
+void
+FileTreeView :: update( const FileList& files, bool torrentChanged )
 {
     foreach( const TrFile file, files ) {
         QList<QModelIndex> added;
-        myModel.addFile( file.index, file.filename, file.wanted, file.priority, file.size, file.have, added );
+        myModel.addFile( file.index, file.filename, file.wanted, file.priority, file.size, file.have, added, torrentChanged );
         foreach( QModelIndex i, added )
             expand( i );
     }
