@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,7 +7,7 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: mainwin.cc 9367 2009-10-22 03:01:38Z charles $
+ * $Id: mainwin.cc 11475 2010-12-05 18:52:09Z charles $
  */
 
 #include <cassert>
@@ -17,14 +17,18 @@
 #include <QCloseEvent>
 #include <QDesktopServices>
 #include <QFileDialog>
+#include <QHBoxLayout>
+#include <QInputDialog>
 #include <QLabel>
+#include <QMessageBox>
+#include <QSignalMapper>
 #include <QSize>
 #include <QStyle>
-#include <QHBoxLayout>
 #include <QSystemTrayIcon>
 #include <QUrl>
-#include <QSignalMapper>
 
+#include <libtransmission/transmission.h>
+#include <libtransmission/utils.h>
 #include <libtransmission/version.h>
 
 #include "about.h"
@@ -82,11 +86,10 @@ namespace
 TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& model, bool minimized ):
     myLastFullUpdateTime( 0 ),
     mySessionDialog( new SessionDialog( session, prefs, this ) ),
-    myPrefsDialog( new PrefsDialog( session, prefs, this ) ),
+    myPrefsDialog( 0 ),
     myAboutDialog( new AboutDialog( this ) ),
     myStatsDialog( new StatsDialog( session, this ) ),
     myDetailsDialog( 0 ),
-    myFileDialog( 0 ),
     myFilterModel( prefs ),
     myTorrentDelegate( new TorrentDelegate( this ) ),
     myTorrentDelegateMin( new TorrentDelegateMin( this ) ),
@@ -110,7 +113,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     const QSize smallIconSize( i, i );
 
     // icons
-    ui.action_Add->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
+    ui.action_AddFile->setIcon( getStockIcon( "list-add", QStyle::SP_DialogOpenButton ) );
     ui.action_New->setIcon( getStockIcon( "document-new", QStyle::SP_DesktopIcon ) );
     ui.action_Properties->setIcon( getStockIcon( "document-properties", QStyle::SP_DesktopIcon ) );
     ui.action_OpenFolder->setIcon( getStockIcon( "folder-open", QStyle::SP_DirOpenIcon ) );
@@ -133,7 +136,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_TrayIcon, SIGNAL(toggled(bool)), this, SLOT(setTrayIconVisible(bool)));
     connect( ui.action_Filterbar, SIGNAL(toggled(bool)), this, SLOT(setFilterbarVisible(bool)));
     connect( ui.action_Statusbar, SIGNAL(toggled(bool)), this, SLOT(setStatusbarVisible(bool)));
-    connect( ui.action_MinimalView, SIGNAL(toggled(bool)), this, SLOT(setMinimalView(bool)));
+    connect( ui.action_CompactView, SIGNAL(toggled(bool)), this, SLOT(setCompactView(bool)));
     connect( ui.action_SortByActivity, SIGNAL(toggled(bool)), this, SLOT(onSortByActivityToggled(bool)));
     connect( ui.action_SortByAge,      SIGNAL(toggled(bool)), this, SLOT(onSortByAgeToggled(bool)));
     connect( ui.action_SortByETA,      SIGNAL(toggled(bool)), this, SLOT(onSortByETAToggled(bool)));
@@ -152,13 +155,15 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     connect( ui.action_Announce, SIGNAL(triggered()), this, SLOT(reannounceSelected()) );
     connect( ui.action_StartAll, SIGNAL(triggered()), this, SLOT(startAll()));
     connect( ui.action_PauseAll, SIGNAL(triggered()), this, SLOT(pauseAll()));
-    connect( ui.action_Add, SIGNAL(triggered()), this, SLOT(openTorrent()));
+    connect( ui.action_AddFile, SIGNAL(triggered()), this, SLOT(openTorrent()));
+    connect( ui.action_AddURL, SIGNAL(triggered()), this, SLOT(openURL()));
     connect( ui.action_New, SIGNAL(triggered()), this, SLOT(newTorrent()));
-    connect( ui.action_Preferences, SIGNAL(triggered()), myPrefsDialog, SLOT(show()));
+    connect( ui.action_Preferences, SIGNAL(triggered()), this, SLOT(openPreferences()));
     connect( ui.action_Statistics, SIGNAL(triggered()), myStatsDialog, SLOT(show()));
     connect( ui.action_About, SIGNAL(triggered()), myAboutDialog, SLOT(show()));
     connect( ui.action_Contents, SIGNAL(triggered()), this, SLOT(openHelp()));
     connect( ui.action_OpenFolder, SIGNAL(triggered()), this, SLOT(openFolder()));
+    connect( ui.action_CopyMagnetToClipboard, SIGNAL(triggered()), this, SLOT(copyMagnetLinkToClipboard()));
     connect( ui.action_SetLocation, SIGNAL(triggered()), this, SLOT(setLocation()));
     connect( ui.action_Properties, SIGNAL(triggered()), this, SLOT(openProperties()));
     connect( ui.action_SessionDialog, SIGNAL(triggered()), mySessionDialog, SLOT(show()));
@@ -166,17 +171,21 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
 
     QAction * sep2 = new QAction( this );
     sep2->setSeparator( true );
+    QAction * sep3 = new QAction( this );
+    sep3->setSeparator( true );
 
     // context menu
     QList<QAction*> actions;
     actions << ui.action_Properties
             << ui.action_OpenFolder
-            << ui.action_SetLocation
             << sep2
             << ui.action_Start
-            << ui.action_Pause
-            << ui.action_Verify
             << ui.action_Announce
+            << ui.action_Pause
+            << ui.action_CopyMagnetToClipboard
+            << sep3
+            << ui.action_Verify
+            << ui.action_SetLocation
             << sep
             << ui.action_Remove
             << ui.action_Delete;
@@ -214,7 +223,8 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
     actionGroup->addAction( ui.action_SortByTracker );
 
     QMenu * menu = new QMenu( );
-    menu->addAction( ui.action_Add );
+    menu->addAction( ui.action_AddFile );
+    menu->addAction( ui.action_AddURL );
     menu->addSeparator( );
     menu->addAction( ui.action_ShowMainWindow );
     menu->addAction( ui.action_ShowMessageLog );
@@ -249,7 +259,7 @@ TrMainWindow :: TrMainWindow( Session& session, Prefs& prefs, TorrentModel& mode
              << Prefs :: STATUSBAR_STATS
              << Prefs :: TOOLBAR
              << Prefs :: ALT_SPEED_LIMIT_ENABLED
-             << Prefs :: MINIMAL_VIEW
+             << Prefs :: COMPACT_VIEW
              << Prefs :: DSPEED
              << Prefs :: DSPEED_ENABLED
              << Prefs :: USPEED
@@ -428,8 +438,8 @@ QWidget *
 TrMainWindow :: createStatusBar( )
 {
     QMenu * m;
-    QLabel *l, *l2;
-    QWidget *w;
+    QLabel * l;
+    QWidget * w;
     QHBoxLayout * h;
     QPushButton * p;
     QActionGroup * a;
@@ -443,12 +453,16 @@ TrMainWindow :: createStatusBar( )
 
         p = myOptionsButton = new TrIconPushButton( this );
         p->setIcon( QIcon( ":/icons/options.png" ) );
+        p->setIconSize( QPixmap( ":/icons/options.png" ).size() );
         p->setFlat( true );
         p->setMenu( createOptionsMenu( ) );
         h->addWidget( p );
 
-        p = myAltSpeedButton = new TrIconPushButton( this );
+        p = myAltSpeedButton = new QPushButton( this );
         p->setIcon( myPrefs.get<bool>(Prefs::ALT_SPEED_LIMIT_ENABLED) ? mySpeedModeOnIcon : mySpeedModeOffIcon );
+        p->setIconSize( QPixmap( ":/icons/alt-limit-on.png" ).size() );
+        p->setCheckable( true );
+        p->setFixedWidth( p->height() );
         p->setFlat( true );
         h->addWidget( p );
         connect( p, SIGNAL(clicked()), this, SLOT(toggleSpeedMode()));
@@ -462,7 +476,7 @@ TrMainWindow :: createStatusBar( )
         h->addWidget( l );
 
     h->addStretch( 1 );
-  
+
         a = new QActionGroup( this );
         a->addAction( ui.action_TotalRatio );
         a->addAction( ui.action_TotalTransfer );
@@ -478,34 +492,37 @@ TrMainWindow :: createStatusBar( )
         connect( ui.action_SessionRatio, SIGNAL(triggered()), this, SLOT(showSessionRatio()));
         connect( ui.action_SessionTransfer, SIGNAL(triggered()), this, SLOT(showSessionTransfer()));
         p = myStatsModeButton = new TrIconPushButton( this );
-        p->setIcon( getStockIcon( "view-refresh", QStyle::SP_BrowserReload ) );
+        p->setIcon( QIcon( ":/icons/ratio.png" ) );
+        p->setIconSize( QPixmap( ":/icons/ratio.png" ).size() );
         p->setFlat( true );
         p->setMenu( m );
-        h->addWidget( p );  
+        h->addWidget( p );
         l = myStatsLabel = new QLabel( this );
-        h->addWidget( l );  
-   
-        w = new QWidget( this );
-        w->setMinimumSize( HIG::PAD_BIG, 1 );
-        w->setMaximumSize( HIG::PAD_BIG, 1 );
-        h->addWidget( w );
-        l = new QLabel( this );
-        l->setPixmap( getStockIcon( "go-down", QStyle::SP_ArrowDown ).pixmap( smallIconSize ) );
         h->addWidget( l );
-        l2 = myDownloadSpeedLabel = new QLabel( this );
-        h->addWidget( l2 );
-        myDownStatusWidgets << w << l << l2;
+
+    h->addSpacing( HIG::PAD_BIG );
 
         w = new QWidget( this );
         w->setMinimumSize( HIG::PAD_BIG, 1 );
         w->setMaximumSize( HIG::PAD_BIG, 1 );
         h->addWidget( w );
+        l = myDownloadSpeedLabel = new QLabel( this );
+        h->addWidget( l );
+        l = new QLabel( this );
+        l->setPixmap( getStockIcon( "go-down", QStyle::SP_ArrowDown ).pixmap( smallIconSize ) );
+        h->addWidget( l );
+
+    h->addSpacing( HIG::PAD_BIG );
+
+        w = new QWidget( this );
+        w->setMinimumSize( HIG::PAD_BIG, 1 );
+        w->setMaximumSize( HIG::PAD_BIG, 1 );
+        h->addWidget( w );
+        l = myUploadSpeedLabel = new QLabel;
+        h->addWidget( l );
         l = new QLabel;
         l->setPixmap( getStockIcon( "go-up", QStyle::SP_ArrowUp ).pixmap( smallIconSize ) );
         h->addWidget( l );
-        l2 = myUploadSpeedLabel = new QLabel;
-        h->addWidget( l2 );
-        myUpStatusWidgets << w << l << l2;
 
     return top;
 }
@@ -618,6 +635,23 @@ TrMainWindow :: setSortAscendingPref( bool b )
 *****
 ****/
 
+void
+TrMainWindow :: onPrefsDestroyed( )
+{
+    myPrefsDialog = 0;
+}
+
+void
+TrMainWindow :: openPreferences( )
+{
+    if( myPrefsDialog == 0 ) {
+        myPrefsDialog = new PrefsDialog( mySession, myPrefs, this );
+        connect( myPrefsDialog, SIGNAL(destroyed(QObject*)), this, SLOT(onPrefsDestroyed()));
+    }
+
+    myPrefsDialog->show( );
+}
+
 void
 TrMainWindow :: onDetailsDestroyed( )
 {
@@ -628,7 +662,7 @@ void
 TrMainWindow :: openProperties( )
 {
     if( myDetailsDialog == 0 ) {
-        myDetailsDialog = new Details( mySession, myModel, this );
+        myDetailsDialog = new Details( mySession, myPrefs, myModel, this );
         connect( myDetailsDialog, SIGNAL(destroyed(QObject*)), this, SLOT(onDetailsDestroyed()));
     }
 
@@ -639,7 +673,7 @@ TrMainWindow :: openProperties( )
 void
 TrMainWindow :: setLocation( )
 {
-    QDialog * d = new RelocateDialog( mySession, getSelectedTorrents(), this );
+    QDialog * d = new RelocateDialog( mySession, myModel, getSelectedTorrents(), this );
     d->show( );
 }
 
@@ -652,6 +686,13 @@ TrMainWindow :: openFolder( )
     QDesktopServices :: openUrl( QUrl::fromLocalFile( path ) );
 }
 
+void
+TrMainWindow :: copyMagnetLinkToClipboard( )
+{
+    const int id( *getSelectedTorrents().begin() );
+    mySession.copyMagnetLinkToClipboard( id );
+}
+
 void
 TrMainWindow :: openHelp( )
 {
@@ -659,7 +700,7 @@ TrMainWindow :: openHelp( )
     int major, minor;
     sscanf( SHORT_VERSION_STRING, "%d.%d", &major, &minor );
     char url[128];
-    snprintf( url, sizeof( url ), fmt, major, minor/10 );
+    tr_snprintf( url, sizeof( url ), fmt, major, minor/10 );
     QDesktopServices :: openUrl( QUrl( QString( url ) ) );
 }
 
@@ -669,7 +710,7 @@ TrMainWindow :: refreshTitle( )
     QString title( "Transmission" );
     const QUrl url( mySession.getRemoteUrl( ) );
     if( !url.isEmpty() )
-        title += tr( " - %1" ).arg( url.toString(QUrl::RemoveUserInfo) );
+        title += tr( " - %1:%2" ).arg( url.host() ).arg( url.port() );
     setWindowTitle( title );
 }
 
@@ -694,8 +735,6 @@ TrMainWindow :: refreshStatusBar( )
     const Speed down( myModel.getDownloadSpeed( ) );
     myUploadSpeedLabel->setText( Utils :: speedToString( up ) );
     myDownloadSpeedLabel->setText( Utils :: speedToString( down ) );
-    foreach( QWidget * w, myUpStatusWidgets ) w->setVisible( !up.isZero( ) );
-    foreach( QWidget * w, myDownStatusWidgets ) w->setVisible( !down.isZero( ) );
 
     myNetworkLabel->setVisible( !mySession.isServer( ) );
 
@@ -766,6 +805,7 @@ TrMainWindow :: refreshActionSensitivity( )
 
     const bool oneSelection( selected == 1 );
     ui.action_OpenFolder->setEnabled( oneSelection && mySession.isLocal( ) );
+    ui.action_CopyMagnetToClipboard->setEnabled( oneSelection );
 
     ui.action_SelectAll->setEnabled( selected < rowCount );
     ui.action_StartAll->setEnabled( paused > 0 );
@@ -825,12 +865,12 @@ TrMainWindow :: pauseAll( )
 void
 TrMainWindow :: removeSelected( )
 {
-    mySession.removeTorrents( getSelectedTorrents( ), false );
+    removeTorrents( false );
 }
 void
 TrMainWindow :: deleteSelected( )
 {
-    mySession.removeTorrents( getSelectedTorrents( ), true );
+    removeTorrents( true );
 }
 void
 TrMainWindow :: verifySelected( )
@@ -868,9 +908,9 @@ void TrMainWindow :: showSessionTransfer ( ) { myPrefs.set( Prefs::STATUSBAR_STA
 **/
 
 void
-TrMainWindow :: setMinimalView( bool visible )
+TrMainWindow :: setCompactView( bool visible )
 {
-    myPrefs.set( Prefs :: MINIMAL_VIEW, visible );
+    myPrefs.set( Prefs :: COMPACT_VIEW, visible );
 }
 void
 TrMainWindow :: setTrayIconVisible( bool visible )
@@ -969,7 +1009,7 @@ TrMainWindow :: refreshPref( int key )
         case Prefs::DSPEED_ENABLED:
             (myPrefs.get<bool>(key) ? myDlimitOnAction : myDlimitOffAction)->setChecked( true );
             break;
-     
+
         case Prefs::DSPEED:
             myDlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
             break;
@@ -977,7 +1017,7 @@ TrMainWindow :: refreshPref( int key )
         case Prefs::USPEED_ENABLED:
             (myPrefs.get<bool>(key) ? myUlimitOnAction : myUlimitOffAction)->setChecked( true );
             break;
-     
+
         case Prefs::USPEED:
             myUlimitOnAction->setText( tr( "Limited at %1" ).arg( Utils::speedToString( Speed::fromKbps( myPrefs.get<int>(key) ) ) ) );
             break;
@@ -1020,12 +1060,19 @@ TrMainWindow :: refreshPref( int key )
             myTrayIcon.setVisible( b );
             break;
 
-        case Prefs::MINIMAL_VIEW:
+        case Prefs::COMPACT_VIEW: {
+            QItemSelectionModel * selectionModel( ui.listView->selectionModel( ) );
+            const QItemSelection selection( selectionModel->selection( ) );
+            const QModelIndex currentIndex( selectionModel->currentIndex( ) );
             b = myPrefs.getBool( key );
-            ui.action_MinimalView->setChecked( b );
+            ui.action_CompactView->setChecked( b );
             ui.listView->setItemDelegate( b ? myTorrentDelegateMin : myTorrentDelegate );
+            selectionModel->clear( );
             ui.listView->reset( ); // force the rows to resize
+            selectionModel->select( selection, QItemSelectionModel::Select );
+            selectionModel->setCurrentIndex( currentIndex, QItemSelectionModel::NoUpdate );
             break;
+        }
 
         case Prefs::MAIN_WINDOW_X:
         case Prefs::MAIN_WINDOW_Y:
@@ -1064,34 +1111,53 @@ TrMainWindow :: refreshPref( int key )
 void
 TrMainWindow :: newTorrent( )
 {
-    MakeDialog * d = new MakeDialog( mySession, this );
-    d->show( );
+    MakeDialog * dialog = new MakeDialog( mySession, this );
+    dialog->show( );
 }
 
 void
 TrMainWindow :: openTorrent( )
 {
-    if( myFileDialog == 0 )
-    {
-        myFileDialog = new QFileDialog( this,
-                                        tr( "Add Torrent" ),
-                                        myPrefs.getString( Prefs::OPEN_DIALOG_FOLDER ),
-                                        tr( "Torrent Files (*.torrent);;All Files (*.*)" ) );
-        myFileDialog->setFileMode( QFileDialog::ExistingFiles );
+    QFileDialog * myFileDialog;
+    myFileDialog = new QFileDialog( this,
+                                    tr( "Add Torrent" ),
+                                    myPrefs.getString( Prefs::OPEN_DIALOG_FOLDER ),
+                                    tr( "Torrent Files (*.torrent);;All Files (*.*)" ) );
+    myFileDialog->setFileMode( QFileDialog::ExistingFiles );
 
+    QCheckBox * button = new QCheckBox( tr( "Show &options dialog" ) );
+    button->setChecked( myPrefs.getBool( Prefs::OPTIONS_PROMPT ) );
+    QGridLayout * layout = dynamic_cast<QGridLayout*>(myFileDialog->layout());
+    layout->addWidget( button, layout->rowCount( ), 0, 1, -1, Qt::AlignLeft );
+    myFileDialogOptionsCheck = button;
 
-        QCheckBox * button = new QCheckBox( tr( "Display &options dialog" ) );
-        button->setChecked( myPrefs.getBool( Prefs::OPTIONS_PROMPT ) );
-        QGridLayout * layout = dynamic_cast<QGridLayout*>(myFileDialog->layout());
-        layout->addWidget( button, layout->rowCount( ), 0, 1, -1, Qt::AlignLeft );
-        myFileDialogOptionsCheck = button;
-
-        connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)), this, SLOT(addTorrents(const QStringList&)));
-    }
+    connect( myFileDialog, SIGNAL(filesSelected(const QStringList&)),
+             this, SLOT(addTorrents(const QStringList&)));
 
     myFileDialog->show( );
 }
 
+void
+TrMainWindow :: openURL( )
+{
+    QString tmp;
+    openURL( tmp );
+}
+
+void
+TrMainWindow :: openURL( QString url )
+{
+    bool ok;
+    const QString key = QInputDialog::getText( this,
+                                               tr( "Add URL or Magnet Link" ),
+                                               tr( "Add URL or Magnet Link" ),
+                                               QLineEdit::Normal,
+                                               url,
+                                               &ok );
+    if( ok && !key.isEmpty( ) )
+        mySession.addTorrent( key );
+}
+
 void
 TrMainWindow :: addTorrents( const QStringList& filenames )
 {
@@ -1112,6 +1178,101 @@ TrMainWindow :: addTorrent( const QString& filename )
     }
 }
 
+void
+TrMainWindow :: removeTorrents( const bool deleteFiles )
+{
+    QSet<int> ids;
+    QMessageBox msgBox( this );
+    QString primary_text, secondary_text;
+    int incomplete = 0;
+    int connected  = 0;
+    int count;
+
+    foreach( QModelIndex index, ui.listView->selectionModel( )->selectedRows( ) )
+    {
+        const Torrent * tor( index.model()->data( index, TorrentModel::TorrentRole ).value<const Torrent*>( ) );
+        ids.insert( tor->id( ) );
+        if( tor->connectedPeers( ) )
+            ++connected;
+        if( !tor->isDone( ) )
+            ++incomplete;
+    }
+
+    if( ids.isEmpty() )
+        return;
+    count = ids.size();
+
+    if( !deleteFiles )
+    {
+        primary_text = ( count == 1 )
+            ? tr( "Remove torrent?" )
+            : tr( "Remove %1 torrents?" ).arg( count );
+    }
+    else
+    {
+        primary_text = ( count == 1 )
+            ? tr( "Delete this torrent's downloaded files?" )
+            : tr( "Delete these %1 torrents' downloaded files?" ).arg( count );
+    }
+
+    if( !incomplete && !connected )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "Once removed, continuing the transfer will require the torrent file or magnet link." )
+            : tr( "Once removed, continuing the transfers will require the torrent files or magnet links." );
+    }
+    else if( count == incomplete )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "This torrent has not finished downloading." )
+            : tr( "These torrents have not finished downloading." );
+    }
+    else if( count == connected )
+    {
+        secondary_text = ( count == 1 )
+            ? tr( "This torrent is connected to peers." )
+            : tr( "These torrents are connected to peers." );
+    }
+    else
+    {
+        if( connected )
+        {
+            secondary_text = ( connected == 1 )
+                ? tr( "One of these torrents is connected to peers." )
+                : tr( "Some of these torrents are connected to peers." );
+        }
+
+        if( connected && incomplete )
+        {
+            secondary_text += "\n";
+        }
+
+        if( incomplete )
+        {
+            secondary_text += ( incomplete == 1 )
+                ? tr( "One of these torrents has not finished downloading." )
+                : tr( "Some of these torrents have not finished downloading." );
+        }
+    }
+
+    msgBox.setWindowTitle( QString(" ") );
+    msgBox.setText( QString( "<big><b>%1</big></b>" ).arg( primary_text ) );
+    msgBox.setInformativeText( secondary_text );
+    msgBox.setStandardButtons( QMessageBox::Ok | QMessageBox::Cancel );
+    msgBox.setDefaultButton( QMessageBox::Cancel );
+    msgBox.setIcon( QMessageBox::Question );
+    /* hack needed to keep the dialog from being too narrow */
+    QGridLayout* layout = (QGridLayout*)msgBox.layout();
+    QSpacerItem* spacer = new QSpacerItem( 450, 0, QSizePolicy::Minimum, QSizePolicy::Expanding );
+    layout->addItem( spacer, layout->rowCount(), 0, 1, layout->columnCount() );
+
+    if( msgBox.exec() == QMessageBox::Ok )
+    {
+        ui.listView->selectionModel()->clear();
+        mySession.removeTorrents( ids, deleteFiles );
+    }
+}
+
 /***
 ****
 ***/
