@@ -1,5 +1,5 @@
 /*
- * This file Copyright (C) 2009 Charles Kerr <charles@transmissionbt.com>
+ * This file Copyright (C) 2009-2010 Mnemosyne LLC
  *
  * This file is licensed by the GPL version 2.  Works owned by the
  * Transmission project are granted a special exemption to clause 2(b)
@@ -7,17 +7,18 @@
  * This exemption does not extend to derived works not owned by
  * the Transmission project.
  *
- * $Id: torrent.cc 8875 2009-08-06 07:04:19Z charles $
+ * $Id: torrent.cc 10873 2010-06-26 17:48:57Z Longinus00 $
  */
 
 #include <cassert>
 #include <iostream>
 
 #include <QApplication>
-#include <QStyle>
+#include <QFileIconProvider>
+#include <QFileInfo>
 #include <QSet>
 #include <QString>
-#include <QFileInfo>
+#include <QStyle>
 #include <QVariant>
 
 #include <libtransmission/transmission.h>
@@ -26,7 +27,6 @@
 
 #include "app.h"
 #include "prefs.h"
-#include "qticonloader.h"
 #include "torrent.h"
 #include "utils.h"
 
@@ -55,7 +55,6 @@ Torrent :: myProperties[] =
     { ID, "id", QVariant::Int, INFO, },
     { UPLOAD_SPEED, "rateUpload", QVariant::Int, STAT } /* B/s */,
     { DOWNLOAD_SPEED, "rateDownload", QVariant::Int, STAT }, /* B/s */
-    { SWARM_SPEED, "swarmSpeed", QVariant::Int, STAT_EXTRA },/* KB/s */
     { DOWNLOAD_DIR, "downloadDir", QVariant::String, STAT },
     { ACTIVITY, "status", QVariant::Int, STAT },
     { NAME, "name", QVariant::String, INFO },
@@ -65,6 +64,7 @@ Torrent :: myProperties[] =
     { LEFT_UNTIL_DONE, "leftUntilDone", QVariant::ULongLong, STAT },
     { HAVE_UNCHECKED, "haveUnchecked", QVariant::ULongLong, STAT },
     { HAVE_VERIFIED, "haveValid", QVariant::ULongLong, STAT },
+    { DESIRED_AVAILABLE, "desiredAvailable", QVariant::ULongLong, STAT },
     { TOTAL_SIZE, "totalSize", QVariant::ULongLong, INFO },
     { PIECE_SIZE, "pieceSize", QVariant::ULongLong, INFO },
     { PIECE_COUNT, "pieceCount", QVariant::Int, INFO },
@@ -72,6 +72,7 @@ Torrent :: myProperties[] =
     { PEERS_SENDING_TO_US, "peersSendingToUs", QVariant::Int, STAT },
     { WEBSEEDS_SENDING_TO_US, "webseedsSendingToUs", QVariant::Int, STAT_EXTRA },
     { PERCENT_DONE, "percentDone", QVariant::Double, STAT },
+    { METADATA_PERCENT_DONE, "metadataPercentComplete", QVariant::Double, STAT },
     { PERCENT_VERIFIED, "recheckProgress", QVariant::Double, STAT },
     { DATE_ACTIVITY, "activityDate", QVariant::DateTime, STAT_EXTRA },
     { DATE_ADDED, "addedDate", QVariant::DateTime, INFO },
@@ -84,30 +85,22 @@ Torrent :: myProperties[] =
     { UPLOADED_EVER, "uploadedEver", QVariant::ULongLong, STAT },
     { FAILED_EVER, "corruptEver", QVariant::ULongLong, STAT_EXTRA },
     { TRACKERS, "trackers", QVariant::StringList, INFO },
+    { TRACKERSTATS, "trackerStats", TrTypes::TrackerStatsList, STAT_EXTRA },
     { MIME_ICON, "ccc", QVariant::Icon, DERIVED },
-    { SEED_RATIO_LIMIT, "seedRatioLimit", QVariant::Double, STAT_EXTRA },
-    { SEED_RATIO_MODE, "seedRatioMode", QVariant::Int, STAT_EXTRA },
-    { DOWN_LIMIT, "downloadLimit", QVariant::Int, STAT_EXTRA }, /* KB/s */
+    { SEED_RATIO_LIMIT, "seedRatioLimit", QVariant::Double, STAT },
+    { SEED_RATIO_MODE, "seedRatioMode", QVariant::Int, STAT },
+    { DOWN_LIMIT, "downloadLimit", QVariant::Int, STAT_EXTRA }, /* KiB/s */
     { DOWN_LIMITED, "downloadLimited", QVariant::Bool, STAT_EXTRA },
-    { UP_LIMIT, "uploadLimit", QVariant::Int, STAT_EXTRA }, /* KB/s */
+    { UP_LIMIT, "uploadLimit", QVariant::Int, STAT_EXTRA }, /* KiB/s */
     { UP_LIMITED, "uploadLimited", QVariant::Bool, STAT_EXTRA },
     { HONORS_SESSION_LIMITS, "honorsSessionLimits", QVariant::Bool, STAT_EXTRA },
     { PEER_LIMIT, "peer-limit", QVariant::Int, STAT_EXTRA },
     { HASH_STRING, "hashString", QVariant::String, INFO },
+    { IS_FINISHED, "isFinished", QVariant::Bool, STAT },
     { IS_PRIVATE, "isPrivate", QVariant::Bool, INFO },
     { COMMENT, "comment", QVariant::String, INFO },
     { CREATOR, "creator", QVariant::String, INFO },
-    { LAST_ANNOUNCE_TIME, "lastAnnounceTime", QVariant::DateTime, STAT_EXTRA },
-    { LAST_SCRAPE_TIME, "lastScrapeTime", QVariant::DateTime, STAT_EXTRA },
     { MANUAL_ANNOUNCE_TIME, "manualAnnounceTime", QVariant::DateTime, STAT_EXTRA },
-    { NEXT_ANNOUNCE_TIME, "nextAnnounceTime", QVariant::DateTime, STAT_EXTRA },
-    { NEXT_SCRAPE_TIME, "nextScrapeTime", QVariant::DateTime, STAT_EXTRA },
-    { SCRAPE_RESPONSE, "scrapeResponse", QVariant::String, STAT_EXTRA },
-    { ANNOUNCE_RESPONSE, "announceResponse", QVariant::String, STAT_EXTRA },
-    { ANNOUNCE_URL, "announceURL", QVariant::String, STAT_EXTRA },
-    { SEEDERS, "seeders", QVariant::Int, STAT_EXTRA },
-    { LEECHERS, "leechers", QVariant::Int, STAT_EXTRA },
-    { TIMES_COMPLETED, "timesCompleted", QVariant::Int, STAT_EXTRA },
     { PEERS, "peers", TrTypes::PeerList, STAT_EXTRA },
     { TORRENT_FILE, "torrentFile", QVariant::String, STAT_EXTRA },
     { BANDWIDTH_PRIORITY, "bandwidthPriority", QVariant::Int, STAT_EXTRA }
@@ -219,7 +212,7 @@ Torrent :: setDateTime( int i, const QDateTime& value )
 
     return changed;
 }
-  
+
 bool
 Torrent :: setSize( int i, qulonglong value )
 {
@@ -390,8 +383,8 @@ Torrent :: compareETA( const Torrent& that ) const
     const bool haveA( hasETA( ) );
     const bool haveB( that.hasETA( ) );
     if( haveA && haveB ) return getETA() - that.getETA();
-    if( haveA ) return -1;
-    if( haveB ) return 1;
+    if( haveA ) return 1;
+    if( haveB ) return -1;
     return 0;
 }
 
@@ -416,9 +409,11 @@ Torrent :: updateMimeIcon( )
     QIcon icon;
 
     if( files.size( ) > 1 )
-        icon = QtIconLoader :: icon( "folder", QApplication::style()->standardIcon( QStyle::SP_DirIcon ) );
-    else
+        icon = QFileIconProvider().icon( QFileIconProvider::Folder );
+    else if( files.size( ) == 1 )
         icon = Utils :: guessMimeIcon( files.at(0).filename );
+    else
+        icon = QIcon( );
 
     setIcon( MIME_ICON, icon );
 }
@@ -499,7 +494,6 @@ Torrent :: update( tr_benc * d )
             default:
                 assert( 0 && "unhandled type" );
         }
-        
     }
 
     tr_benc * files;
@@ -556,6 +550,72 @@ Torrent :: update( tr_benc * d )
         }
     }
 
+    tr_benc * trackerStats;
+    if( tr_bencDictFindList( d, "trackerStats", &trackerStats ) ) {
+        tr_benc * child;
+        TrackerStatsList  trackerStatsList;
+        int childNum = 0;
+        while(( child = tr_bencListChild( trackerStats, childNum++ ))) {
+            tr_bool b;
+            int64_t i;
+            const char * str;
+            TrackerStat trackerStat;
+            if( tr_bencDictFindStr( child, "announce", &str ) )
+                trackerStat.announce = QString::fromUtf8( str );
+            if( tr_bencDictFindInt( child, "announceState", &i ) )
+                trackerStat.announceState = i;
+            if( tr_bencDictFindInt( child, "downloadCount", &i ) )
+                trackerStat.downloadCount = i;
+            if( tr_bencDictFindBool( child, "hasAnnounced", &b ) )
+                trackerStat.hasAnnounced = b;
+            if( tr_bencDictFindBool( child, "hasScraped", &b ) )
+                trackerStat.hasScraped = b;
+            if( tr_bencDictFindStr( child, "host", &str ) )
+                trackerStat.host = QString::fromUtf8( str );
+            if( tr_bencDictFindInt( child, "id", &i ) )
+                trackerStat.id = i;
+            if( tr_bencDictFindBool( child, "isBackup", &b ) )
+                trackerStat.isBackup = b;
+            if( tr_bencDictFindInt( child, "lastAnnouncePeerCount", &i ) )
+                trackerStat.lastAnnouncePeerCount = i;
+            if( tr_bencDictFindInt( child, "lastAnnounceResult", &i ) )
+                trackerStat.lastAnnounceResult = i;
+            if( tr_bencDictFindInt( child, "lastAnnounceStartTime", &i ) )
+                trackerStat.lastAnnounceStartTime = i;
+            if( tr_bencDictFindBool( child, "lastAnnounceSucceeded", &b ) )
+                trackerStat.lastAnnounceSucceeded = b;
+            if( tr_bencDictFindInt( child, "lastAnnounceTime", &i ) )
+                trackerStat.lastAnnounceTime = i;
+            if( tr_bencDictFindBool( child, "lastAnnounceTimedOut", &b ) )
+                trackerStat.lastAnnounceTimedOut = b;
+            if( tr_bencDictFindStr( child, "lastScrapeResult", &str ) )
+                trackerStat.lastScrapeResult = QString::fromUtf8( str );
+            if( tr_bencDictFindInt( child, "lastScrapeStartTime", &i ) )
+                trackerStat.lastScrapeStartTime = i;
+            if( tr_bencDictFindBool( child, "lastScrapeSucceeded", &b ) )
+                trackerStat.lastScrapeSucceeded = b;
+            if( tr_bencDictFindInt( child, "lastScrapeTime", &i ) )
+                trackerStat.lastScrapeTime = i;
+            if( tr_bencDictFindBool( child, "lastScrapeTimedOut", &b ) )
+                trackerStat.lastScrapeTimedOut = b;
+            if( tr_bencDictFindInt( child, "leecherCount", &i ) )
+                trackerStat.leecherCount = i;
+            if( tr_bencDictFindInt( child, "nextAnnounceTime", &i ) )
+                trackerStat.nextAnnounceTime = i;
+            if( tr_bencDictFindInt( child, "nextScrapeTime", &i ) )
+                trackerStat.nextScrapeTime = i;
+            if( tr_bencDictFindInt( child, "scrapeState", &i ) )
+                trackerStat.scrapeState = i;
+            if( tr_bencDictFindInt( child, "seederCount", &i ) )
+                trackerStat.seederCount = i;
+            if( tr_bencDictFindInt( child, "tier", &i ) )
+                trackerStat.tier = i;
+            trackerStatsList << trackerStat;
+        }
+        myValues[TRACKERSTATS].setValue( trackerStatsList );
+        changed = true;
+    }
+
     tr_benc * peers;
     if( tr_bencDictFindList( d, "peers", &peers ) ) {
         tr_benc * child;
@@ -575,6 +635,8 @@ Torrent :: update( tr_benc * d )
                 peer.clientIsChoked = b;
             if( tr_bencDictFindBool( child, "clientIsInterested", &b ) )
                 peer.clientIsInterested = b;
+            if( tr_bencDictFindStr( child, "flagStr", &str ) )
+                peer.flagStr = QString::fromUtf8( str );
             if( tr_bencDictFindBool( child, "isDownloadingFrom", &b ) )
                 peer.isDownloadingFrom = b;
             if( tr_bencDictFindBool( child, "isEncrypted", &b ) )
@@ -616,7 +678,7 @@ Torrent :: activityString( ) const
         case TR_STATUS_CHECK:      str = tr( "Verifying local data" ); break;
         case TR_STATUS_DOWNLOAD:   str = tr( "Downloading" ); break;
         case TR_STATUS_SEED:       str = tr( "Seeding" ); break;
-        case TR_STATUS_STOPPED:    str = tr( "Paused" ); break;
+        case TR_STATUS_STOPPED:    str = isFinished() ? tr( "Finished" ): tr( "Paused" ); break;
     }
 
     return str;
