@@ -46,12 +46,19 @@ Transmission.prototype =
 		$('#filter_paused_link').parent().bind('click', function(e){ tr.showPausedClicked(e); });
 		$('#prefs_save_button').bind('click', function(e) { tr.savePrefsClicked(e); return false;});
 		$('#prefs_cancel_button').bind('click', function(e){ tr.cancelPrefsClicked(e); return false; });
+		$('#stats_close_button').bind('click', function(e){ tr.closeStatsClicked(e); return false; });
 		$('.inspector_tab').bind('click', function(e){ tr.inspectorTabClicked(e, this); });
 		$('.file_wanted_control').live('click', function(e){ tr.fileWantedClicked(e, this); });
 		$('.file_priority_control').live('click', function(e){ tr.filePriorityClicked(e, this); });
+		$('#files_select_all').live('click', function(e){ tr.filesSelectAllClicked(e, this); });
+		$('#files_deselect_all').live('click', function(e){ tr.filesDeselectAllClicked(e, this); });
 		$('#open_link').bind('click', function(e){ tr.openTorrentClicked(e); });
 		$('#upload_confirm_button').bind('click', function(e){ tr.confirmUploadClicked(e); return false;});
 		$('#upload_cancel_button').bind('click', function(e){ tr.cancelUploadClicked(e); return false; });
+		$('#turtle_button').bind('click', function(e){ tr.toggleTurtleClicked(e); return false; });
+		$('#prefs_tab_general_tab').click(function(e){ changeTab(this, 'prefs_tab_general') });
+		$('#prefs_tab_speed_tab').click(function(e){ changeTab(this, 'prefs_tab_speed') });
+
 		if (iPhone) {
 			$('#inspector_close').bind('click', function(e){ tr.hideInspector(); });
 			$('#preferences_link').bind('click', function(e){ tr.releaseClutchPreferencesButton(e); });
@@ -65,9 +72,11 @@ Transmission.prototype =
 			this.createContextMenu();
 			this.createSettingsMenu();
 		}
+		this.initTurtleDropDowns();
 
 		this._torrent_list             = $('#torrent_list')[0];
 		this._inspector_file_list      = $('#inspector_file_list')[0];
+		this._inspector_trackers_list  = $('#inspector_trackers_list')[0];
 		this._inspector_tab_files      = $('#inspector_tab_files')[0];
 		this._toolbar_buttons          = $('#torrent_global_menu ul li');
 		this._toolbar_pause_button     = $('li#pause_selected')[0];
@@ -81,6 +90,7 @@ Transmission.prototype =
 		var ti = '#torrent_inspector_';
 		this._inspector = { };
 		this._inspector._info_tab = { };
+		this._inspector._info_tab.availability = $(ti+'availability')[0];
 		this._inspector._info_tab.comment = $(ti+'comment')[0];
 		this._inspector._info_tab.creator_date = $(ti+'creator_date')[0];
 		this._inspector._info_tab.creator = $(ti+'creator')[0];
@@ -97,10 +107,7 @@ Transmission.prototype =
 		this._inspector._info_tab.secure = $(ti+'secure')[0];
 		this._inspector._info_tab.size = $(ti+'size')[0];
 		this._inspector._info_tab.state = $(ti+'state')[0];
-		this._inspector._info_tab.swarm_speed = $(ti+'swarm_speed')[0];
-		this._inspector._info_tab.total_leechers = $(ti+'total_leechers')[0];
-		this._inspector._info_tab.total_seeders = $(ti+'total_seeders')[0];
-		this._inspector._info_tab.tracker = $(ti+'tracker')[0];
+		this._inspector._info_tab.pieces = $(ti+'pieces')[0];
 		this._inspector._info_tab.uploaded = $(ti+'uploaded')[0];
 		this._inspector._info_tab.upload_speed = $(ti+'upload_speed')[0];
 		this._inspector._info_tab.upload_to = $(ti+'upload_to')[0];
@@ -115,9 +122,11 @@ Transmission.prototype =
 		var tr = this;
 		var async = false;
 		this.loadDaemonPrefs( async );
+		this.loadDaemonStats( async );
 		this.initializeAllTorrents();
 
 		this.togglePeriodicRefresh( true );
+		this.togglePeriodicSessionRefresh( true );
 	},
 
 	loadDaemonPrefs: function( async ){
@@ -129,6 +138,14 @@ Transmission.prototype =
 		}, async );
 	},
 
+	loadDaemonStats: function( async ){
+		var tr = this;
+		this.remote.loadDaemonStats( function(data){
+			var o = data.arguments;
+			tr.updateStats( o );
+		}, async );
+	},
+
 	preloadImages: function() {
 		if (iPhone) {
 			this.loadImages(
@@ -289,8 +306,10 @@ Transmission.prototype =
 			boundingRightPad:  20,
 			boundingBottomPad: 5,
 			onContextMenu:     function(e) {
-				tr.setSelectedTorrent( $(e.target).closest('.torrent')[0]._torrent, true );
-				return true;
+                var closestRow = $(e.target).closest('.torrent')[0]._torrent; 
+                if(!closestRow.isSelected()) 
+                    tr.setSelectedTorrent( closestRow, true );
+                return true;
 			}
 		});
 	},
@@ -311,6 +330,23 @@ Transmission.prototype =
 	},
 
 
+	initTurtleDropDowns: function() {
+		var i, out, hour, mins, start, end, value, content;
+		// Build the list of times
+		out = "";
+		start = $('#turtle_start_time')[0];
+		end = $('#turtle_end_time')[0];
+		for (i = 0; i < 24 * 4; i++) {
+			hour = parseInt(i / 4);
+			mins = ((i % 4) * 15);
+
+			value = (i * 15);
+			content = hour + ":" + (mins == 0 ? "00" : mins);
+			start.options[i] = new Option(content, value);
+			end.options[i]  = new Option(content, value);
+		}
+	},
+
 	/*--------------------------------------------
 	 *
 	 *  U T I L I T I E S
@@ -415,11 +451,10 @@ Transmission.prototype =
 	},
 
 	selectElement: function( e, doUpdate ) {
-		$.className.add( e[0], 'selected' );
+		e.addClass('selected');
 		this.scrollToElement( e );
 		if( doUpdate )
 			this.selectionChanged( );
-		$.className.add( e[0], 'selected' );
 	},
 	selectRow: function( rowIndex, doUpdate ) {
 		this.selectElement( this._rows[rowIndex], doUpdate );
@@ -430,7 +465,7 @@ Transmission.prototype =
 	},
 
 	deselectElement: function( e, doUpdate ) {
-		$.className.remove( e[0], 'selected' );
+		e.removeClass('selected');
 		if( doUpdate )
 			this.selectionChanged( );
 	},
@@ -596,7 +631,6 @@ Transmission.prototype =
 	{
 		// handle the clutch prefs locally
 		var tr = this;
-		tr.setPref( Prefs._AutoStart, $('#prefs_form #auto_start')[0].checked );
 		var rate = parseInt( $('#prefs_form #refresh_rate')[0].value );
 		if( rate != tr[Prefs._RefreshRate] ) {
 			tr.setPref( Prefs._RefreshRate, rate );
@@ -606,20 +640,32 @@ Transmission.prototype =
 		
 		// pass the new prefs upstream to the RPC server
 		var o = { };
-		o[RPC._PeerPort]         = parseInt( $('#prefs_form #port')[0].value );
-		o[RPC._UpSpeedLimit]     = parseInt( $('#prefs_form #upload_rate')[0].value );
-		o[RPC._DownSpeedLimit]   = parseInt( $('#prefs_form #download_rate')[0].value );
-		o[RPC._DownloadDir]      = $('#prefs_form #download_location')[0].value;
-		o[RPC._UpSpeedLimited]   = $('#prefs_form #limit_upload')[0].checked ? 1 : 0;
-		o[RPC._DownSpeedLimited] = $('#prefs_form #limit_download')[0].checked ? 1 : 0;
-		o[RPC._Encryption]       = $('#prefs_form #encryption')[0].checked
-		                               ? RPC._EncryptionRequired
-		                               : RPC._EncryptionPreferred;
+		o[RPC._StartAddedTorrent]    = $('#prefs_form #auto_start')[0].checked;
+		o[RPC._PeerPort]             = parseInt( $('#prefs_form #port')[0].value );
+		o[RPC._UpSpeedLimit]         = parseInt( $('#prefs_form #upload_rate')[0].value );
+		o[RPC._DownSpeedLimit]       = parseInt( $('#prefs_form #download_rate')[0].value );
+		o[RPC._DownloadDir]          = $('#prefs_form #download_location')[0].value;
+		o[RPC._UpSpeedLimited]       = $('#prefs_form #limit_upload')[0].checked;
+		o[RPC._DownSpeedLimited]     = $('#prefs_form #limit_download')[0].checked;
+		o[RPC._Encryption]           = $('#prefs_form #encryption')[0].checked
+		                                   ? RPC._EncryptionRequired
+		                                   : RPC._EncryptionPreferred;
+		o[RPC._TurtleDownSpeedLimit] = parseInt( $('#prefs_form #turtle_download_rate')[0].value );
+		o[RPC._TurtleUpSpeedLimit]   = parseInt( $('#prefs_form #turtle_upload_rate')[0].value );
+		o[RPC._TurtleTimeEnabled]    = $('#prefs_form #turtle_schedule')[0].checked;
+		o[RPC._TurtleTimeBegin]      = parseInt( $('#prefs_form #turtle_start_time').val() );
+		o[RPC._TurtleTimeEnd]        = parseInt( $('#prefs_form #turtle_end_time').val() );
+		o[RPC._TurtleTimeDay]        = parseInt( $('#prefs_form #turtle_days').val() );
+
 		tr.remote.savePrefs( o );
 		
 		tr.hidePrefsDialog( );
 	},
 
+	closeStatsClicked: function(event) {
+		this.hideStatsDialog( );
+	},
+
 	removeClicked: function( event ) {	
 		var tr = this;
 		if( tr.isButtonEnabled( event ) ) {
@@ -654,6 +700,7 @@ Transmission.prototype =
 		this.hideiPhoneAddressbar();
 	
 		this.updateVisibleFileLists();
+		this.updateTrackersLists();
 	},
 
 	fileWantedClicked: function(event, element){
@@ -664,6 +711,26 @@ Transmission.prototype =
 		this.extractFileFromElement(element).filePriorityControlClicked(event, element);
 	},
 
+	filesSelectAllClicked: function(event) {
+		var tr = this;
+		var ids = jQuery.map(this.getSelectedTorrents( ), function(t) { return t.id(); } );
+		var files_list = this.toggleFilesWantedDisplay(ids, true);
+		for (i = 0; i < ids.length; ++i) {
+			if (files_list[i].length)
+				this.remote.filesSelectAll( [ ids[i] ], files_list[i], function() { tr.refreshTorrents( ids ); } );
+		}
+	},
+
+	filesDeselectAllClicked: function(event) {
+		var tr = this;
+		var ids = jQuery.map(this.getSelectedTorrents( ), function(t) { return t.id(); } );
+		var files_list = this.toggleFilesWantedDisplay(ids, false);
+		for (i = 0; i < ids.length; ++i) {
+			if (files_list[i].length)
+				this.remote.filesDeselectAll( [ ids[i] ], files_list[i], function() { tr.refreshTorrents( ids ); } );
+		}
+	},
+
 	extractFileFromElement: function(element) {
 		var match = $(element).closest('.inspector_torrent_file_list_entry').attr('id').match(/^t(\d+)f(\d+)$/);
 		var torrent_id = match[1];
@@ -672,6 +739,22 @@ Transmission.prototype =
 		return torrent._file_view[file_id];
 	},
 
+	toggleFilesWantedDisplay: function(ids, wanted) {
+		var i, j, k, torrent, files_list = [ ];
+		for (i = 0; i < ids.length; ++i) {
+			torrent = this._torrents[ids[i]];
+			files_list[i] = [ ];
+			for (j = k = 0; j < torrent._file_view.length; ++j) {
+				if (torrent._file_view[j].isEditable() && torrent._file_view[j]._wanted != wanted) {
+					torrent._file_view[j].setWanted(wanted, false);
+					files_list[i][k++] = j;
+				}
+			}
+			torrent.refreshFileView;
+		}
+		return files_list;
+	},
+
 	toggleFilterClicked: function(event) {
 		if (this.isButtonEnabled(event))
 			this.toggleFilter();
@@ -716,7 +799,7 @@ Transmission.prototype =
 	},
 
 	/*
-	 * Turn the periodic ajax-refresh on & off
+	 * Turn the periodic ajax torrents refresh on & off
 	 */
 	togglePeriodicRefresh: function(state) {
 		var tr = this;
@@ -732,6 +815,70 @@ Transmission.prototype =
 		}
 	},
 
+	/*
+	 * Turn the periodic ajax session refresh on & off
+	 */
+	togglePeriodicSessionRefresh: function(state) {
+		var tr = this;
+		if (state && this._periodic_session_refresh == null) {
+			// sanity check
+			if( !this[Prefs._SessionRefreshRate] )
+			     this[Prefs._SessionRefreshRate] = 5;
+			remote = this.remote;
+			this._periodic_session_refresh = setInterval(
+				function(){ tr.loadDaemonPrefs(); }, this[Prefs._SessionRefreshRate] * 1000
+			);
+		} else {
+			clearInterval(this._periodic_session_refresh);
+			this._periodic_session_refresh = null;
+		}
+	},
+
+	/*
+	 * Turn the periodic ajax stats refresh on & off
+	 */
+	togglePeriodicStatsRefresh: function(state) {
+		var tr = this;
+		if (state && this._periodic_stats_refresh == null) {
+			// sanity check
+			if( !this[Prefs._SessionRefreshRate] )
+			     this[Prefs._SessionRefreshRate] = 5;
+			remote = this.remote;
+			this._periodic_stats_refresh = setInterval(
+				function(){ tr.loadDaemonStats(); }, this[Prefs._SessionRefreshRate] * 1000
+			);
+		} else {
+			clearInterval(this._periodic_stats_refresh);
+			this._periodic_stats_refresh = null;
+		}
+	},
+
+	toggleTurtleClicked: function() {
+		// Toggle the value
+		this[Prefs._TurtleState] = !this[Prefs._TurtleState];
+		// Store the result
+		var args = { };
+		args[RPC._TurtleState] = this[Prefs._TurtleState];
+		this.remote.savePrefs( args );
+	},
+
+	updateTurtleButton: function() {
+		var w = $('#turtle_button');
+		var t;
+		if ( this[Prefs._TurtleState] ) {
+			w.addClass('turtleEnabled');
+			w.removeClass('turtleDisabled');
+			t = "Click to disable Temporary Speed Limits";
+		} else {
+			w.removeClass('turtleEnabled');
+			w.addClass('turtleDisabled');
+			t = "Click to enable Temporary Speed Limits";
+		}
+		t += " (" + this._prefs[RPC._TurtleUpSpeedLimit] + " kB/s up, "
+		          + this._prefs[RPC._TurtleDownSpeedLimit] + " kB/s down)";
+		w.attr( 'title', t );
+	},
+
 	/*--------------------------------------------
 	 *
 	 *  I N T E R F A C E   F U N C T I O N S
@@ -745,6 +892,7 @@ Transmission.prototype =
 		if( Safari3 )
 			setTimeout("$('div#prefs_container div.dialog_window').css('top', '0px');",10);
 		this.updateButtonStates( );
+		this.togglePeriodicSessionRefresh(false);
 	},
 
 	hidePrefsDialog: function( )
@@ -760,6 +908,7 @@ Transmission.prototype =
 			$('#prefs_container').hide();
 		}
 		this.updateButtonStates( );
+		this.togglePeriodicSessionRefresh(true);
 	},
 
 	/*
@@ -774,29 +923,87 @@ Transmission.prototype =
 		var down_limited  = prefs[RPC._DownSpeedLimited];
 		var up_limit      = prefs[RPC._UpSpeedLimit];
 		var up_limited    = prefs[RPC._UpSpeedLimited];
-		
+
 		$('div.download_location input')[0].value = prefs[RPC._DownloadDir];
 		$('div.port input')[0].value              = prefs[RPC._PeerPort];
-		$('div.auto_start input')[0].checked      = prefs[Prefs._AutoStart];
-		$('input#limit_download')[0].checked      = down_limited == 1;
+		$('div.auto_start input')[0].checked      = prefs[RPC._StartAddedTorrent];
+		$('input#limit_download')[0].checked      = down_limited;
 		$('input#download_rate')[0].value         = down_limit;
-		$('input#limit_upload')[0].checked        = up_limited == 1;
+		$('input#limit_upload')[0].checked        = up_limited;
 		$('input#upload_rate')[0].value           = up_limit;
 		$('input#refresh_rate')[0].value          = prefs[Prefs._RefreshRate];
 		$('div.encryption input')[0].checked      = prefs[RPC._Encryption] == RPC._EncryptionRequired;
+		$('input#turtle_download_rate')[0].value  = prefs[RPC._TurtleDownSpeedLimit];
+		$('input#turtle_upload_rate')[0].value    = prefs[RPC._TurtleUpSpeedLimit];
+		$('input#turtle_schedule')[0].checked     = prefs[RPC._TurtleTimeEnabled];
+		$('select#turtle_start_time').val(          prefs[RPC._TurtleTimeBegin] );
+		$('select#turtle_end_time').val(            prefs[RPC._TurtleTimeEnd] );
+		$('select#turtle_days').val(                prefs[RPC._TurtleTimeDay] );
+		$('#transmission_version').text(            prefs[RPC._DaemonVersion] );
 
 		if (!iPhone)
 		{
-			setInnerHTML( $('#limited_download_rate')[0], 'Limit (' + down_limit + ' KB/s)' );
+			setInnerHTML( $('#limited_download_rate')[0], 'Limit (' + down_limit + ' KiB/s)' );
 			var key = down_limited ? '#limited_download_rate'
 			                       : '#unlimited_download_rate';
 			$(key).deselectMenuSiblings().selectMenuItem();
 		
-			setInnerHTML( $('#limited_upload_rate')[0], 'Limit (' + up_limit + ' KB/s)' );
+			setInnerHTML( $('#limited_upload_rate')[0], 'Limit (' + up_limit + ' KiB/s)' );
 			key = up_limited ? '#limited_upload_rate'
 			                 : '#unlimited_upload_rate';
 			$(key).deselectMenuSiblings().selectMenuItem();
 		}
+
+		this[Prefs._TurtleState] = prefs[RPC._TurtleState];
+		this.updateTurtleButton();
+	},
+
+	showStatsDialog: function( ) {
+		this.loadDaemonStats();
+		$('body').addClass('stats_showing');
+		$('#stats_container').show();
+		this.hideiPhoneAddressbar();
+		if( Safari3 )
+			setTimeout("$('div#stats_container div.dialog_window').css('top', '0px');",10);
+		this.updateButtonStates( );
+		this.togglePeriodicStatsRefresh(true);
+	},
+
+	hideStatsDialog: function( ){
+		$('body.stats_showing').removeClass('stats_showing');
+		if (iPhone) {
+			this.hideiPhoneAddressbar();
+			$('#stats_container').hide();
+		} else if (Safari3) {
+			$('div#stats_container div.dialog_window').css('top', '-425px');
+			setTimeout("$('#stats_container').hide();",500);
+		} else {
+			$('#stats_container').hide();
+		}
+		this.updateButtonStates( );
+		this.togglePeriodicStatsRefresh(false);
+	},
+
+	/*
+	 * Process got some new session stats from the server
+	 */
+	updateStats: function( stats )
+	{
+		// can't think of a reason to remember this
+		//this._stats = stats;
+
+		var session = stats["current-stats"];
+		var total = stats["cumulative-stats"];
+
+		setInnerHTML( $('#stats_session_uploaded')[0], Math.formatBytes(session["uploadedBytes"]) );
+		setInnerHTML( $('#stats_session_downloaded')[0], Math.formatBytes(session["downloadedBytes"]) );
+		setInnerHTML( $('#stats_session_ratio')[0], Math.ratio(session["uploadedBytes"],session["downloadedBytes"]));
+		setInnerHTML( $('#stats_session_duration')[0], Math.formatSeconds(session["secondsActive"]) );
+		setInnerHTML( $('#stats_total_count')[0], total["sessionCount"] + " times" );
+		setInnerHTML( $('#stats_total_uploaded')[0], Math.formatBytes(total["uploadedBytes"]) );
+		setInnerHTML( $('#stats_total_downloaded')[0], Math.formatBytes(total["downloadedBytes"]) );
+		setInnerHTML( $('#stats_total_ratio')[0], Math.ratio(total["uploadedBytes"],total["downloadedBytes"]));
+		setInnerHTML( $('#stats_total_duration')[0], Math.formatSeconds(total["secondsActive"]) );
 	},
 
 	setSearch: function( search ) {
@@ -831,6 +1038,11 @@ Transmission.prototype =
 					$('div#prefs_container h2.dialog_heading').show();
 					tr.showPrefsDialog( );
 				}
+				else if ($element[0].id == 'statistics') {
+					$('div#stats_container div#stats_error').hide();
+					$('div#stats_container h2.dialog_heading').show();
+					tr.showStatsDialog( );
+				}
 				break;
 			
 			// Limit the download rate
@@ -841,7 +1053,7 @@ Transmission.prototype =
 					$element.deselectMenuSiblings().selectMenuItem();
 					args[RPC._DownSpeedLimited] = false;
 				} else {
-					setInnerHTML( $('#limited_download_rate')[0], 'Limit (' + rate + ' KB/s)' );
+					setInnerHTML( $('#limited_download_rate')[0], 'Limit (' + rate + ' KiB/s)' );
 					$('#limited_download_rate').deselectMenuSiblings().selectMenuItem();
 					$('div.preference input#download_rate')[0].value = rate;
 					args[RPC._DownSpeedLimit] = parseInt( rate );
@@ -859,7 +1071,7 @@ Transmission.prototype =
 					$element.deselectMenuSiblings().selectMenuItem();
 					args[RPC._UpSpeedLimited] = false;
 				} else {
-					setInnerHTML( $('#limited_upload_rate')[0], 'Limit (' + rate + ' KB/s)' );
+					setInnerHTML( $('#limited_upload_rate')[0], 'Limit (' + rate + ' KiB/s)' );
 					$('#limited_upload_rate').deselectMenuSiblings().selectMenuItem();
 					$('div.preference input#upload_rate')[0].value = rate;
 					args[RPC._UpSpeedLimit] = parseInt( rate );
@@ -874,6 +1086,7 @@ Transmission.prototype =
 
 				// The 'reverse sort' option state can be toggled independently of the other options
 				if ($element.is('#reverse_sort_order')) {
+                                        if(!$element.is('#reverse_sort_order.active')) break;
 					var dir;
 					if ($element.menuItemIsSelected()) {
 						$element.deselectMenuItem();
@@ -904,6 +1117,12 @@ Transmission.prototype =
 	setLastTorrentClicked: function( torrent )
 	{
 		this._last_torrent_clicked = torrent;
+				}
+				else if ($element[0].id == 'homepage') {
+					window.open('http://www.transmissionbt.com/');
+				}
+				else if ($element[0].id == 'tipjar') {
+					window.open('http://www.transmissionbt.com/donate.php');
 	},
 
 	/*
@@ -935,13 +1154,11 @@ Transmission.prototype =
 		var total_download = 0;
 		var total_download_peers = 0;
 		var total_download_speed = 0;
+		var total_availability = 0;
 		var total_have = 0;
-		var total_leechers = 0;
 		var total_size = 0;
-		var total_seeders = 0;
 		var total_state = null;
-		var total_swarm_speed = 0;
-		var total_tracker = null;
+		var pieces = 'N/A';
 		var total_upload = 0;
 		var total_upload_peers = 0;
 		var total_upload_speed = 0;
@@ -955,17 +1172,15 @@ Transmission.prototype =
 		{
 			setInnerHTML( tab.name, 'No Selection' );
 			setInnerHTML( tab.size, na );
-			setInnerHTML( tab.tracker, na );
+			setInnerHTML( tab.pieces, na );
 			setInnerHTML( tab.hash, na );
 			setInnerHTML( tab.state, na );
 			setInnerHTML( tab.download_speed, na );
 			setInnerHTML( tab.upload_speed, na );
 			setInnerHTML( tab.uploaded, na );
 			setInnerHTML( tab.downloaded, na );
+			setInnerHTML( tab.availability, na );
 			setInnerHTML( tab.ratio, na );
-			setInnerHTML( tab.total_seeders, na );
-			setInnerHTML( tab.total_leechers, na );
-			setInnerHTML( tab.swarm_speed, na );
 			setInnerHTML( tab.have, na );
 			setInnerHTML( tab.upload_to, na );
 			setInnerHTML( tab.download_from, na );
@@ -974,8 +1189,10 @@ Transmission.prototype =
 			setInnerHTML( tab.progress, na );
 			setInnerHTML( tab.comment, na );
 			setInnerHTML( tab.creator, na );
+			setInnerHTML( tab.download_dir, na );
 			setInnerHTML( tab.error, na );
 			this.updateVisibleFileLists();
+			this.updateTrackersLists();
 			$("#torrent_inspector_size, .inspector_row > div:contains('N/A')").css('color', '#666');
 			return;
 		}
@@ -998,6 +1215,7 @@ Transmission.prototype =
 				download_dir = t._download_dir;
 
 			hash = t.hash();
+			pieces = t._pieceCount + ', ' + Math.formatBytes(t._pieceSize);
 			date_created = Math.formatTimestamp( t._creator_date );
 		}
 
@@ -1011,20 +1229,13 @@ Transmission.prototype =
 			total_download       += t.downloadTotal();
 			total_upload_speed   += t.uploadSpeed();
 			total_download_speed += t.downloadSpeed();
-			total_seeders        += t.totalSeeders();
-			total_leechers       += t.totalLeechers();
 			total_upload_peers   += t.peersGettingFromUs();
 			total_download_peers += t.peersSendingToUs();
-			total_swarm_speed    += t.swarmSpeed();
+			total_availability   += t._sizeWhenDone - t._leftUntilDone + t._desiredAvailable;
 			if( total_state == null )
 				total_state = t.stateStr();
 			else if ( total_state.search ( t.stateStr() ) == -1 )
 				total_state += '/' + t.stateStr();
-			var tracker = t._tracker;
-			if( total_tracker == null )
-				total_tracker = tracker;
-			else if ( total_tracker.search ( tracker ) == -1 )
-				total_tracker += ', ' + tracker;
 			if( t._is_private )
 				have_private = true;
 			else
@@ -1038,28 +1249,27 @@ Transmission.prototype =
 
 		setInnerHTML( tab.name, name );
 		setInnerHTML( tab.size, torrents.length ? Math.formatBytes( total_size ) : na );
-		setInnerHTML( tab.tracker, total_tracker.replace(/\//g, '/&#8203;') );
+		setInnerHTML( tab.pieces, pieces );
 		setInnerHTML( tab.hash, hash );
 		setInnerHTML( tab.state, total_state );
 		setInnerHTML( tab.download_speed, torrents.length ? Math.formatBytes( total_download_speed ) + '/s' : na );
 		setInnerHTML( tab.upload_speed, torrents.length ? Math.formatBytes( total_upload_speed ) + '/s' : na );
 		setInnerHTML( tab.uploaded, torrents.length ? Math.formatBytes( total_upload ) : na );
 		setInnerHTML( tab.downloaded, torrents.length ? Math.formatBytes( total_download ) : na );
+		setInnerHTML( tab.availability, torrents.length ? Math.ratio( total_availability*100, sizeWhenDone ) + '%' : na );
 		setInnerHTML( tab.ratio, torrents.length ? Math.ratio( total_upload, total_download ) : na );
-		setInnerHTML( tab.total_seeders, torrents.length ? total_seeders : na );
-		setInnerHTML( tab.total_leechers, torrents.length ? total_leechers : na );
-		setInnerHTML( tab.swarm_speed, torrents.length ? Math.formatBytes(total_swarm_speed) + '/s' : na );
 		setInnerHTML( tab.have, torrents.length ? Math.formatBytes(total_completed) + ' (' + Math.formatBytes(total_verified) + ' verified)' : na );
 		setInnerHTML( tab.upload_to, torrents.length ? total_upload_peers : na );
 		setInnerHTML( tab.download_from, torrents.length ? total_download_peers : na );
 		setInnerHTML( tab.secure, private_string );
 		setInnerHTML( tab.creator_date, date_created );
 		setInnerHTML( tab.progress, torrents.length ? Math.ratio( sizeDone*100, sizeWhenDone ) + '%' : na );
-		setInnerHTML( tab.comment, comment.replace(/\//g, '/&#8203;') );
+		setInnerHTML( tab.comment, comment == na ? comment : comment.replace(/\//g, '/&#8203;') );
 		setInnerHTML( tab.creator, creator );
-		setInnerHTML( tab.download_dir, download_dir.replace(/([\/_\.])/g, "$1&#8203;") );
+		setInnerHTML( tab.download_dir, download_dir == na ? download_dir : download_dir.replace(/([\/_\.])/g, "$1&#8203;") );
 		setInnerHTML( tab.error, error );
 		
+		this.updateTrackersLists();
 		$(".inspector_row > div:contains('N/A')").css('color', '#666');
 		this.updateVisibleFileLists();
 	},
@@ -1070,11 +1280,116 @@ Transmission.prototype =
 	
 	updateVisibleFileLists: function() {
 		if( this.fileListIsVisible( ) === true ) {
-			jQuery.each( this.getSelectedTorrents(), function() { this.showFileList(); } );
+			var selected = this.getSelectedTorrents();
+			jQuery.each( selected, function() { this.showFileList(); } );
 			jQuery.each( this.getDeselectedTorrents(), function() { this.hideFileList(); } );
+			// Check if we need to display the select all buttions
+			if ( !selected.length ) {
+				if ( $("#select_all_button_container").is(':visible') )
+					$("#select_all_button_container").hide();
+			} else {
+				if ( !$("#select_all_button_container").is(':visible') )
+					$("#select_all_button_container").show();
+			}
 		}
 	},
 
+	updateTrackersLists: function() {
+		// By building up the HTML as as string, then have the browser
+		// turn this into a DOM tree, this is a fast operation.
+		var tr = this;
+		var html = '';
+		var na = 'N/A';
+		var torrents = this.getSelectedTorrents( );
+		if( $(this._inspector_trackers_list).is(':visible') && torrents.length == 1 ) {
+			for( var i=0, tier; tier=torrents[0]._trackerStats[i]; ++i ) {
+				html += '<div class="inspector_group"><div class="inspector_group_label">';
+				html += 'Tier ' + (i + 1) + '</div><ul class="tier_list">';
+				for( var j=0, tracker; tracker=tier[j]; ++j ) {
+					var lastAnnounceStatusHash = tr.lastAnnounceStatus(tracker);
+					var announceState = tr.announceState(tracker);
+					var lastScrapeStatusHash = tr.lastScrapeStatus(tracker);
+
+					// Display construction
+					var parity = ((j+1) % 2 == 0 ? 'even' : 'odd');
+					html += '<li class="inspector_tracker_entry ' + parity + '"><div class="tracker_host" title="' + tracker.announce + '">';
+					html += tracker.host + '</div>';
+					html += '<div class="tracker_activity">';
+					html += '<div>' + lastAnnounceStatusHash['label'] + ': ' + lastAnnounceStatusHash['value'] + '</div>';
+					html += '<div>' + announceState + '</div>';
+					html += '<div>' + lastScrapeStatusHash['label'] + ': ' + lastScrapeStatusHash['value'] + '</div>';
+					html += '</div><table class="tracker_stats">';
+					html += '<tr><th>Seeders:</th><td>' + (tracker.seederCount > -1 ? tracker.seederCount : na) + '</td></tr>';
+					html += '<tr><th>Leechers:</th><td>' + (tracker.leecherCount > -1 ? tracker.leecherCount : na) + '</td></tr>';
+					html += '<tr><th>Downloads:</th><td>' + (tracker.downloadCount > -1 ? tracker.downloadCount : na)+ '</td></tr>';
+					html += '</table></li>';
+				}
+				html += '</ul></div>';
+			}
+		}
+		setInnerHTML(this._inspector_trackers_list, html);
+	},
+
+	lastAnnounceStatus: function(tracker){
+		var lastAnnounceLabel = 'Last Announce';
+		var lastAnnounce = 'N/A';
+		if (tracker.hasAnnounced) {
+			var lastAnnounceTime = Math.formatTimestamp(tracker.lastAnnounceTime);
+			if (tracker.lastAnnounceSucceeded) {
+				lastAnnounce = lastAnnounceTime;
+				lastAnnounce += ' (got ' + tracker.lastAnnouncePeerCount + ' peer';
+				if (tracker.lastAnnouncePeerCount != 1){ lastAnnounce += 's'; }
+				lastAnnounce += ')';
+			} else {
+				lastAnnounceLabel = 'Announce error';
+				lastAnnounce = (tracker.lastAnnounceResult ? tracker.lastAnnounceResult + ' - ' : '') + lastAnnounceTime;
+			}
+		}
+		return {'label':lastAnnounceLabel, 'value':lastAnnounce};
+	},
+
+	announceState: function(tracker){
+		var announceState = '';
+		switch (tracker.announceState) {
+			case Torrent._TrackerActive:
+				announceState = 'Announce in progress';
+				break;
+			case Torrent._TrackerWaiting:
+				var timeUntilAnnounce = tracker.nextAnnounceTime - ((new Date()).getTime() / 1000);
+				if(timeUntilAnnounce < 0){
+					timeUntilAnnounce = 0;
+				}
+				announceState = 'Next announce in ' + Math.formatSeconds(timeUntilAnnounce);
+				break;
+			case Torrent._TrackerQueued:
+				announceState = 'Announce is queued';
+				break;
+			case Torrent._TrackerInactive:
+				announceState = tracker.isBackup ?
+					'Tracker will be used as a backup' :
+					'Announce not scheduled';
+				break;
+			default:
+				announceState = 'unknown announce state: ' + tracker.announceState;
+		}
+		return announceState;
+	},
+
+	lastScrapeStatus: function(tracker){
+		var lastScrapeLabel = 'Last Scrape';
+		var lastScrape = 'N/A';
+		if (tracker.hasScraped) {
+			var lastScrapeTime = Math.formatTimestamp(tracker.lastScrapeTime);
+			if (tracker.lastScrapeSucceeded) {
+				lastScrape = lastScrapeTime;
+			} else {
+				lastScrapeLabel = 'Scrape error';
+				lastScrape = (tracker.lastScrapeResult ? tracker.lastScrapeResult + ' - ' : '') + lastScrapeTime;
+			}
+		}
+		return {'label':lastScrapeLabel, 'value':lastScrape}
+	},
+
 	/*
 	 * Toggle the visibility of the inspector (used by the context menu)
 	 */
@@ -1150,6 +1465,27 @@ Transmission.prototype =
 		this.setFilter( Prefs._FilterAll );
 	},
 
+	refreshMetaData: function(ids) {
+		var tr = this;
+		this.remote.getMetaDataFor(ids, function(active, removed){ tr.updateMetaData(active); });
+	},
+
+	updateMetaData: function( torrents )
+	{
+		var tr = this;
+		var refresh_files_for = [ ];
+		jQuery.each( torrents, function( ) {
+			var t = tr._torrents[ this.id ];
+			if( t ) {
+				t.refreshMetaData( this );
+				if( t.isSelected( ) )
+					refresh_files_for.push( t.id( ) );
+			}
+		} );
+		if( refresh_files_for.length > 0 )
+			tr.remote.loadTorrentFiles( refresh_files_for );
+	},
+
 	refreshTorrents: function(ids) {
 		var tr = this;
 		if (!ids)
@@ -1235,7 +1571,15 @@ Transmission.prototype =
 				removedAny = true;
 				var e = torrent.element();
 				if( e ) {
-					var row_index = tr.getTorrentIndex(tr._rows, torrent);
+					var row_index;
+					for( var i=0, row; row = tr._rows[i]; ++i ) {
+						if( row._id == torrent._id )
+						{
+							row_index = i;
+							e = tr._rows[row_index];
+							break;
+						}
+					}
 					delete e._torrent; //remove circular refernce to help IE garbage collect
 					tr._rows.splice(row_index, 1)
 					e.remove();
@@ -1307,6 +1651,7 @@ Transmission.prototype =
 		if (! confirmed) {
 				$('input#torrent_upload_file').attr('value', '');
 				$('input#torrent_upload_url').attr('value', '');
+				$('input#torrent_auto_start').attr('checked', $('#prefs_form #auto_start')[0].checked);
 				$('#upload_container').show();
 			if (!iPhone && Safari3) {
 				setTimeout("$('div#upload_container div.dialog_window').css('top', '0px');",10);
@@ -1316,10 +1661,11 @@ Transmission.prototype =
 		} else {
 			var tr = this;
 			var args = { };
+			var paused = !$('#torrent_auto_start').is(':checked');
 			if ('' != $('#torrent_upload_url').val()) {
-				tr.remote.addTorrentByUrl($('#torrent_upload_url').val(), { paused: !this[Prefs._AutoStart] });
+				tr.remote.addTorrentByUrl($('#torrent_upload_url').val(), { paused: paused });
 			} else {
-				args.url = '/transmission/upload?paused=' + (this[Prefs._AutoStart] ? 'false' : 'true');
+				args.url = '/transmission/upload?paused=' + paused;
 				args.type = 'POST';
 				args.data = { 'X-Transmission-Session-Id' : tr.remote._token };
 				args.dataType = 'xml';
